function fitWHITE(X,y)
## {b,sb,s2}:= fitWHITE(X,y) returns the White's consistent heteroskedastic
##             estimation of .

n:=rows(X); k:=cols(X);
G:=(X'.X)\id(k);
b:=G.(X'.y);
e:=y-X.b;
s2:=e'.e/(n-k);
Z:=X.*abs(e); V:=Z.G;
sb:=sqrt(diag(V'.V,0)');
return {b,sb,s2};
endfunction;


function fitRWHITE(X,y,R,r)
## {b,sb,s2}:= fitRWHITE(X,y) returns a feasible restricted White-type 
##             heteroskedastic estimation of .

n:=rows(X); k:=cols(X); q:=rows(R);
d:=fitRLS(X,y,R,r);
e:=y-X.d;
s2:=e'.e/(n-k);

G:=(X'.X)\id(k);
Z:=X.*abs(e);
V:=Z.G; W:=(Z'.Z)\id(k);

C:=(X'.X).W.X';
A:=((C.X)_R)|(R'_zeros(q,q)); x:=(C.y)_r;
b:=A\x;
b:=b[1:k];

S:=V'.V; T:=R.S;
sb:=sqrt(diag(S-(T'.inv(R.S.R').T),0)');
return {b,sb,s2};
endfunction;

function fitRANK(X,y)
## fitRANK(X,y) returns the forcasted y and the estimator b after the
##              rank-based transformation of X and y.
n:=rows(X);
k:=cols(X);
Z:=y|X;
for j=1 to k+1,
    Z[:,j]:=ranking(Z[:,j]')';
end;
b:=fit(Z[:,2:k+1],Z[:,1]);
r:=Z[:,2:k+1].b;

v:=sort(y); 
w:=sort(Z[:,1]);
yest:=[];
for i=1 to n,
    a:=nonzeros(v==y[i])+((y[i]!=v[1])*(-1)|(y[i]!=v[n]));    
    yest:=yest_(v[a[1]]+((r[i]-Z[i,1])/(w[a[2]]-w[a[1]]))*(v[a[2]]-v[a[1]]));
end;
return {b,yest};
endfunction;


function ranking(v)
## This is an auxiliary function that returns the ranks of a row vector v.
## In case of ties, ranking(.) returns the mean rank.

w:=sort(v);
h:=[];
for j=1 to cols(v), 
    m:=nonzeros(w==v[j]); 
    h:=h|(m[-1]-m[1]+1)*(m[-1]+m[1])/(2*cols(m));
end;
return h;
endfunction;

function corrcoef(X)
## corrcoef(X) returns the correlation coefficient matrix of matrix X.
##             As a rule of thumb, if at least one R[i,j]>0.9 there is 
##             evidence of linear association between the columns of X.
##             Remember to omit the columns of 1 when calling X.
##
## May 5th., 2011.

k:=cols(X); 
R:=zeros(k,k); 
m:=mean(X')'; 

for j=1 to k, 
    X[:,j]:=X[:,j]-m[j]; 
end; 

for j=1 to k, 
    for i=1 to k,
        R[i,j]:=(X[:,i]'.X[:,j])/sqrt((X[:,i]'.X[:,i])*(X[:,j]'.X[:,j]));
        R[j,i]:=R[i,j]; 
    end; 
end;
return R;
endfunction;


function partialcorr(X,y)
## partialcorr(X,y) returns the partial correlation coefficient between y and each
##                  X[:,j]. High R2 but low partial correlations reveals evidence of
##                  multicollinearity.
##
## May 3rd., 2011.

k:=cols(X);
A:=[];
for j=2 to k,
    Z:=X[:,nonzeros(1:k!=j)];
    v:=fit(Z,y);
    w:=fit(Z,X[:,j]);    
    e1:=y-Z.v;
    e2:=X[:,j]-Z.w;
    rho:=(e1'.e2)/sqrt((e1'.e1)*(e2'.e2));
    A:=A_(j|rho);
end;
return A;
endfunction;


function vif(Z)
## vif(X). Computes the R2 of tha auxilliary regression xj = X(j) and the
##         associated Variance Inflation Factor. The function returns the 
##         a matrix j|R2|VIF. As a rule of thumb, multicollinearity is serious
##         when VIF>10.
##
## May 3rd., 2011.

vif:=[];
for j=2 to cols(Z),
    y:=Z[:,j];    
    X:=Z[:,nonzeros(1:cols(Z)!=j)];
    b:=fit(X,y); 
    e:=y-X.b;

    y:=y-mean(y');
    R2:=1-(e'.e)/(y'.y);
    vif:=vif_(j|R2|(1/(1-R2)));
end;
return vif;
endfunction;


function FGtest(X)
## FGtest(X) returns the Farrar Glauber F statistic and its p-value to
##           test for linear dependency between xj and the X(j). Under the
##           null =0 in the model xj = X(j) + v.
##
## May 24th., 2011.

n:=rows(X);
k:=cols(X);
F:=[];

c:=(n-(k-1))/(k-2);
A:=vif(X);
for h=1 to k-1
    f:=(A[h,2]/A[h,3])*c;
    F:=F_(A[h,1]|f|(1-fdis(f,k-1,n-k+1)));
end;
return F;
endfunction;


function cond(X)
## cond(X). Returns the square root of the ratio between the maximun and 
##          minimun eigenvalues of X'X. According to Judge et al. (1985)
##          5<=cond(X)<=10 reveals low to moderate multicolinearity, whereas
##          30<=cond(X)<=100 reveals moderate to high linear dependance.
##
## May 5th., 2011.

Z:=[]; 
for j=1 to cols(X),
    Z:=Z|(X[:,j]/sqrt(X[:,j]'.X[:,j]));
end;
{d,P}:=eigen(Z'.Z);
d:=real(d);
return sqrt(max(d)/min(d));	
endfunction;


function numerate(v)
## This is an auxiliari function for rankcond(X). It returns the ranks
##      of a row vector v. If ties appear, numerate(.) returns the delta 
##      that should be substracted to the rank-SC.

w:=sort(v);
h:=[]; d:=[]; 
for j=1 to cols(v), 
    m:=nonzeros(w==v[j]); 
    h:=h|(m[-1]-m[1]+1)*(m[-1]+m[1])/(2*cols(m));
    d:=d|((m[-1]-m[1])*(m[-1]-m[1]+2)/12);
end;
delta:=sum(d);
return {h,delta};
endfunction;


function rankcond(X)
## rankcond(X) returns the condition number of a rank-transformed matrix X,
##             transformed in such a way that the diagonal elements of X'X 
##             are equal to 1.

n:=rows(X);
k:=cols(X);
m:=[1:n].[1:n]';
Z:=[];
for j=1 to k,
    {h,delta}:=numerate(X[:,j]');
    Z:=Z|(h'/sqrt(m-delta));
end;
{d,P}:=eigen(Z'.Z);
d:=real(d);
return sqrt(max(d)/min(d));
endfunction;


function bartlett(X)
## bartlett(X) returns Bartletts's T statistic and its p-value to test for 
##             collinearity among the columns of X. Under the null T is 
##             distributed chi-square with k*(k-1)/2 degrees of freedom.
##             Recall that Ho assumes that X is a normal multivariate sample with
##             correlation matrix equal to id(k).
##
## May 24th., 2011.

n:=rows(X);
k:=cols(X);

Z:=corrcoef(X);
T:=-(n-1-(2*k+5)/6)*ln(det(Z));
pvalue:=1-chidis(T,k*(k-1)/2);
return {T,pvalue};
endfunction;

function GQtest(X,y,j)
## GQtest(X,y,j) returns the p-value of the Golfeld-Quandt test for the j-th column.
##               The c omitted medium values are c:=max(4,floor(n/10).

n:=rows(X);
k:=cols(X);
c:=max(4,floor(n/10));
{w,i}:=sort(X[:,j]);

v:=i[1:round((n-c)/2,0)];
w:=i[round((n-c)/2,0)+c+1:n];

y1:=y[v]; X1:=X[v,:]; b1:=fit(X1,y1);
y2:=y[w]; X2:=X[w,:]; b2:=fit(X2,y2);

e1:=y1-X1.b1; e2:=y2-X2.b2;
df:=round((n-c-2*k)/2,0);

F:=(e1'.e1/df)/(e2'.e2/df); 
pval:=1-fdis(F,df,df);

return {pval,F,df};
endfunction;


function whitetest(X,y)
## whitetest(X,y) returns the p-value of White's statistic nR^2 to test for 
##            heteroskedasticity.
##
## May 15th., 2010.

n:=rows(X);
k:=cols(X);
Z:=X;
for i=2 to k, 
    for j=i to k, 
        Z:=Z|(X[:,i].*X[:,j]);
    end; 
end;
b:=fit(X,y);
e:=y-X.b;

z:=e.*e;
c:=fit(Z,z);
v:=z-Z.c;
w:=z-mean(z');
R2:=1-(v'.v)/(w'.w);
pvalue:=1-chidis(n*R2,cols(Z)-1);
return {pvalue,R2,n};
endfunction;


function ARCHtest(X,y,p)
## ARCHtest(X,y,p). returns the p-value of Engle's test for AutoRegressive Conditional
##                  Heteroskedasticity (ARCH). 
n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

E:=xlag(e,p); Z:=ones(n-p,1)|(E.*E); e2:=e[p+1:n].*e[p+1:n];
a:=fit(Z,e2); u:=e2-Z.a; w:=e2-mean(e2');

R2:=1-(u'.u)/(w'.w);
pvalue:=1-chidis(n*R2,p);
return {pvalue,R2};
endfunction;


function xlag(x,p)
X:=[]; 
for i=p+1 to length(x), 
    X:=X_x[i-1:-1:i-p]'; 
end; 
return X;
endfunction;

function fitCO(X,y,p)
n:=rows(X);
k:=cols(X);

b:=fit(X,y);
e:=y-X.b;
E:=xlag(e,p);
rho:=fit(E,e[p+1:n]);

for m=1 to 10
    r:=rho;
    Z:=y[p+1:n]|X[p+1:n,1:k];
    for h=1 to p,
        Z:=Z-(y[p-h+1:n-h,1]|X[p-h+1:n-h,1:k])*rho[h];
    end;
    b:=fit(Z[:,2:k+1],Z[:,1]);
    e:=y-X.b;
    E:=xlag(e,p);
    rho:=fit(E,e[p+1:n]);
    if max(abs(rho-r))<0.001,
       break,
    endif;
end;
return {b,rho};
endfunction;

function xlag(x,p)
X:=[]; 
for i=p+1 to length(x), 
    X:=X_x[i-1:-1:i-p]'; 
end; 
return X;
endfunction;

function fitLAD(X,y)
## fitLAD(X,y) Returns the Least Absolute Deviation estimates of the parameters of
##             y=Xb+e, E(e)=0 and var(e)=const. Example:
##
##             >y:=[2 5 6 7 1 2]'; X:=[1 1 1 1 1 1; 8 5 5 6 7 10]';
##             >{b,sb,R2}:=fitLAD(X,y);
##             >b|sb|(b./sb), R2,
##
##             returns the estimate of b, its asymptotic standard deviation and an 
##             analog to the R2. Estimates b by linear programming. Note: invokes kron.e

n:=rows(X); 
k:=cols(X);

A:=X|kron(id(n),[-1,1]);
b:=y;
c:=zeros(1,k)|ones(1,2*n);
eq:=zeros(n,1);
restrict:=zeros(1,k)|ones(1,2*n);

{x,r,i}:=simplex(A,b,c,eq,restrict,min=1);

e:=sort(b-A[1:n,1:k].x[1:k]);

sde:=sum(abs(e'));
sdb:=sum(abs(b'-median(b')));
R2:=1-(sde/sdb);

even:=(floor((n-k)/2)==(n-k)/2);
m:=even*((n-k)/2+1)+(1-even)*(n-k+1)/2;
d:=max(1,min(4,floor((n-k)/6)));
f:=2*d/(n*(e[m+d]-e[m-d]));
lambda:=1/(2*f);
sx:=sqrt(diag((lambda^2)*inv(X'.X),0)');

return {x[1:k],sx,R2};

endfunction;

function fitRLS(X,y,R,r)
## Write {b,sb,s2}:=fitRLS(X,y,R,r)  

n:=rows(X);
k:=cols(X);

if isempty(R)==1,
   q:=0;
   A:=X'.X;
   b:=X'.y;
else,
   q:=rows(R);
   A:=(X'.X|R')_(R|zeros(q,q));
   b:=(X'.y)_r;
endif;
W:=A\id(k+q);
x:=W.b;

d:=x[1:k];
e:=y-X.d;
s2:=e'.e/(n-k);
sd:=sqrt(s2*diag(W[1:k,1:k],0)');
return {d,sd,s2};
endfunction;

function fitRGLS(X,y,R,r)
## Write {b,sb,s2}:=fitRGLS(X,y,R,r)  

d:=fitRLS(X,y,R,r);
e:=abs(y-X.d);
y:=y./e;
X:=X./e;
{d,sd,s2}:=fitRLS(X,y,R,r);
return {d,sd,s2};
endfunction;

function mixed(y,X,Z,dim,s20,method)
##MIXED Computes ML,REML,MINQE(I),MINQE(U,I),BLUE(b),BLUP(u)
## by Henderson's Mixed Model Equations Algorithm.
##
##======================================================================
## Syntax:
## {s2,b,u,Is2,C,H,q,loglik,loops}:=mixed(y,X,Z,dim,s20,method);
##
##======================================================================
## Model: Y=X*b+Z*u+e,
## b=(b_1',...,b_f')' and u=(u_1',...,u_r')',
## E(u)=0, Var(u)=diag(sigma^2_i*I_{m_i}), i=1,...,r
## E(e)=0, Var(e)=sigma^2_{r+1}*I_n,
## Var(y)=Sig=sum_{i=1}^{r+1} sigma^2_i*Sig_i.
## We assume normality and independence of u and e.
##
## Inputs:
## y - n-dimensional vector of observations.
## X - (n * k)-design matrix for
## fixed effects b=[b_1;...;b_f],
## typically X=[X_1,...,X_f] for some X_i.
## Z - (n * m)-design matrix for
## random efects u=[u_1;...;u_r],
## typically Z=[Z_1,...,Z_r] for some Z_i.
## dim - Vector of dimensions of u_i, i=1,...,r,
## dim=[m_1;...;m_r], m=sum(dim).
## s20 - A prior choice of the variance components,
## s20=[s20_1;...;s20_r;s20_{r+1}].
## SHOULD BE POSITIVE for method>0
## method - Method of estimation of variance components;
## 0:NO estimation, 1:ML, 2:REML, 3:MINQE(I), 4:MINQE(U,I)
##
##======================================================================
## Outputs:
## s2 - Estimated vector of variance components
## (sigma^2_1,..., sigma^2_{r+1})'.
## A warning message appears if some of the estimated
## variance components is negative or equal to zero.
## In such cases the calculated Fisher information
## matrices are inadequate.
## b - k-dimensional vector of estimated fixed effects beta,
## b=[b_1;...;b_f]=(X'Sig^{-1}X)^{+}X'Sig^{-1}y.
## u - m-dimensional vector of EBLUP's of random effects U,
## u=[u_1;...;u_r].
## Is2 - Fisher information matrix for variance components;
## if method=0 the output is Is2=[];
## if metod=3 or method=4 the output is inversion of the
## covariance matrix of MINQE calculated at estimated s2.
## C - g-inverse of Henderson's MME matrix, where
## C=pinv([XX XZ; XZ' ZZ+inv(D)*s0]/s0), if inv(D) exists
## or C=s0*[I 0; 0 D]*pinv([XX XZ*D; XZ' V]) otherwise
## H - Criterial matrix for MINQE calculated at priors s20;
## if method=3
## H_ij=trace(Sig_0^{-1}*Sig_i*Sig_0^{-1}*Sig_j),
## if method=4
## H_ij=trace((M*Sig_0*M)^{+}*Sig_i*(M*Sig_0*M)^{+}*Sig_j)
## q - (r+1)-dimensional vector of MINQE(U,I) quadratic forms
## calculated at prior values s20;
## if method=0,1,2 the output is q=[], otherwise
## q_i=y'*(M*Sig_0*M)^{+}*Sig_i*(M*Sig_0*M)^{+}*y.
## loglik - Log-likelihood evaluated at the estimated parameters;
## if method=1 loglik=log-likelihood(ML),
## if method=2 loglik=log-likelihood(REML),
## if method=3 or method=4 loglik=[],
## if method=0 loglik=informative value of
## log of the joint pdf of (y,u).
## loops - Number of loops.
##
##======================================================================
## REFERENCES
##
## Searle, S.R., Cassela, G., McCulloch, C.E.: Variance Components.
## John Wiley & Sons, INC., New York, 1992. (pp. 275-286).
##
## Witkovsky, V.: MATLAB Algorithm mixed.m for solving
## Henderson's Mixed Model Equations.
## Technical Report, Institute of Measurement Science,
## Slovak Academy of Sciences, Bratislava, Dec. 2001.
## See http://www.mathpreprints.com.
##
## The algorithm mixed.m is available at
## http://www.mathworks.com/matlabcentral/fileexchange
## see the Statistics Category.
##
##======================================================================
## Ver.: 2.0
## Revised 21-Dec-2001 20:31:48
## Copyright (c) 1998-2001 Viktor Witkovsky
##======================================================================
## CONTACT ADDRESS:
## Viktor Witkovsky
## Institute of Measurement Science
## Slovak Academy of Sciences
## Dubravska cesta 9
## 84219 BRATISLAVA, Slovak Republic
## Tel:(+421905) 223191
## Fax:(+4212) 54775943
## E-mail: umerwitk@savba.sk
## http://nic.savba.sk/sav/inst/umer/
##======================================================================
## BEGIN MIXED.M
##======================================================================
## This is the (only) required input.
## The algorith mixed.m could be easily changed in such a way
## that the required inputs will be y, a, XX, XZ, and ZZ,
## and the call would be mixed(y,a,XX,XZ,ZZ,dim,s20,method);
## instead of mixed(y,X,Z,dim,s20,method);
##======================================================================
## Original algorithm translated to Euler Math language by
## Luis Frank and available at: 
## http://compute.ku-eichstaett.de/MGF/wikis/euler/doku.php
## under software/ Frank's Toolbox.
##======================================================================
##y=y(:);
yy:=y'.y;
Xy:=X'.y;
Zy:=Z'.y;
XX:=X'.X;
XZ:=X'.Z;
ZZ:=Z'.Z;
a:=Xy_Zy;
## end of required input parameters
n:=rows(y);
k:=rows(XZ);
m:=cols(XZ);
rx:=rank(XX);
if cols(s20)>rows(s20), s20:=s20'; endif;
r:=rows(s20)-1;
Im:=id(m);
loops:=0;
##======================================================================
## METHOD=0:
## No estimation of variance components
## Output is BLUE(b), BLUP(u), and C,
## calculated at chosen values s20
##======================================================================
if method==0,
   s0:=s20[r+1];
   d:=s20[1]*ones(dim[1],1);
   for i=2 to r,
       d:=d_(s20[i]*ones(dim[i],1));
   end;
   D:=diag(d,0)';
   V:=s0*Im+ZZ.D;
   A:=(XX|XZ.D)_(XZ'|V);
   A:=mypinv(A);
   C:=s0*(A[1:k,1:k]|A[1:k,k+1:k+m])_(D.A[k+1:k+m,1:k]|D.A[k+1:k+m,k+1:k+m]);
   bb:=A.a;
   b:=bb[1:k];
   v:=bb[k+1:k+m];
   u:=D.v;
   Aux:=yy-b'.Xy-u'.Zy;
   if all(s20),
      loglik:=-((n+m)*log(2*pi)+n*log(s0)+log(prod(d'))+Aux/s0)/2;
   elseif
      loglik=[];
   endif;
s2:=s20;
Is2:=[];
H:=[];
q:=[];
return;
endif;
##======================================================================
## METHOD=1,2,3,4: ESTIMATION OF VARIANCE COMPONENTS
##======================================================================
fk:=nonzeros(s20<=0);
if any(fk),
   s20[fk]:=100*epsilon*ones(size(fk));
   "Priors in s20 are negative or zeros !CHANGED!";
endif;
sig0:=s20;
s21:=s20;
ZMZ:=ZZ-XZ'.mypinv(XX).XZ;
q:=zeros(r+1,1);
##======================================================================
## START OF THE MAIN LOOP
##======================================================================
epss:=0.000000001; ## Given precission for stopping rule
crit:=1;
repeat,
   loops:=loops+1;
   sigaux:=s20;
   s0:=s20[r+1];
   d:=s20[1]*ones(dim[1],1);
   for i=2 to r,
       d:=d_(s20[i]*ones(dim[i],1));
   end; 
   D:=diag(rows(d),rows(d),0,d');
   V:=s0*Im+ZZ.D;
   W:=s0*inv(V);
   T:=inv(Im+ZMZ.D/s0);
   A:=(XX|XZ.D)_(XZ'|V);
   bb:=mypinv(A).a;
   b:=bb[1:k];
   v:=bb[k+1:k+m];
   u:=D.v;
##======================================================================
## ESTIMATION OF ML AND REML OF VARIANCE COMPONENTS
##======================================================================

   iupp:=0;
   for i=1 to r,
       ilow:=iupp+1;
       iupp:=iupp+dim[i];
       Wii:=W[ilow:iupp,ilow:iupp];
       Tii:=T[ilow:iupp,ilow:iupp];
       w:=u[ilow:iupp];
       ww:=w'.w;
       q[i]:=ww/(s20[i]*s20[i]);
       s20[i]=ww/(dim[i]-sum(diag(Wii,0)));
       s21[i]=ww/(dim[i]-sum(diag(Tii,0)));
   end;
   Aux:=yy-b'.Xy-u'.Zy;
   Aux1:=Aux-(u'.v)*s20[r+1];
   q[r+1]:=Aux1/(s20[r+1]*s20[r+1]);
   s20[r+1]:=Aux/n;
   s21[r+1]:=Aux/(n-rx);
   if method==1,
      crit:=abs(sigaux-s20);
      H:=[];
      q:=zeros(r+1,1); ## originaly q:=[];
   elseif method==2,
      s20:=s21;
      crit:=abs(sigaux-s20);
      H:=[];
      q:=zeros(r+1,1); ## originaly q:=[];
   else
      crit:=0;
   endif;
   if crit<=epss then break endif;
end;
##======================================================================
## END OF THE MAIN LOOP
##======================================================================
## COMPUTING OF THE MINQE CRITERIAL MATRIX H
##======================================================================
if method>2,
   H:=id(r+1);
   if method==4,
      W:=T;
      H[r+1,r+1]:=(n-rx-m+sum(diag(W.W,0)))/(sigaux[r+1]*sigaux[r+1]); 
   else
      H[r+1,r+1]:=(n-m+sum(diag(W.W,0)))/(sigaux[r+1]*sigaux[r+1]);
   endif;
   iupp:=0;
   for i=1 to r;
       ilow:=iupp+1;
       iupp:=iupp+dim[i];
       trii:=sum(diag(W[ilow:iupp,ilow:iupp],0));
       trsum:=0;
       jupp:=0;
       for j=1 to r,
           jlow:=jupp+1;
           jupp:=jupp+dim[j];
           tr:=sum(diag(W[ilow:iupp,jlow:jupp].W[jlow:jupp,ilow:iupp],0));
           trsum:=trsum+tr;
           H[i,j]:=((i==j)*(dim[i]-2*trii)+tr)/(sigaux[i]*sigaux[j]);
       end;
       H[r+1,i]:=(trii-trsum)/(sigaux[r+1]*sigaux[i]);
       H[i,r+1]:=H[r+1,i];
   end;
endif;
##======================================================================
## SET THE RESULTS: MINQE(I), MINQE(U,I), ML, AND REML
##======================================================================
if method>2,
   s2:=mypinv(H).q;
   loglik=[];
else
   s2:=s20;
endif;
fk:=nonzeros(s2<10*epss);
if any(fk),
   "Estimated variance components are negative or zeros!";
endif;
##======================================================================
## BLUE, BLUP, THE MME'S C MATRIX AND THE LOG-LIKELIHOOD
##======================================================================
s0:=s2[r+1];
d:=s2[1]*ones(dim[1],1);
for i=2 to r,
    d:=d_(s2[i]*ones(dim[i],1));
end;
D:=diag(rows(d),rows(d),0,d');
V:=s0*Im+ZZ.D;
W:=s0*inv(V);
T:=inv(Im+ZMZ.D/s0);
A:=(XX|XZ.D)_(XZ'|V);
A:=mypinv(A);
C:=s0*(A[1:k,1:k]|A[1:k,k+1:k+m])_(D.A[k+1:k+m,1:k]|D.A[k+1:k+m,k+1:k+m]);
bb:=A.a;
b:=bb[1:k];
v:=bb[k+1:k+m];
u:=D.v;
if method==1,
   loglik:=-(n*log(2*pi*s0)-log(det(W))+n)/2;
elseif method==2,
   loglik:=-((n-rx)*log(2*pi*s0)-log(det(T))+(n-rx))/2;
endif;
##======================================================================
## FISHER INFORMATION MATRIX FOR VARIANCE COMPONENTS
##======================================================================
Is2:=id(r+1);
if method==2
   W:=T;
   Is2[r+1,r+1]:=(n-rx-m+sum(diag(W.W,0)))/(s2[r+1]*s2[r+1]);
elseif method==4
   W:=T;
   Is2[r+1,r+1]:=(n-rx-m+sum(diag(W.W,0)))/(s2[r+1]*s2[r+1]);
else
   Is2[r+1,r+1]:=(n-m+sum(diag(W.W,0)))/(s2[r+1]*s2[r+1]);
endif;

iupp:=0;
for i=1 to r;
    ilow:=iupp+1;
    iupp:=iupp+dim[i];
    trii:=sum(diag(W[ilow:iupp,ilow:iupp],0));
    trsum:=0;
    jupp:=0;
    for j=1 to r,
        jlow:=jupp+1;
        jupp:=jupp+dim[j];
        tr:=sum(diag(W[ilow:iupp,jlow:jupp].W[jlow:jupp,ilow:iupp],0));
        trsum:=trsum+tr;
        Is2[i,j]=((i==j)*(dim[i]-2*trii)+tr)/(s2[i]*s2[j]);
    end;
    Is2[r+1,i]:=(trii-trsum)/(s2[r+1]*s2[i]);
    Is2[i,r+1]:=Is2[r+1,i];
end;
Is2:=Is2/2;
return {s2,b,u,Is2,C,H,q,loglik,loops};
##======================================================================
## EOF MIXED.M
##======================================================================
endfunction;

