function parabpde(a,b,c,d,funy0,t0,t1,nt,x0,x1,nx,A,B)
## Using the method of lines for solving 
## parabolic partial differential equation of the form
##
## D(y,t) = a(x)*D(y,x,x) + b(x)*D(y,x) + c(x)*y + d(x)
## where y = y(x,t) and x,t - independent variables
##
## Function arguments:
## a(x),b(x),c(x),d(x) - function of scalar argument x
## fyny0(x,t0) - initial condition (x-vector,t0-scalar)
## t0,t1 - boundaries of t
## nt - number of intervals in t
## x0,x1 - boundaries of x
## nx - number of intervals in x
## A,B - vectors of 3 constants in linear boundary conditions 
## x = x0, A[1]*y + A[2]*D(y,x) = A[3]
## x = x1, B[1]*y + B[2]*D(y,x) = B[3]
## Function returns: {t,x,Y}
## x,y - vectors in x and t domain
## Y - matrix of solutions (x-in rows, t-in columns)
    x = linspace(x0,x1,nx);
    dx = x[2]-x[1];
    y0 = funy0(x,t0);
    t = linspace(t0,t1,nt);
    Y = adaptiverunge("Dyt",t,y0;nx,dx,"a","b","c","d",A,B);
    Y[1,:] = (dx*A[3]-Y[2,:]*A[2])/(A[1]*dx-A[2]);
    Y[nx+1,:] = (dx*B[3]+Y[nx,:]*B[2])/(B[1]*dx+B[2]);
    return {t,x,Y};
endfunction;

function Dyt(t,y,nx,dx,a,b,c,d,A,B)
##function used by parabpde
##(finite difference aproximations in x)
    dy = zeros(1,nx+1);
    y[1] = (dx*A[3]-y[2]*A[2])/(A[1]*dx-A[2]);
    y[nx+1] = (dx*B[3]+y[nx]*B[2])/(B[1]*dx+B[2]);
    loop 2 to nx;
         x=(#-1)*dx;
         dy[#]=a(x)/dx^2*(y[#+1]-2*y[#]+y[#-1]) ..
         +b(x)/2/dx*(y[#+1]-y[#-1]) + c(x)*y[#] + d(x);
    end;
    return dy;
endfunction;

function plotY(t,x,Y)
##animate the solution of parabolic pde
##obtained by parabpde
##t,x - vectors returned by parabpde
##Y - matrix returned by parabpde
##
    deletepages();
	clg;
	addpage();
	showpage(1);
    a = totalmin(x); 
    b = totalmax(x); 
    c = totalmin(Y); 
    d = totalmax(Y);
    loop 1 to cols(Y);
        plot2d(x,Y[:,#]',0,a,b,c,d);
        title("y = f(x,t)");
        xlabel("x");
        ylabel("y");
        text(printf("t=%8.3g",t[#]),100,1);
        addpage();
    end;
    animate();
    return 0;
endfunction;
