function fitOLS(X,y)
## fitOLS(X,y) returns the OLS estimated parameters b, s(b), and the  
##             corresponding t statistics with their p-values. It also  
##             returns F and its p-value, and R^2 and the adjusted R^2.
##             To get all these statistics use the sintax
##
## {table,R2,s2,F}:=fitOLS(X,y)
##
## Last revised: July 22nd. 2011.

n:=rows(X); 
k:=cols(X);
c:=(n-1)/(n-k);
z:=y-mean(y');
b:=fit(X,y); 
e:=y-X.b; 
s2:=e'.e/(n-k); 
sb:=sqrt(diag(s2*inv(X'.X),0)');
t:=b./sb;
pt:=1-tdis(abs(t),n-k);
R2:=1-(e'.e)/(z'.z);
R2:=R2|(1-c+R2*c);
F:=(R2[1]/(k-1))/((1-R2[1])/(n-k));
pF:=1-fdis(F,k-1,n-k);
return {(b|sb|t|pt),R2,s2,(F|pF)};
endfunction;


function fitCO(X,y,p)
## fitCO(X,y,p) returns the iterated estimation b following the Cochrane-Orcutt
##              procedure when the error term is autorregresive of order p. The
##              iteration will stop when the maximun difference between two
##              is smaller than 0.001 or after 10 iterations, what ever happens first.
##
## May 26th., 2011.

n:=rows(X);
k:=cols(X);

b:=fit(X,y);
e:=y-X.b;
E:=xlag(e,p);
rho:=fit(E,e[p+1:n]);
for m=1 to 10
    r:=rho;
    Z:=y[p+1:n]|X[p+1:n,1:k];
    for h=1 to p,
        Z:=Z-(y[p-h+1:n-h,1]|X[p-h+1:n-h,1:k])*rho[h];
    end;
    b:=fit(Z[:,2:k+1],Z[:,1]);
    e:=y-X.b;
    E:=xlag(e,p);
    rho:=fit(E,e[p+1:n]);
    if all(max(abs(rho-r))<0.001),
       s2:=e'.e/(n-k);
       sb:=sqrt(s2*diag(inv(Z[:,2:k+1]'.Z[:,2:k+1]),0))';
       break,
    endif;
end;
return {b,sb,s2,rho};
endfunction;


function xlag(x,p)
X:=[]; 
for i=p+1 to length(x), 
    X:=X_x[i-1:-1:i-p]'; 
end; 
return X;
endfunction;


function fitLAD(X,y)
## fitLAD(X,y) Returns the Least Absolute Deviation estimates of the parameters of
##             y=Xb+e, E(e)=0 and var(e)=const. Example:
##
##             >y:=[2 5 6 7 1 2]'; X:=[1 1 1 1 1 1; 8 5 5 6 7 10]';
##             >{b,sb,R2}:=fitLAD(X,y);
##             >b|sb|(b./sb), R2,
##
##             returns the estimate of b, its asymptotic standard deviation and an 
##             analog to the R2. Estimates b by linear programming. Note: invokes kron.e

## References:
## Pynnnen, Seppo and Timo Salmi (1994). A Report on Least Absolute Deviation
##   Regression with Ordinary Linear Programming. Finnish Journal of Business
##   Economics 43(1):33-49. Available at: 
##   http://lipas.uwasa.fi/~ts/publicat/RobuPynnonenSalmi.pdf
## Dasgupta, M. and S.K. Mishra (2004).  Least absolute deviation estimation of 
##   linear econometric models: A literature review. Available at:
##   http://mpra.ub.uni-muenchen.de/1781/ .
## 
## See also:
## Ojeda M. y Otero Pereira J.M. (1995). Programacin lineal y estimacin MVA en 
##   regresin lineal. La Ciencia y el Hombre, 19:77-90. Available at:
##   http://cdigital.uv.mx/handle/123456789/5298
##
## May, 2011.

n:=rows(X); 
k:=cols(X);

A:=X|kron(id(n),[-1,1]);
b:=y;
c:=zeros(1,k)|ones(1,2*n);
eq:=zeros(n,1);
restrict:=zeros(1,k)|ones(1,2*n);

{x,r,i}:=simplex(A,b,c,eq,restrict,min=1);

e:=sort(b-A[1:n,1:k].x[1:k]);

sde:=sum(abs(e'));
sdb:=sum(abs(b'-median(b')));
R2:=1-(sde/sdb);

even:=(floor((n-k)/2)==(n-k)/2);
m:=even*((n-k)/2+1)+(1-even)*(n-k+1)/2;
d:=max(1,min(4,floor((n-k)/6)));
f:=2*d/(n*(e[m+d]-e[m-d]));
lambda:=1/(2*f);
sx:=sqrt(diag((lambda^2)*inv(X'.X),0)');

return {x[1:k],sx,R2};
endfunction;


function kron(A,B)
## kron(A,B). Returns the Kronecker tensor product between A and B.
##
## Luis Frank, November 2009.

x:=size(A);
y:=size(B);
C:=zeros(x[1]*y[1],x[2]*y[2]);
for i=1 to x[1]
    for j=1 to x[2]
        a:=[1:y[1]]+(i-1)*y[1];
        b:=[1:y[2]]+(j-1)*y[2];
        C[a,b]=A[i,j].B;
    end;
end;
return C;
endfunction;


function iterLAD(X,y)
n:=rows(X);
k:=cols(X);
b:=zeros(k,1);
for m=1 to 1000
    d:=b;
    for j=2 to k
        h:=[2:k];
        h:=h[nonzeros(h!=j)];
        A:=X[:,[1,j]];
        c:=y-X[:,h].b[h];
        b[j]:=fitLAD(A,c)'[2];
    end;
    b[1]:=fitLAD(ones(n,1),(y-X[:,2:k].b[2:k]));
    if max(d'-b')<0.00001, 
       break; 
    endif;
end;
return {b,m};
endfunction;

function isempty(R) := return cols(R)==0

function fitRLS(X,y,R,r)
## fitRLS(X,y,R,r)   returns the LS estimate of  for the linear model
##        y=X+v s.t. R=r, where v is homoskedastic. b is the estimate
##        of , sb is the vector of standard deviations of b and s2 is
##        the estimate of var(e). Use [] for R and r just to fit X on y.
##        See:
##
## References:
##
## Greene W. and T. Seaks, 1991. The Restricted Least Squares Estimator: A 
## pedagogical Note. The Review of Economics and Statistics 73(3): 563-567.
##
## Luis Frank, March 2011.

n:=rows(X);
k:=cols(X);

if isempty(R)==1,
   q:=0;
   A:=X'.X;
   b:=X'.y;
else,
   q:=rows(R);
   A:=(X'.X|R')_(R|zeros(q,q));
   b:=(X'.y)_r;
endif;
W:=A\id(k+q);
x:=W.b;

d:=x[1:k];
e:=y-X.d;
s2:=e'.e/(n-k);
sd:=sqrt(s2*diag(W[1:k,1:k],0)');
return {d,sd,s2};
endfunction;


function fitRGLS(X,y,R,r)
## {b,sb,s2}:= fitRGLS(X,y,R,r) returns the feasible GLS estimate of   
##        for the linear model y = X+v s.t. R=r, v being heteroskedastic.   
##        b is the estimate of , sb is the vector of std.dev. of b
##        and s2 is var.est.(e). Use [] for R and r to fit X on y only. 
##
## Luis Frank, March 2011.

d:=fitRLS(X,y,R,r);
e:=abs(y-X.d);
y:=y./e;
X:=X./e;
{d,sd,s2}:=fitRLS(X,y,R,r);
return {d,sd,s2};
endfunction;


function fitWHITE(X,y)
## {b,sb,s2}:= fitWHITE(X,y) returns the White's consistent heteroskedastic
##             estimation of .

n:=rows(X); k:=cols(X);
G:=(X'.X)\id(k);
b:=G.(X'.y);
e:=y-X.b;
s2:=e'.e/(n-k);
Z:=X.*abs(e); V:=Z.G;
sb:=sqrt(diag(V'.V,0)');
return {b,sb,s2};
endfunction;


function fitRWHITE(X,y,R,r)
## {b,sb,s2}:= fitRWHITE(X,y,R,r) returns a feasible restricted White-type 
##             heteroskedastic estimation of .

n:=rows(X); k:=cols(X); q:=rows(R);
d:=fitRLS(X,y,R,r);
e:=y-X.d;
s2:=e'.e/(n-k);

G:=(X'.X)\id(k);
Z:=X.*abs(e);
V:=Z.G; W:=(Z'.Z)\id(k);

C:=(X'.X).W.X';
A:=((C.X)_R)|(R'_zeros(q,q)); x:=(C.y)_r;
b:=A\x;
b:=b[1:k];

S:=V'.V; T:=R.S;
sb:=sqrt(diag(S-(T'.inv(R.S.R').T),0)');
return {b,sb,s2};
endfunction;


function fitSRLS(X,y,R,r,V,w)

n:=rows(X);
k:=cols(X);
q:=rows(R);
if isempty(V)==1
   W:=id(q);
else
   W:=inv(V);
endif;
G:=X'.X;
M:=-X.pinv(G).X';
for i=1 to n
    M[i,i]:=1+M[i,i];
end;
s2:=y'.M.y/(n-k);
A:=w*s2*(R'.W.R);
B:=w*s2*(R'.W.r);
b:=(G+A)\(X'.y+B);
sb:=s2*sqrt(diag((G+w*R'.W.R)\id(k),0)'); ##ver
return {b,sb,s2};
endfunction;


function fitTHEIL(X,y)
## fitTHEIL(X,y) computes the Theil-Sen estimator of b in the linear 
##         model y=Xb+e. The first column of X is the vector of 1.
##         The procedure calls fitSimpleTHEIL(X,y) at each step.
##
## Luis Frank, March 2011.

k:=cols(X);
b:=zeros(k,1);
for m=1 to 1000
    d:=b;
    for j=2 to k
        h:=[2:k];
        h:=h[nonzeros(h!=j)];
        A:=X[:,[1,j]];
        c:=y-X[:,h].b[h];
        b[j]:=fitSimpleTHEIL(A,c)[2];
    end;
    b[1]:=median((y-X[:,2:k].b[2:k])');
    if max(d'-b')<0.00001, 
       break; 
    endif;
end;
return {b,m};
endfunction;


function fitSimpleTHEIL(X,y)
## fitSimpleTHEIL(X,y) returns Theil-Sen the robust estimation
##      for a simple linear model.
##
## Luis Frank, March 2011.

{w,k}:=unique(X[:,2]');
y:=y[k];
X:=X[k,:];
a:=[];
for i=2 to length(y)
    for j=1 to i-1
        a:=a|((y[i]-y[j])/(X[i,2]-X[j,2]));
    end;
end;
b1:=median(a);
b0:=median(y'-X[:,2]'*b1);
b:=[b0,b1]';
return b;
endfunction;


function fitRANK(X,y)
## fitRANK(X,y) returns the forcasted y and the estimator b after the
##              rank-based transformation of X and y.
##
## Syntax: {table,yest}:=fitRANK(X,y)
##
## References:
## Conover W., 1982. Practical Nonparametric Statistics. 2nd Edition. 
##                   Wiley Series in Probability and Statistics.
##
## Last revised November 15th., 2011.

n:=rows(X);
k:=cols(X);
Z:=y|X;
for j=1 to k+1,
    Z[:,j]:=ranking(Z[:,j]')';
end;

{table,R2,s2,F}:=fitOLS(Z[:,2:k+1],Z[:,1]);
b:=table[:,1];
r:=Z[:,2:k+1].b;
 
w:=sort(Z[:,1]);
s:=min(Z[:,1]')|max(Z[:,1]');
yest:=[];

for i=1 to n,
    r[i]:=r[i]*(1-(r[i]<s[1]))+s[1]*(r[i]<s[1]);
    r[i]:=r[i]*(1-(r[i]>s[2]))+s[2]*(r[i]>s[2]);

    m:=find(w,r[i]);
    d:=(r[i]-w[m])/(w[m+1]-w[m]);
    v:=(y[nonzeros(Z[:,1]==w[m])][1])|(y[nonzeros(Z[:,1]==w[m+1])][1]);
    yest:=yest_(v[1]+d*(v[2]-v[1]));
end;
shortformat; 
return {table,yest};
endfunction;


function ranking(v)
## This is an auxiliary function that returns the ranks of a row vector v.
## In case of ties, ranking(.) returns the mean rank.

w:=sort(v);
h:=[];
for j=1 to cols(v), 
    m:=nonzeros(w==v[j]); 
    h:=h|(m[-1]-m[1]+1)*(m[-1]+m[1])/(2*cols(m));
end;
return h;
endfunction;


function fitPROBIT(X,y)
## fitPROBIT(X,y) returns the estimated parameters and the asymptotic s.d. of a probit
##                model. The estimates are computed by the Newton-Raphson method. The
##                stop criterium is max(|b(n+1)-b(n)|)<0.0001. Recall that b is normally
##                distributed as the function provides the asymptotic s.d. 
##
## Example. Let 
##
##    X:=[-48.5,24.4,82.8,-24.6,-31.6,91.0,52.1,-87.7,-17.0,-51.5,-90.7,65.5,-44.0,
##        -7.0,51.6,32.4,-61.8,34.0,27.9,-72.9,49.9]';
##    X:=ones(21,1)|X;
##    y:=[0,0,1,0,0,1,1,0,0,0,0,1,1,0,1,1,0,1,1,0,1]';
##
## >{table,lnL,m}:=fitPROBIT(X,y)
##
## >table
##
##    -0.0644337952279     0.3992382921739 
##    0.02999898077314    0.01028642882809
##
##    where the first column is the vector b of estimates and the second is s(b).
##    The example belongs to Lee Atkins (2011). The Newton-Raphson method is decribed 
##    by Thomas Fomby (2010).
##
##    Atkins L. (2011). Econ 4213 Page. At: http://www.learneconometrics.com/class/4213/4213.htm .
##    Fomby T. B. (2010). Lecture Notes on Binary Choice Models: Logit and Probit.
##
## December 4th. 2011.

n:=rows(X);
k:=cols(X);
b:=fit(X,y);

for m=1 to 50
    d:=zeros(k,1);
    H:=zeros(k,k);
    lnL:=0;
    c:=b;

    z:=X.b;
    f:=(1/sqrt(2*pi))*exp(-z^2/2);
    F:=normaldis(z);
    for i=1 to n
        lnL:=lnL+(y[i]*ln(F[i])+(1-y[i])*ln(1-F[i]));
        d:=d+(y[i]*f[i]/F[i]-(1-y[i])*f[i]/(1-F[i]))*X[i,:]';
        v:=(f[i]+z[i]*F[i])/F[i]^2;
        w:=(f[i]-z[i]*(1-F[i]))/(1-F[i])^2;
        H:=H-f[i]*(y[i]*v+(1-y[i])*w)*(X[i,:]'.X[i,:]);
    end;
    G:=H\id(k);
    b:=b-G.d;
    if max(abs(b[2:k]'-c[2:k]'))<0.0001
       break;
    endif;  
end;
sb:=sqrt(diag(-G,0))';
return {b|sb,lnL,m};
endfunction;


function fitLOGIT(X,y)
## fitLOGIT(X,y) returns the estimated parameters and the asymptotic s.d. of a logit
##               model. The estimates are computed by the Newton-Raphson method. The
##               stop criterium is max(|b(n+1)-b(n)|)<0.0001.
##
## For the example in fitPROBIT's help, fitLOGIT(X,y) returns
##
## >{table,lnL,m}:=fitLOGIT(X,y)
##
##  -6.166046276153
##
## >table
##    -0.2375740709789     0.7499211644473 
##     0.05310975416303    0.02061707281261 
##
## December 4th. 2011.

n:=rows(X);
k:=cols(X);
b:=fit(X,y);

for m=1 to 50
    d:=zeros(k,1);
    H:=zeros(k,k);
    lnL:=0;
    c:=b;

    t:=X.b;
    f:=exp(-t)./(1+exp(-t))^2;
    F:=ones(n,1)./(1+exp(-t));
    for i=1 to n
        lnL:=lnL+(y[i]*ln(F[i])+(1-y[i])*ln(1-F[i]));
        d:=d+(y[i]*f[i]/F[i]-(1-y[i])*f[i]/(1-F[i]))*X[i,:]';
        H:=H-f[i]*(X[i,:]'.X[i,:]);
    end;
    G:=H\id(k);
    b:=b-G.d;
    if max(abs(b[2:k]'-c[2:k]'))<0.0001
       break;
    endif;  
end;
sb:=sqrt(diag(-G,0))';
return {b|sb,lnL,m};
endfunction;


function corrcoef(X)
## corrcoef(X) returns the correlation coefficient matrix of matrix X.
##             As a rule of thumb, if at least one R[i,j]>0.9 there is 
##             evidence of linear association between the columns of X.
##             Remember to omit the columns of 1 when calling X.
##
## May 5th., 2011.

k:=cols(X); 
R:=zeros(k,k); 
m:=mean(X')'; 

for j=1 to k, 
    X[:,j]:=X[:,j]-m[j]; 
end; 

for j=1 to k, 
    for i=1 to k,
        R[i,j]:=(X[:,i]'.X[:,j])/sqrt((X[:,i]'.X[:,i])*(X[:,j]'.X[:,j]));
        R[j,i]:=R[i,j]; 
    end; 
end;
return R;
endfunction;


function partialcorr(X,y)
## partialcorr(X,y) returns the partial correlation coefficient between y and each
##                  X[:,j]. High R2 but low partial correlations reveals evidence of
##                  multicollinearity.
##
## May 3rd., 2011.

k:=cols(X);
A:=[];
for j=2 to k,
    Z:=X[:,nonzeros(1:k!=j)];
    v:=fit(Z,y);
    w:=fit(Z,X[:,j]);    
    e1:=y-Z.v;
    e2:=X[:,j]-Z.w;
    rho:=(e1'.e2)/sqrt((e1'.e1)*(e2'.e2));
    A:=A_(j|rho);
end;
return A;
endfunction;


function vif(Z)
## vif(X). Computes the R2 of tha auxilliary regression xj = X(j) and the
##         associated Variance Inflation Factor. The function returns the 
##         matrix j|R2|VIF. As a rule of thumb, multicollinearity is serious
##         when VIF>10.
##
## May 3rd., 2011.

vif:=[];
for j=2 to cols(Z),
    y:=Z[:,j];    
    X:=Z[:,nonzeros(1:cols(Z)!=j)];
    b:=fit(X,y); 
    e:=y-X.b;

    y:=y-mean(y');
    R2:=1-(e'.e)/(y'.y);
    vif:=vif_(j|R2|(1/(1-R2)));
end;
return vif;
endfunction;


function FGtest(X)
## FGtest(X) returns the Farrar Glauber F statistic and its p-value to
##           test for linear dependency between xj and the X(j). Under the
##           null =0 in the model xj = X(j) + v.
##
## May 24th., 2011.

n:=rows(X);
k:=cols(X);
F:=[];

c:=(n-(k-1))/(k-2);
A:=vif(X);
for h=1 to k-1
    f:=(A[h,2]/A[h,3])*c;
    F:=F_(A[h,1]|f|(1-fdis(f,k-1,n-k+1)));
end;
return F;
endfunction;


function cond(X)
## cond(X). Returns the square root of the ratio between the maximun and 
##          minimun eigenvalues of X'X. According to Judge et al. (1985)
##          5<=cond(X)<=10 reveals low to moderate multicolinearity, whereas
##          30<=cond(X)<=100 reveals moderate to high linear dependance.
##
## May 5th., 2011.

Z:=[]; 
for j=1 to cols(X),
    Z:=Z|(X[:,j]/sqrt(X[:,j]'.X[:,j]));
end;
{d,P}:=eigen(Z'.Z);
d:=real(d);
return sqrt(max(d)/min(d));	
endfunction;


function numerate(v)
## This is an auxiliary function for rankcond(X). It returns the ranks
##      of a row vector v. If ties appear, numerate(.) returns the delta 
##      that should be substracted to the rank-SC.

w:=sort(v);
h:=[]; d:=[]; 
for j=1 to cols(v), 
    m:=nonzeros(w==v[j]); 
    h:=h|(m[-1]-m[1]+1)*(m[-1]+m[1])/(2*cols(m));
    d:=d|((m[-1]-m[1])*(m[-1]-m[1]+1)*(m[-1]-m[1]+2)/12)/(m[-1]-m[1]+1);
end;
delta:=sum(d);
return {h,delta};
endfunction;


function rankcond(X)
## rankcond(X) returns the condition number of a rank-transformed matrix X,
##             transformed in such a way that the diagonal elements of X'X 
##             are equal to 1.

n:=rows(X);
k:=cols(X);
m:=[1:n].[1:n]';
Z:=[];
for j=1 to k,
    {h,delta}:=numerate(X[:,j]');
    Z:=Z|(h'/sqrt(m-delta));
end;
{d,P}:=eigen(Z'.Z);
d:=real(d);
return sqrt(max(d)/min(d));
endfunction;


function bartlett(X)
## bartlett(X) returns Bartletts's T statistic and its p-value to test for 
##             collinearity among the columns of X. Under the null T is 
##             distributed chi-square with k*(k-1)/2 degrees of freedom.
##             Recall that Ho assumes that X is a normal multivariate sample with
##             correlation matrix equal to id(k).
##
## May 24th., 2011.

n:=rows(X);
k:=cols(X);

Z:=corrcoef(X);
T:=-(n-1-(2*k+5)/6)*ln(det(Z));
pvalue:=1-chidis(T,k*(k-1)/2);
return {T,pvalue};
endfunction;


function GQtest(X,y,j)
## GQtest(X,y,j) returns the p-value of the Golfeld-Quandt heteroskedasticity 
##               test where the the error variance is supposed to be a function 
##               of the j-th column. The number of deleted values are computed
##               as c:=max(4,floor(n/10).

n:=rows(X);
k:=cols(X);
c:=max(4,floor(n/10));
{w,i}:=sort(X[:,j]);

v:=i[1:round((n-c)/2,0)];
w:=i[round((n-c)/2,0)+c+1:n];
y1:=y[v]; X1:=X[v,:]; b1:=fit(X1,y1);
y2:=y[w]; X2:=X[w,:]; b2:=fit(X2,y2);
e1:=y1-X1.b1; e2:=y2-X2.b2;
df:=round((n-c-2*k)/2,0);

F:=(e1'.e1/df)/(e2'.e2/df); 
pvalue:=1-fdis(F,df,df);
return {pvalue,F,df};
endfunction;


function white(X,y)
## white(X,y) returns the p-value of White's statistic nR^2 to test for 
##            heteroskedasticity.
##
## May 15th., 2010.

n:=rows(X);
k:=cols(X);
Z:=X;
for i=2 to k, 
    for j=i to k,
        Z:=Z|(X[:,i].*X[:,j]);
    end; 
end;
b:=fit(X,y);
e:=y-X.b;

z:=e.*e;
c:=fit(Z,z);
v:=z-Z.c;
w:=z-mean(z');
R2:=1-(v'.v)/(w'.w);
pvalue:=1-chidis(n*R2,cols(Z)-1);
return {pvalue,R2,n};
endfunction;


function ARCHtest(X,y,p)
## ARCHtest(X,y,p). returns the p-value of Engle's test for AutoRegressive 
##                  Conditional
 Heteroskedasticity (ARCH). 
n:=rows(X);

b:=fit(X,y);
e:=y-X.b;

E:=xlag(e,p); Z:=ones(n-p,1)|(E.*E); e2:=e[p+1:n].*e[p+1:n];
a:=fit(Z,e2); u:=e2-Z.a; w:=e2-mean(e2');

R2:=1-(u'.u)/(w'.w);
pvalue:=1-chidis(n*R2,p);
return {pvalue,R2};
endfunction;


function xlag(x,p)
X:=[]; 
for i=p+1 to length(x), 
    X:=X_x[i-1:-1:i-p]'; 
end; 
return X;
endfunction;


function spearman(X,y)
## spearman(X,y) returns Spearman's correlation coefficient between the 
##               residuals (in absolute value) of y regressed on X, and 
##               each column of X.

n:=rows(X);
k:=cols(X);
table:=[];

b:=fit(X,y);
e:=y-X.b;
v:=ranking(abs(e'));
for j=2 to k,
    d:=v-ranking(X[:,j]');
    rho:=1-6*(d.d')/(n*(n^2-1));
    table:=table_(j|rho);
end;
return table;
endfunction;


function gearytest(x)
## gearytest(x) returns the p-value of the streak test statistic under the null "the
##              signs of x appear at random". x is a row vector.
##
## 25th. May, 2011.

m:=length(x);
v:=x>=0;
w:=abs(1|(v[2:m]-v[1:m-1])); k:=sum(w);
n1:=sum(w.*v); n2:=k-n1;

mu:=2*(n1*n2)/(n1+n2)+1;
s2:=2*(n1*n2)*(2*n1*n2-n1-n2)/((n1+n2)^2)*(n1+n2-1);
z:=(k-mu)/sqrt(s2);
pval:=1-normaldis(z);
return {pval,n1,n2};
endfunction;


function DWtest(X,y)
## DWtest(X,y) returns the Durbin-Watson statistic to test for first order
##             autocorrelation in the residuals and the estimated rho. Before
##             running this test it is suggested to check for ARCH.
##
## 25th. May, 2011.

n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

v:=e[2:n]-e[1:n-1];
d:=(v'.v)/(e[1:n]'.e[1:n]);
rho:=1-d/2;

return {d,rho};
endfunction;


function wallistest(X,y)
## wallistest(X,y,p) returns the Wallis statistic to test for fourth order
##             autocorrelation in the residuals.
##
## References
##  Wallis, Econometrica, Vol. 40, 1972 
##  Giles and King, Journal of Econometrics, Vol. 8, 1978
##  King M. and Giles D. 1977. A Note on Wallis' Bounds Test and Negative 
##          Autocorrelation. Econometrica 45(4):1023-1026.
##
## 25th. May, 2011.

p:=4;
n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

w:=e[p+1:n]-e[1:n-p];
d:=(w'.w)/(e[1:n]'.e[1:n]);

return d;
endfunction;


function BGtest(X,y,p)
## BGtest(X,y,p) returns the p-value of Breusch-Godfrey's statistic to test for
##               autocorrelation of order p. This test has two versions, one of 
##               which computes the chi-square statistic and another that uses 
##               the F statistic.
##
## 25th. May, 2011.

n:=rows(X);
k:=cols(X);
b:=fit(X,y);
e:=y-X.b;

Z:=[]; 
for i=p+1 to n, 
    Z:=Z_e[i-1:-1:i-p]'; 
end;

Z:=X[p+1:n,1:k]|Z;
z:=e[p+1:n];
c:=fit(Z,z);
v:=z-Z.c;

w:=z-mean(z');
R2=1-(v'.v)/(w'.w);
pvalue:=1-chidis((n-p)*R2,p); ## F:=(n-k-1-p)*R2/(p*(1-R2)); 1-fdis(F,p,n-k-1-p);
return {pvalue,R2};
endfunction;


function BPtest(X,y,p)
## BPtest(X,y,p) returns the p-value of the Box-Pierce Q statistic. This
##               statistic tests for autocorrelation on the first p OLS 
##               residuals. The distribution of Q is derived under the
##               assumption that residuals are generated by an AR or ARMA
##               process. Under the null hypothesis of no autocorrelation the
##               Q statistic has asymptotically a chi-square distribution with
##               degrees of freedom equal to p.
##
## 25th. May, 2011.

n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

r:=[];
s:=e[1:n]'.e[1:n];
for j=1 to p
    q:=(e[j+1:n]'.e[1:n-j])/s;
    r:=r_q;
end;
Q:=n*(r'.r);
pvalue:=1-chidis(Q,p);
return {pvalue,Q};
endfunction;


function LBtest(X,y,p)
## LBtest(X,y,p) returns the p-value of the Ljung-Box Q statistic. This
##               statistic is a refined version of the Box-Pierce test.
##               Under the null hypothesis of no autocorrelation the Q
##               statistic has asymptotically a chi-square distribution
##               with degrees of freedom equal to p. This test is suitable
##               for ARIMA residuals. 
##
## 25th. May, 2011.

n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

r:=[];
s:=e[1:n]'.e[1:n];
for j=1 to p
    q:=((e[j+1:n]'.e[1:n-j])/s)/sqrt(n-j);
    r:=r_q;
end;
Q:=n*(n+2)*(r'.r);
pvalue:=1-chidis(Q,p);
return {pvalue,Q};
endfunction;


function BWtest(X,y)
## BWtest(X,y) returns the g statistic of Berenblutt-Webb test for the null
##             rho=1. That is, the g statistic is distributed as Durbin-Watson's d.
##
## 27th. May, 2011.

n:=rows(X);
k:=cols(X);

b:=fit(X,y);
e:=y-X.b;

Z:=X[2:n,2:k]-X[1:n-1,2:k];
z:=y[2:n]-y[1:n-1];
c:=fit(Z,z);
u:=z-Z.c;

g:=(u'.u)/(e'.e);
return g;
endfunction;


function vNtest(X,y)
## vNtest(X,y) returns the p-value of the von Neumann autocorrelation test.
##             This is a large sample test to test for first orden autocorrelation.
##
## 27th. May, 2011.

n:=rows(X);
b:=fit(X,y);
e:=y-X.b;

v:=e[2:n]-e[1:n-1];
r:=((v'.v)/(n-1))/((e'.e)/n);

mu:=2*n/(n-1);
s2:=4*(n^2)*(n-2)/((n+1)*(n-1)^3);
z:=(r-mu)/sqrt(s2);

pvalue:=1-normaldis(abs(z));
return {pvalue,z};
endfunction;


function JBtest(x)
## JBtest(x) returns the Jarque-Bera test statistic and its p-value under the null
##           hypothesis that x is a normal random vector. In this case the JB statistic
##           is asymptotically distributed chi squared with 2 degrees of freedom.
##
## December 1st. 2011.

if cols(x)==1, 
   x:=x'; 
endif;
n:=cols(x);
u:=x-mean(x);

S:=(sum(u^3)/n)/(u.u'/n)^1.5;
K:=(sum(u^4)/n)/(u.u'/n)^2;
jb=n*(S^2+0.25*(K-3)^2)/6;

return {jb,1-chidis(jb,2)};
endfunction;


function dummy(A)
## dummy(A) Given an A matrix of factors and levels, dummy(A) returns a zero-one
##         matrix where each column is a factor. If A has a sigle factor, this 
##         should be a column vector. The levels appear in ascending order.
##
## Luis Frank, October 2009.

 s:=size(A);
 Z:=[];
 for j=1 to s[2]
    u:=unique(A[:,j]');
    Z:=Z|A[:,j]==u;
 end;
 return Z;
endfunction;
