comment
Functions to create incidence matix for a graph, and
to solve resistance potential in a graph. For a demo
load the Resitance.en notebook.
For more information use "help incidence.e" or see the
user menu.
endcomment

function makeIncidence (K,n)
## Make an incidence matrix A from the lines of K.
## K[n] is [i,j,c], and A[i,j]=A[j,i] will be 1/c.
## So A[i,j] contains the capacity of the edge from
## i to j, when c[n] is the resistance of this edge.
## see: solvePotential
	A=zeros(n,n);
	loop 1 to rows(K);
		i=K[#,1]; j=K[#,2];
		A[i,j]=A[i,j]+1/K[#,3];
		A[j,i]=A[i,j];
	end
	return A
endfunction

addmenu makeIncidence(?rowsOfKnots,?numberOfPoints)

function solvePotential (A,i,j)
## Solve the potential problem of resistance in a graph.
## A is an incidence matrix.
## Return the potential u for the nodes in A,
## such that u[i]=1, u[j]=-1, and the flow
## to each knot is equal to the flow from the knot,
## and the flow from i to j is (u[i]-u[j])*A[i,j].
## see: makeIncidence
	B=setdiag(A,0,-sum(A)');
	B[i]=0; B[i,i]=1;
	B[j]=0; B[j,j]=1;
	v=zeros(cols(B),1); v[i]=1; v[j]=-1;
	u=B\v; f=sum(A[i])*u[i]-A[i].u;
	return {u,2/f}
endfunction

addmenu solvePotential (?incidenceMatrix,?fromNumber,?toNumber)

function solvePotentialX (A:cpx, i:index ,j:index)
## Solve the potential problem of resistance in a graph.
## This functions uses the conjugate gradient method.
## A is a compressed incidence matrix.
## Return the potential u for the nodes in A,
## such that u[i]=1, u[j]=-1, and the flow
## to each knot is equal to the flow from the knot,
## and the flow from i to j is (u[i]-u[j])*A[i,j].
## see: makeIncidence
	n=size(A)[1];
	b=ones(n,1); f=-cpxmult(A,b);
	h=1:n; B=cpxset(A,h'|h'|f);
	B=cpxset(B,i|h'|0);
	B=cpxset(B,[i,i,1]);
	B=cpxset(B,j|h'|0);
	B=cpxset(B,[j,j,1]);
	v=zeros(n,1); v[i]=1; v[j]=-1;
	u=cpxfit(B,v); 
	f=(-f[i])*u[i]-cpxmult(A,u)[i];
	return {u,2/f}
endfunction

addmenu makeRectangleIncidence(?rows,?columns)

function makeRectangleIncidence (n:index,m:index)
## Make the incidence matrix of a rectangle grid.
## see: makeIncidence
	K=zeros(n*(m-1)+m*(n-1),3);
	k=1;
	for i=1 to n;
		for j=1 to m-1;
		K[k,1]=(i-1)*m+j; K[k,2]=(i-1)*m+j+1; K[k,3]=1;
		k=k+1;
		end;
	end;
	for i=1 to n-1;
		for j=1 to m;
		K[k,1]=(i-1)*m+j; K[k,2]=i*m+j; K[k,3]=1;
		k=k+1;
		end;
	end;
	return makeIncidence(K,n*m);
endfunction

function makeRectangleX (n:index,m:index)
## Make the incidence matrix of a rectangle grid in compact form.
## see: makeRectangleIncidence
	K=zeros(n*(m-1)+m*(n-1),3);
	k=1;
	for i=1 to n;
		for j=1 to m-1;
		K[k,1]=(i-1)*m+j; K[k,2]=(i-1)*m+j+1; K[k,3]=1;
		k=k+1;
		end;
	end;
	for i=1 to n-1;
		for j=1 to m;
		K[k,1]=(i-1)*m+j; K[k,2]=i*m+j; K[k,3]=1;
		k=k+1;
		end;
	end;
	H=cpxzeros([n*m,n*m]);
	H=cpxset(H,K);
	H=cpxset(H,K[:,[2,1,3]]);
	return H;
endfunction

addmenu makeRectangleX(?rows,?columns)

function computeChain (v)
## Special resistance problem for a parallel chain.
## Load Restistanc.en for a demo
	n=cols(v);
	x=v[n];
	for i=n-1 to 1 step -1;
		x=1/(1/v[i]+1/(2+x));
	end
	return x;
endfunction

function getGraph (P:real, M:real)
## Draw the points P with incidence matrix M.
##
## P contains lines of the points x[i],y[i].
## and M contains the incidence matrix of the graph.
## M must be a 0-1 matrix.
##
## The function returns {x,y} with lines
##   x[i],x[j] and y[i],y[j]
## for connected points i and j.
	m=(totalsum(M)-sum(diag(M,0)))/2;
	x=zeros(m,2);
	y=zeros(m,2);
	n=cols(M);
	k=1;
	for i=2 to n;
		for j=i+1 to n;
	  		if M[i,j] then x[k]=[P[i,1],P[j,1]]; y[k]=[P[i,2],P[j,2]]; k=k+1; endif;
		end;
	end;
	return {x,y};
endfunction

