// Non-uniform Random Generators

comment
Based on Julia code by John D. Cook.
endcomment

function rand ()
## Alias to random()
	return random()
endfunction

function randuniform (a, b)
## return a uniform random sample from the interval (a, b)
    return a + rand()*(b - a)
endfunction

function randnormal (mean, stdev)
## return a random sample from a normal (Gaussian) distribution
    if stdev <= 0.0 then
        error("standard deviation must be positive")
    endif
    u1 = rand();
    u2 = rand();
    r = sqrt( -2.0*log(u1) );
    theta = 2.0*pi*u2;
    return mean + stdev*r*sin(theta)
endfunction

function randexponential (mean)
## return a random sample from an exponential distribution
    if mean <= 0.0 then
        error("mean must be positive")
    endif
    return -mean*log(rand())
endfunction

function randgamma (shape, scale)
## return a random sample from a gamma distribution
    if shape <= 0.0 then
        error("Shape parameter must be positive")
    endif
    if scale <= 0.0 then
        error("Scale parameter must be positive")
    endif

## Implementation based on "A Simple Method for Generating Gamma Variables"
## by George Marsaglia and Wai Wan Tsang.
## ACM Transactions on Mathematical Software
## Vol 26, No 3, September 2000, pages 363-372.

    if shape >= 1.0 then
        d = shape - 1.0/3.0;
        c = 1.0/sqrt(9.0*d);
        repeat
            x = randnormal(0, 1);
            v = 1.0 + c*x;
            while v <= 0.0
                x = randnormal(0, 1);
                v = 1.0 + c*x;
            end
            v = v*v*v;
            u = rand();
            xsq = x*x;
            if u < 1.0 -.0331*xsq*xsq or log(u) < 0.5*xsq + d*(1.0 - v + log(v)) then
                return scale*d*v
            endif
        end
    else
        g = randgamma(shape+1.0, 1.0);
        w = rand();
        return scale*g*pow(w, 1.0/shape)
    endif
endfunction

function randchisquare(dof)
## return a random sample from a chi square distribution
## with the specified degrees of freedom
    return randgamma(0.5, 2.0*dof)
end

function randinversegamma(shape, scale)
## return a random sample from an inverse gamma random variable
    ## If X is gamma(shape, scale) then
    ## 1/Y is inverse gamma(shape, 1/scale)
    return 1.0 / randgamma(shape, 1.0 / scale)
endfunction

function randweibull(shape, scale)
## return a sample from a Weibull distribution
    if shape <= 0.0 then
        error("Shape parameter must be positive")
    endif
    if scale <= 0.0 then
        error("Scale parameter must be positive")
    endif
    return scale * pow(-log(rand()), 1.0 / shape)
endfunction

function randcauchy(median, scale)
## return a random sample from a Cauchy distribution
    if scale <= 0.0 then
        error("Scale parameter must be positive")
    endif
    p = rand();
    return median + scale*tan(pi*(p - 0.5))
endfunction

function randstudentt(dof)
## return a random sample from a Student t distribution
    if dof <= 0 then
        error("Degrees of freedom must be positive")
    endif

## See Seminumerical Algorithms by Knuth
    y1 = randnormal(0, 1);
    y2 = randchisquare(dof);
    return y1 / sqrt(y2 / dof)
endfunction

function randlaplace(mean, scale)
## return a random sample from a Laplace distribution
## The Laplace distribution is also known as the double exponential distribution.
    if scale <= 0.0 then
        error("Scale parameter must be positive")
    endif
    u = rand();
    if u < 0.5 then
        return mean + scale*log(2.0*u);
    else
        return mean - scale*log(2*(1-u));
    endif
endfunction

function randlognormal(mu, sigma)
## return a random sample from a log-normal distribution
    return exp(randnormal(mu, sigma))
endfunction

function randbeta(a, b)
## return a random sample from a beta distribution
    if a <= 0 || b <= 0 then
        error("Beta parameters must be positive")
    endif

## There are more efficient methods for generating beta samples.
## However such methods are a little more efficient and much more complicated.
## For an explanation of why the following method works, see
## http://www.johndcook.com/distributionchart.html#gammabeta

    u = randgamma(a, 1.0);
    v = randgamma(b, 1.0);
    return u / (u + v)
endfunction

function randmatrix (n:index,m:index,f:string)
## Apply the random generator to generate a matrix.
	M=zeros(n,m);
	loop 1 to prod(size(M)); 
		M{#} = f(args());
	end;
	return M
endfunction
