// Large and Sparse Matrices

comment
Routines for large, thin matrices, and for incidence matrices. 
endcomment

// See also the functions in the Euler core.

// To solve large, sparse equations the CG-method implemented in cgX or
// cpxfit is the method of choice. The function cpxfit uses the normal
// equation to fit Ax-b. The Gau-Seidel method seidelX is an
// alternative for diagonal dominant matrices.

// * Gau-Seidel

function seidelX (H:cpx, b:real column, x:column=0, ..
	om:real positive scalar=1.2)
## Solve Hx=b using the Gau-Seidel method for compressed matrices H.
##
## The Gau-Seidel method  with Relaxation is an iterative method,
## which converges for all positive definite matrices. For large
## matrices, it may work well. However, the conjugate gradient method
## "cgX" is the method of choice.
##
## H must be diagonal dominant, at least not have 0 on the diagonal.
## om is the relaxation parameter between 1 and 2. x is start value
## (automatic if 0). It is possible to specify the accuracy with
## eps=...
##
## H : compressed matrix (nxm)
## b : column vector (mx1)
## om : optional relaxation coefficient
##
## Returns the solution x, a column vector.
##
## See: cgX, seidel
	if isvar("eps") then localepsilon(eps); endif;
	if x==0 then x=zeros(size(b)); endif;
	repeat
		xn=cpxseidel(H,b,x,om);
		if all(xn~=x) then return xn; endif;
		x=xn;
	end;
endfunction

function cgX (H:cpx, b:real column, x0:real column=none, f:index=10)
## Conjugate gradient method to solve Hx=b for compressed H.
##
## This is the method of choice for large, sparse matrices. In most
## cases, it will work well, fast, and accurate.
##
## H must be positive definite. Use cpxfit, if it is not.
##
## The accuarcy can be controlled with an additional parameter
## eps. The algorithm stops, when the error gets smaller then eps, or
## after f*n iterations, if the error gets larger. x0 is an optional
## start vector.
##
## H : compressed matrix (nxm)
## b : column vector (mx1)
## x0 : optional start point (mx1)
## f : number of steps, when the method should be restarted
##
## See: cpxfit, cg, cgXnormal
	if isvar("eps") then localepsilon(eps); endif;
	n=cols(H);
	if x0==none then x=zeros(size(b));
	else; x=x0;
	endif;
	loop 1 to 10
		r=b-cpxmult(H,x); p=r; fehler=r'.r;
		loop 1 to f*n
			if sqrt(fehler)~=0 then return x; endif;
			Hp=cpxmult(H,p);
			a=fehler/(p'.Hp);
			x=x+a*p;
			rn=r-a*Hp;
			fehlerneu=rn'.rn;
			p=rn+fehlerneu/fehler*p;
			r=rn; fehler=fehlerneu;
		end;
	end;
	return x;
endfunction

function cgXnormal (H:cpx, Ht:cpx, b:real column, x0:real column=none, ..
	f:index=10)
## Conjugate gradient method to solve Ht.H.x=Ht.b for compressed H.
##
## This algorithm is used by cpxfit to solve the normal equation
## H'Hx=H'b, which minimizes |Hx-b|, i.e., to find an optimal solution
## of an linear system with more equations than unknowns.
##
## Stops, when the error gets smaller then eps, or after f*n
## iterations, when the error gets larger. x0 is an optional start
## vector.
##
## H : compressed matrix (nxm)
## Ht : compressed matrix (mxn)
## b : column vector (mx1)
## x0 : optional start (mx1)
## f : number of steps, when the method should be restarted
##
## See: cgX, cpxfit
	if isvar("eps") then localepsilon(eps); endif;
	n=cols(H);
	if x0==none then x=zeros(size(b));
	else; x=x0;
	endif;
	loop 1 to 10
		r=b-cpxmult(H,cpxmult(Ht,x)); p=r; fehler=r'.r;
		loop 1 to f*n
			if sqrt(fehler)~=0 then return x; endif;
			Hp=cpxmult(H,cpxmult(Ht,p));
			a=fehler/(p'.Hp);
			x=x+a*p;
			rn=r-a*Hp;
			fehlerneu=rn'.rn;
			p=rn+fehlerneu/fehler*p;
			r=rn; fehler=fehlerneu;
		end;
	end;
	return x;
endfunction

function cpxfit (H:cpx, b:real column, f:index=10)
## Minimize |Hx-b| for a compressed matrix H.
##
## This function uses the conjugate gradient method to solve the
## normal equation H'Hx=H'b for sparse compressed matrices H.
##
## H : compressed matrix (nxm)
## b : column vector (mx1)
## f : number of steps, when the method should be restarted
##
## See: fit, fitnormal, svdsolve
	if isvar("eps") then localepsilon(eps); endif;
	Ht=cpxtranspose(H);
	return cgXnormal(Ht,H,cpxmult(Ht,b),f=f,eps=epsilon());
endfunction

// * Utility Functions

// Some utility functions for the compressed matrix format.

function cpxsetdiag (R:cpx, k:integer scalar, d:real vector)
## Set the k-th diagonal of the compressed matrix R to the value d.
##
## k=0 is the main diagonal, k=-1 the diagonal below, and k=1 the
## diagonal above.
##
## Note that this function does not change R, but returns a new matrix
## with the changes.
##
## See: setdiag
	s=size(R);
	if k>=0 and k<s[2] then
		i=1:min(s[1],s[2]-k);
		j=k+(1:cols(i));
		if rows(d)>1 then d=d[1:cols(i)]; endif;
		return cpxset(R,i'|j'|d');
	else if k<0 && -k<s[1] then
		j=1:min(s[2],s[1]+k);
		i=-k+(1:cols(j));
		if rows(d)>1 then d=d[1:cols(i)]; endif;
		return cpxset(R,i'|j'|d');		
	else return R;
	endif;
endfunction

function cpxmultrows (c:real column, A:cpx)
## Multiply the i-th row of the compressed matrix A by c[i].
##
## Note that this function does not change R, but returns a new matrix
## with the changes.
##
## c : column vector (nx1)
## A : compressed matrix (nxm)
##
## See: cpxsetrow
	n=size(A)[1];
	D=cpxzeros([n,n]); D=cpxsetdiag(D,0,c');
	return cpxmult(D,A);
endfunction

function cpxsetrow (A:cpx, i:index, r:real vector)
## Set the i-th row of the compressed matrix A to r.
##
## Note that this function does not change R, but returns a new matrix
## with the changes.
##
## A : compressed matrix (nxm)
## i : index
## r : row vector (1xm)
##
## See: cpxnultrows
	s=size(A);
	j=1:s[2];
	return cpxset(A,(i|j')|r');
endfunction

function cpxsetcolumn (A:cpx, j:index, c:real column)
## Set the j-th column of the compressed matrix A to c.
##
## Note that this function does not change R, but returns a new matrix
## with the changes.
##
## A : compressed matrix (nxm)
## j : integer
## c : column vector (mx1)
##
## See: cpxsetrow
	s=size(A);
	i=1:s[1];
	return cpxset(A,(i'|j)|c);
endfunction

function cpxsum (A:cpx)
## The sums of all rows of the compressed matrix A.
##
## See: sum
	n=size(A)[2];
	return cpxmult(A,ones(n,1));
endfunction

// * Incidence Matrices

function rectangleIncidenceX (n:index, m:index)
## Incidence matrix of a rectangle grid in compact form.
##
## The incidence matrix of a graph is the matrix H, such that H(i,j)
## contains 1, if node i is connected to node j. In this function, the
## graph consists of the points of a rectangle, and the edges connect
## adjactant points. The points in the rectangle are numbered row by
## row.
##
## Returns a compressed nm x nm matrix.
##
## See: cpxset
	K=zeros(n*(m-1)+m*(n-1),3);
	k=1;
	for i=1 to n;
		for j=1 to m-1;
		K[k,1]=(i-1)*m+j; K[k,2]=(i-1)*m+j+1; K[k,3]=1;
		k=k+1;
		end;
	end;
	for i=1 to n-1;
		for j=1 to m;
		K[k,1]=(i-1)*m+j; K[k,2]=i*m+j; K[k,3]=1;
		k=k+1;
		end;
	end;
	H=cpxzeros([n*m,n*m]);
	H=cpxset(H,K);
	H=cpxset(H,K[:,[2,1,3]]);
	return H;
endfunction
