// Exact Computation

comment
Exact arithmetic.
endcomment

// Euler has a long accumulator, which can compute the scalar product,
// and the residuum of a linear equation exactly. On this basis, Euler
// implements a residuum iteration to solve linear systems. This is
// used to invert matrices more exactly, or to evaluate polynomials.

// See: ../Programs/14 - Introduction to Exact Computation

// Residuum iterations uses the long accumulator of Euler to compute
// an exact scalar product.

// See: eulercore.html#Numerical_Algorithms | Accumulator Functions

// * Linear Systems

function xlgs (A:complex, b:complex, n:integer=20)
## Compute a more exact solution of Ax=b using residuum iteration.
## n is the number of residual iterations.
##
## You can specify the relative accuracy with eps=... This epsilon is
## used to determine, if the algorithm should stop.
##
## See: ilgs
	if (isvar("eps")); localepsilon(eps); endif;
	{LU,rows,c,d}=lu(A); LU=LU[rows];
	if d==0 then v=A\b;
	else v=lusolve(LU,b[rows]);
	endif;
	loop 1 to n;
		r=residuum(A,v,b);
		if d==0 then vn=v-A\r;
		else vn=v-lusolve(LU,r[rows]);
		endif;
		if all(vn~=v) then break; endif;
		v=vn;
	end;
	return vn;
endfunction

function xinv (A:complex, n:integer=20)
## Cumpute the invers of A using residuum iteration.
##
## You can specify the relative accuracy with eps=... as last
## parameter. Addtionally, a maximal number of iteration n can be
## set.
##
## See: xlgs
	if (isvar("eps")); return xlgs(A,id(cols(A)),eps=eps);
	else (isvar("eps")); return xlgs(A,id(cols(A)));
	endif
endfunction

function xlusolve (A:complex, b:complex, n:integer=20)
## Compute the solution of Ax=b for L- or U-matrices A.
##
## Works for lower triangle matrices with diagonal 1, or for upper
## triangle matrices. The function is just a faster version of xlgs.
##
## You can specify the relative accuracy with eps=... as last
## parameter. Addtionally, a maximal number of iteration n can be
## set.
##
## See: xlgs, lu, lusolve
	if (isvar("eps")); localepsilon(eps); endif;
	v=lusolve(A,b);
	loop 1 to n;
		r=residuum(A,v,b);
		vn=v-lusolve(A,r);
		if all(vn~=v) then break; endif;
		v=vn;
	end;
	return vn;
endfunction

// * Polynomials

function xpolyval (p:complex vector, t:complex, n:integer=20)
## Evaluate the polynomial at values t using residuum iteration.
##
## Alias to xevalpoly.
##
## See: xevalpoly
	return xevalpoly (p,t,n);
endfunction

function xevalpoly (t:complex, p:complex vector, n:integer=20)
## Evaluate the polynomial at values t using residuum iteration.
##
## You can specify the relative accuracy with eps=... as last
## parameter. Addtionally, a maximal number of iteration n can be
## set.
##
## See: evalpoly, xlgs
	if (isvar("eps")); localepsilon(eps); endif;
	s=t; si=size(t);
	A=id(cols(p));
	b=flipx(p)';
	loop 1 to prod(si);
		A=setdiag(A,-1,-t{#});
		v=lusolve(A,b);
		loop 1 to n;
			r=residuum(A,v,b);
			vn=v-lusolve(A,r);
			if all(vn~=v) then break; endif;
			v=vn;
		end;
		s{#}=v[cols(p)];
	end;
	return s;
endfunction

// * Eigenvalues

function xeigenvalue (a,l,x=none)
## Returns an improved eigenvalue of A, starting with the approximation l.
##
## l must be close to a simple eigenvalue, and x close to an
## eigenvector, if x is not 0. This is the inverse iteration due to
## Wielandt.
##
## Returns the eigenvalue and the eigenvector.
##
## >H=hilbert(10);
## >lambda=jacobi(H)[1]
##  255023613680
## >lambda=lambda+1000;
## >x=eigenspace(H,lambda);
## >norm(H.x-lambda*x)
##  7445.4368515
## >{lambda,x}=xeigenvalue(H,lambda,x);
## >norm(H.x-lambda*x)
##  3.7570410926e-005
##
## See: eigenvalues
	l1=l;
	if x==none then x=kernel(a-l1*id(cols(a))); endif;
	if all(x~=0) then error("Eigenvalue generation failed!"); endif;
	repeat;
		eps=setepsilon(0); x=(a-l1*id(cols(a)))\x; setepsilon(eps);
		x=x/sqrt(x'.x);
		l2=residuum(x',residuum(a,x,0),0);
		if l1~=l2 then return {l2,x}; endif;
		l1=l2;
	end;
endfunction
