// Mathematical Functions

comment
Special Functions.
endcomment

// This file contains special functions, like hyperbolic functions,
// elliptic functions, beta functions etc. It does also contain some
// polynomial functions.

// * Hyperbolic Functions

function sinh (x:numerical)
## Computes (exp(x)-exp(-x))/2
	h=exp(x);
	return (h-1/h)/2;
endfunction

function cosh (x:numerical)
## Computes (exp(x)+exp(-x))/2
	h=exp(x);
	return (h+1/h)/2;
endfunction

function asinh (z:numerical)
## Computes log(z+sqrt(z^2+1))
	return log(z+sqrt(z*z+1))
endfunction

function acosh (z:numerical)
## Computes log(z+(z^2-1))
	return log(z+sqrt(z*z-1))
endfunction

function map sinc (z:numerical)
## Computes sin(x)/x
##
## Takes care of x~=0, returning 1 in this case.
## For intervals containting 0 the function fails.
	if z~=0 then return 1;
	else return sin(z)/z; 
	endif;
endfunction

function sec (x:numerical)
## Computes 1/cos(x)
	return 1/cos(x)
endfunction

function cosec (x:numerical)
## Computes 1/sin(x)
	return 1/sin(x)
endfunction

function cot (x:numerical)
## Computes 1/tan(x)
	return 1/tan(x)
endfunction

// * Polar Coordinates

function args polar
## polar(x,y), polar(x,y,z) compute polar coordinates.
##
## This function computes the polar coordinates in two or three
## dimensions.
## >{phi,r}=polar(1,2); degprint(phi), r,
##  6326'5.82''
##  2.2360679775
##
## Returns {phi,r} or {phi,psi,r}
##
## See: rect
	if argn==2 then return {atan2(arg1,arg2),sqrt(arg1*arg1+arg2*arg2)}; 
		elseif argn==3 then
		return {atan2(arg1,arg2),atan2(sqrt(arg1*arg1+arg2*arg2),arg3), ..
			sqrt(arg1*arg1+arg2*arg2+arg3*arg3)}
	else error("Illegal arguments for topolar");
	endif
endfunction

function args rect
## rect(phi,r), rect(phi,psi,r) compute rectangle coordinates.
##
## Computes rectangle coordinates in two or three dimensions.
##
## >{x,y}=rect(45,sqrt(2)); [x,y]
##  [ 1  1 ]
##
## Returns {x,y} or {x,y,z}
##
## See: polar
	if argn==2 then return {arg2*cos(arg1),arg2*sin(arg1)};
	elseif argn==3 then return {arg3*cos(arg1)*cos(arg2), ..
		arg3*sin(arg1)*cos(arg2),arg3*sin(arg2)};
	else error("Illegal argumtenst for rect");
	endif;
endfunction

// * Logarithms

function logbase (x:numerical, a:numerical)
## Computes the logarithm to base a
	return log(x)/log(a);
endfunction

function log10 (x:numerical)
## Computes the logarithm to base 10
	return log(x)/log(10)
endfunction

// * Chebychev Polynomials

function polydif (p:vector)
## Returns the polynomial p'.
##
## Polynomials are stored in Euler starting with the constant
## coefficient.
##
## >polydif([0,0,1])
##  [ 0  2 ]
	n=cols(p);
	if (n==1); return 0; endif;
	return p[2:n]*(1:n-1);
endfunction

function map cheb (x:complex, n:nonnegative integer)
## Computes the Chebyshev polynomial T_n(x).
##
## The functions used T(x,n) = cos(n*acos(x)), if possible. For x<-1
## and x>1, it uses T(x,n) = (w+1/w)/2, where w is the n-th power of
## the inverse Joukowski transform.
##
## >x=-1.1:0.01:1.1; y=x'; z=x+I*y;
## >plot2d(abs(cheb(z,5)),niveau=exp(0:10))
##
## See: chebpoly, chebrek
	if iscomplex(x) then
		return cos(n*acos(x));
	elseif x>1 then
		w=(x+sqrt(x^2-1))^n;
		return (w+1/w)/2;
	elseif x<-1 then
		signum=-mod(n,2)*2+1;
		w=(-x+sqrt(x^2-1))^n;
		return signum*(w+1/w)/2;
	else
		return cos(n*acos(x));
	endif;
endfunction

function chebrek (x:vector, n:nonnegative integer)
## Computes the Chebyshev polynomial via the recursion formula.
##
## >chebrek(-1:0.1:1,10)
##  [ 1  -0.2007474688  0.9884965888  -0.0998400512  -0.9884965888  -0.5
##  0.5623462912  0.9955225088  0.4284556288  -0.5388927488  -1
##  -0.5388927488  0.4284556288  0.9955225088  0.5623462912  -0.5
##  -0.9884965888  -0.0998400512  0.9884965888  -0.2007474688  1 ]
##
## Works for vector x and scalar n.
	if (n==0); return ones(size(x)); endif;
	if (n==1); return x; endif;
	z=ones(size(x))_x;
	loop 3 to n+1;
		z=z_(2*x*z[#-1]-z[#-2]);
	end;
	return z[n+1];
endfunction

function chebpoly (n:nonnegative integer)
## Computes the coefficients of the n-th chebyshev polynomial.
##
## >p=chebpoly(10)
##  [ -1  0  50  0  -400  0  1120  0  -1280  0  512 ]
## >plot2d("polyval(p,x)",-1,1);
##
## >x=-1.1:0.01:1.1; y=x'; z=x+I*y;
## >p=chebpoly(10); w=abs(polyval(p,z));
## >plot2d(log(w+0.2),niveau=0:10,>hue,>spectral)
##
## The function uses a loop.
	t0=[1]; t1=[0,1];
	if (n==0); return t0; endif;
	if (n==1); return t1; endif;
	loop 2 to n
		t=(0|2*t1[1:#])-(t0|[0,0]);
		t0=t1; t1=t;
	end;
	return t;
endfunction

function chebfit (xp:vector, yp:vector, n:nonnegative integer, ..
	a:real scalar=-1, b:real scalar=1)
## Fits Chebyshev polynomials on [a,b] to (xp,yp).
##
## Returns the vector of coefficients p(x) = sum a_[i] T(x,i)
##
## See: chebval
	return fit(cheb((xp'-(a+b)/2)/((b-a)/2),0:n),yp')';
endfunction

function map chebval (x:number; alpha:vector, ..
	a:real scalar=-1, b:real scalar=1)
## Evaluates sum a[i]*T(x,i).
##
## See: chebfit
	return sum(alpha*cheb((x-(a+b)/2)/((b-a)/2),0:cols(alpha)-1));
endfunction

// * Beta, Gamma and Elliptical Functions

function beta (a:numerical, b:numerical)
## Computes the beta function
	return gamma(a)*gamma(b)/gamma(a+b);
endfunction

function betai (x:real, a:real, b:real)
## Computes the incomplete beta function.
##
## See: betai1
	return map("betai1",x,a,b);
endfunction

gammaparams:=[0.99999999999980993, 676.5203681218851, ..
-1259.1392167224028, 771.32342877765313, ..
-176.61502916214059, 12.507343278686905, ..
-0.13857109526572012, 9.9843695780195716e-6, ..
1.5056327351493116e-7];

function map complexgamma (z:complex, p=gammaparams)
## Complex gamma function.
  if im(z)~=0 then 
  	if re(-z)>=0 && floor(re(-z))~=-z then return complex(1e20)
  	else return complex(_gamma(re(z)))
  	endif
  else
	  g=cols(p)-2;
	  if re(z)<0.5 then
	    return pi/(sin(pi*z)*gamma(1-z))
	  else
	    z:=z-1.0;
	    x:=p[1];
	    for i=2 to g+2;
	      x := x+p[i]/(z+i-1);
	    end;
	    t := z + (g + 0.5);
	    return sqrt(2*pi) * t^(z+0.5) * exp(-t) * x;
	  endif;
	endif
endfunction

function overwrite gamma (z:numerical)
## Gamma function in the complex plane.
##
## You can also use _gamma(x) for real gamma function directly. The
## function uses an iteration implemented in the Euler language.
##
## >x=-2:0.05:2; y=x'; z=x+I*y; w=log(abs(gamma(z)));
## >plot3d(x,y,w/totalmax(w),>contour,zoom=3);
##
## See: gamma
	if isreal(z) then return _gamma(z)
	else return complexgamma(z);
	endif;
endfunction

function map ellrf (x:real nonnegative scalar, ..
	y:real nonnegative scalar, z:real nonnegative scalar)
## Carlson's elliptic integral of the first kind RF (x; y; z). 
##
## The iteration is implemented in the Euler language.
##
## x, y, and z must be nonnegative, and at most one can be zero. 
	if isvar("eps") then localepsilon(eps); endif;
	repeat
		mu=(x+y+z)/3;
		xn=1-x/mu; yn=1-y/mu; zn=1-z/mu;
		if max(max(xn,yn),zn)~=0 then break endif;
		lambda=sqrt(x*y)+sqrt(x*z)+sqrt(y*z);
		x=(x+lambda)/4; y=(y+lambda)/4; z=(z+lambda)/4;
	end;
	return 1/sqrt(mu);
	e2=xn*yn-zn^2;
	e3=xn*yn*zn;
	return (1.0+(e2/24-0.1-3*e3/44)*e2+e3/14)/sqrt(mu);
endfunction

function map ellf (phi: real scalar, k: real scalar)
## Elliptic integral of the first kind F(phi,k)
	s=sin(phi);
	k=sqrt(k);
	return s*ellrf(cos(phi)^2,(1.0-s*k)*(1.0+s*k),1.0);
endfunction

function map ellrd (x:real nonnegative scalar, ..
	y:real nonnegative scalar, z:real nonnegative scalar)
## Carlson's elliptic integral of the second kind RD(x; y; z). 
##
## x and y must be nonnegative, and at most one can be zero. z must be
## positive.
##
## See: elle
	if isvar("eps") then localepsilon(eps); endif;
	sum=0.0;
	fac=1.0;
	repeat
		mu=0.2*(x+y+3.0*z);
		xn=(mu-x)/mu; yn=(mu-y)/mu; zn=(mu-z)/mu;
		if max(max(abs(xn),abs(yn)),abs(zn))~=0 then break endif;
		lambda=sqrt(x*y)+sqrt(x*z)+sqrt(y*z);
		sum=sum+fac/(sqrt(z)*(z+lambda));
		fac=0.25*fac;
		x=0.25*(x+lambda);
		y=0.25*(y+lambda);
		z=0.25*(z+lambda);
	end;
	ea=xn*yn;	
	eb=zn*zn;
	ec=ea-eb;
	ed=ea-6.0*eb;
	ee=ed+ec+ec;
	C1=(3.0/14.0);
	C2=(1.0/6.0);
	C3=(9.0/22.0);
	C4=(3.0/26.0);
	C5=(0.25*C3);
	C6=(1.5*C4);
	return 3.0*sum+fac*(1.0+ed*(-C1+C5*ed-C6*zn*ee) ..
		+zn*(C2*ee+zn*(-C3*ec+zn*C4*ea)))/(mu*sqrt(mu));
endfunction

function map elle (phi: real scalar, k: real scalar)
## Elliptic integral of the second kind E(phi,k).
##
## See: ellrd
	s=sin(phi);
	cc=cos(phi)^2;
	k=sqrt(k);
	q=(1.0-s*k)*(1.0+s*k);
	return s*(ellrf(cc,q,1.0)-((s*k)^2)*ellrd(cc,q,1.0)/3.0);
endfunction
