// Gauss Qudrature

comment
Gauss integration.
endcomment

// The file contains the gauss function, which integrates a function
// very effectively with 10 evalutions per interval. For less accuracy
// gauss5 can be used. There is an interval version of the Gau
// integration, which needs an expression, estimatin or function for
// the 20-th derivative.

// Gau integration uses the zeros of Legendre polynomials, which can
// also be generated using lengendre().

function legendre (n:index)
## Compute the coefficients of the n-th Legendre polynomial.
##
## This is used to compute the Gauss coefficients.
##
## See: gauss
	p0=1; p1=[0,1];
	loop 2 to n;
		p=((0|(2*#-1)*p1)-((#-1)*(p0|0|0)))/#;
		p0=p1; p1=p;
	end;
	return p1
endfunction

function gaussparam (n:index)
## Returns the knots and alphas of gauss integration at n points in [-1,1].
##
## Returns {gaussz,gaussa}.
##
## See: gauss, gauss10
	p=legendre(n);
	z=sort(re(polysolve(p)));
	Z=dup(z,n)^dup((0:n-1)',n);
	a=Z\(2/(1:n)'*mod(1:n,2)');
	return {z,a'}
endfunction

gaussz := [ ..
-9.7390652851717172e-0001,
-8.6506336668898451e-0001,
-6.7940956829902441e-0001,
-4.3339539412924719e-0001,
-1.4887433898163121e-0001,
 1.4887433898163121e-0001,
 4.3339539412924719e-0001,
 6.7940956829902440e-0001,
 8.6506336668898451e-0001,
 9.7390652851717172e-0001];
gaussa := [ ..
 6.6671344308688139e-0002,
 1.4945134915058059e-0001,
 2.1908636251598205e-0001,
 2.6926671930999635e-0001,
 2.9552422471475288e-0001,
 2.9552422471475287e-0001,
 2.6926671930999635e-0001,
 2.1908636251598205e-0001,
 1.4945134915058059e-0001,
 6.6671344308688137e-0002];

function map %mapexpression1 (x:scalar; expr:string)
	return expr(x);
endfunction

function %evalexpression (f:string, x:numerical, maps=0)
	if isfunction(f) then
		if maps then return map(f,x;args());
		else return f(x,args());
		endif;
	else
		if maps then return %mapexpression1(x,f);
		else return f(x);
	endif;
endfunction

function %gauss10 (f:string, a:real scalar, b:real scalar, ..
	maps:integer=0)
## Evaluates the gauss integration with 10 knots an [a,b].
##
## f is an expression in x, or a function in one scalar variable.
## Additional parameters after the semicolon are passed to f.
##
## See: gauss
	return gauss(f,a,b,maps;args());
endfunction

function gauss (f:string, a:real scalar, b:real scalar, n:index=1, ..
	maps:integer=0, xn=gaussz, al=gaussa)
## Gauss integration with 10 knots and n subintervals.
##
## This function is exact for polynomials up to degree 19, even for
## one sub-interval. For other functions, it may be necessary to
## specify n subintervals.
##
## f is an expression in x, or a function in x. Additional parameters
## after the semicolon are passed to f.
##
## maps : Map an expression to the arguments (1 or 0). Functions are
##        always mapped.
##
## See: gauss10, gauss5, simpson, integrate, romberg
	h=(b-a)/n/2;
	if n==1 then
		x=a+h;	
	else
		x=linspace(a+h,b-h,n-1);
	endif;
	X=x-(h*xn)';
	return sum(al.%evalexpression(f,X,maps;args()))*h
endfunction

function igauss (expr:string, a:real, b:real, n:index=1, ..
	dexpr20:string=none, dexpr20est:real=none, xn=gaussz, al=gaussa)
## Compute an interval incluseion for an integral
##
## This works only for expressions, unless you provide an estimate for
## the 20-th derivative. The function will compute this derivative
## with Maxima otherwise.
##
## Optional:
##
## dexpr20: Expression or function for the 20-th derivative
## dexpr20est: Interval inclusion of the 20-th derivative 
##
## See: gauss, gauss5
	h=(b-a)/n/2;
	if n==1 then
		x=a+h;	
	else
		x=linspace(a+h,b-h,n-1);
	endif;
	X=~x-(h*xn)'~;
	res=sum(al.expr(X;args()))*h;
	if dexpr20est==none then
		if dexpr20==none then 
			dexpr20=mxm("diff(@expr,x,20)"); 
		endif
		dexpr20est=ieval(dexpr20,~a,b~,10*n);
	endif;
	gausserror=2.925590330737589e-006;
	return res+dexpr20est*((b-a)/(2*n))^21*gausserror/20!;
endfunction

gauss5z := [ ..
-0.8611363115940526, ..
0.8611363115940526, ..
-0.3399810435848562, ..
0.3399810435848562];

gauss5a := [ ..
0.3478548451374539, ..
0.3478548451374539, ..
0.6521451548625461, ..
0.6521451548625461];

function gauss5 (f:string, a:real scalar, b:real scalar, n:index=1, ..
	maps:integer=0, xn=gauss5z, al=gauss5a)
## Gauss integration with 4 knots and n subintervals
##
## This is exact for polynomials to degree 7 (even for n=1).
## The results are usually much better than the Simpson method
## with only twice as many function evaluations.
##
## See: gauss, simpson, integrate, romberg
	h=(b-a)/n/2;
	if n==1 then
		x=a+h;	
	else
		x=linspace(a+h,b-h,n-1);
	endif;
	X=x-(h*xn)';
	return sum(al.%evalexpression(f,X,maps;args()))*h
endfunction


function %evalfxy (x,expr,y)
	return evaluate(expr);
endfunction

function map %gaussinner(y;f,a,b,n=1)
	if isfunction(f) then
		return gauss(f,a,b,n;y)
	else
		return gauss("%evalfxy",a,b,n;f,y)
	endif;
endfunction

function gaussfxy (f:string, a:real scalar, b:real scalar, ..
	c:real scalar, d:real scalar, n:index=1)
## Computes the double integral of f on [a,b]x[c,d].
##
## This function uses Gauss intergration for the inner and the outer
## integral. The function is exact for polynomials in x and y up to
## degree 19.
##
## f must be a functon f(x,y) or an expression of x and y.
##
## See: gauss
	return gauss("%gaussinner",c,d,n;f,a,b,n)
endfunction
