// Interval Solvers and Guaranteed Solutions

comment
Algorihms for interval solvers, and guaranteed inclusions.
endcomment

// * Evaluation

function ieval (f:string, x:interval scalar, n:integer=10)
## Better evaluation of the expression in f for the interval x.
##
## The interval is split into sub-intervals for more accuracy.
##
## See: mxmieval, ievalder
	if n==1 then
		return f(x);
	else
		ti=splitinterval(x,n);
		s=f(ti[1]);
		if isreal(s) then s=~s,s~; endif;
		loop 2 to n
			s=s||f(ti[#]);
		end;
		return s;
	endif;
endfunction

function ievalder (f:string, fd:string, xi:interval scalar, n:integer=10)
## Better evaluation of the expression in f for the interval x.
##
## The derivative is used to improve the interval accuracy.
## The interval is split into sub-intervals for more accuracy.
##
## See: ieval, mxmieval
	if n==1 then
		xm=~middle(xi)~;
		der=fd(x=xi);
		if 0 << der then
			return f(x=xm)+(xi-xm)*der && f(x=xi);
		else
			return f(x=~left(xi)~) || f(x=~right(xi)~);
		endif;
	else
		ti=splitinterval(xi,n);
		s=ievalder(f,fd,ti[1],1);
		loop 2 to n
			s=s||ievalder(f,fd,ti[#],1);
		end;
		return s;
	endif;
endfunction

// * Differential Equations

function idgl (f:string, x:real vector, y0:interval scalar)
## Guaranteed inclusion of y'=f(t,y0;...) at points t with y(t[1])=y0. 
##
## This is a quick inclusion for a differential equation, which avoids
## the use of any Taylor series. The inclusion is not narrow, however.
## The function uses a simple Euler method.
##
## The result is an interval vector of values. 
##
## See: mxmidgl, idglder
	n=length(x);
	y=~zeros(1,n)~; y[1]=y0;
	loop 1 to n-1;
		m=f(x[#],y[#],args());
		i=~x[#],x[#+1]~;
		d=diameter(i);
		repeat
			J=y[#]+m*~0,d~;
			m=expand(f(i,J,args()),6/5);
			y2=y[#]+m*d;
			if all(y2 <<= J) then break; endif;
		end;
		y[#+1]=y2;
	end;
	return y;
endfunction

function idglder (f:string, fx:string, fy:string, x:real vector, ..
	y0:interval scalar)
## Guaranteed inclusion the solution of y'=f(t,y0;...) at t with
## y(t[1])=y0.
##
## This function needs the partial derivatives of f to x and y.
##
## The result is an interval vector of values.
##
## f, fx and fy are functions in f(x,y), or expressions of x and y.
## Additional arguments are passed to the functions.
##
## See: mxmidgl
	n=length(x);
	y=~zeros(1,n)~; y[1]=y0;
	loop 1 to n-1;
		i=~x[#],x[#+1]~;
		d=diameter(i);
		JT=y[#]+f(x[#],y[#],args())*~0,d~;
		J=expand(JT,3/2);
		repeat
			J1=JT+~0,d~^2*(fx(i,J,args())+fy(i,J;args())*f(i,J,args()))/2;
			if all(J1 <<= J) then break; endif;
			J=expand(J,3/2);
		end;
		y[#+1]=y[#]+f(x[#],y[#],args())*~d~+ ..
				~0,d~^2*(fx(i,J1,args())+fy(i,J1,args())*f(i,J1,args()))/2;
	end;
	return y;
endfunction

function isimpson (f:string, der:string, a:number, b:number, ..
	n:index=50)
## Interval Simpson integral of f from a to b.
##
## This function uses the Simpson method and its error estimate to get
## guaranteed inclusions of integrals.
##
## f : expression (must map to arguments and work for intervals)
## der : expression for fourth derivative (like f)
## a,b : interval bounds
## n : number of subintervals
##
## See: mxmisimpson
	a=~a~; b=~b~;
	m=2*n;
	ff=4-mod(1:m+1,2)*2; ff[1]=1; ff[m+1]=1;
	i=~0:m~; t=i/m*a+(m-i)/m*b;
	simps=sum(f(t)*ff)/3*(b-a)/m;
	t=splitinterval(a||b,n);
	error=sum(der(t)*ones(size(t)))*((b-a)/m)^5/90;
	return simps-error
endfunction

// * Linear Algebra

function ilgs (A:interval, b:interval, R="", steps=100)
## Guaranteed interval inclusion for the solution of A.x=b.
##
## This function uses an interval agorithm, and an exact residuum
## calculation. If the algorithm succeeds, the result is a guaranteed
## inclusion for the solution of the linear system. Note that the
## algorithm can only work for regular A, or interval matrices not
## containing singular A.
##
## A and b may be of interval or real type.
##
## The optional R is a provided pseudo inverse to A.
##
## See: xlgs
	n=cols(A);
	count=1;
	if R=="" then
		A1=middle(A); R=inv(A1');
		repeat;
			M=residuum(A1',R,id(n));
			rho=max(sum(abs(M))');
			if rho<1; break; endif;
			R=R-A1'\M;
		count=count+1;
		if (count>steps); error("Could not find a pseudo invers."); endif;
		end;
		R=R';
	endif;
	M=~-residuum(R,A,id(n))~;
	rho=right(max(sum(abs(M))'));
	if (rho>=1); error("Pseudo inverse not good enough."); endif;
	f=~-residuum(R,b,0)~;
	if argn()==2 then x=middle(A)\middle(b); else x=residuum(R,b,0); endif;
	if isreal(x) then xn=residuum(M,~x,x~,f); else xn=residuum(M,x,f); endif;
	x=expand(x,max((right(max(abs(xn-x)'))/(1-rho))'));
	count=1;
	repeat;
		xn=residuum(M,x,f)&&x;
		if not all(xn<<x) then break; endif;
		if count>steps then error("Interval inclusion failed."); endif;
		x=xn;
	end;
	return x;
endfunction

function iinv (A:interval)
## Guaranteed interval inverse of the matrix A.
##
## See: inv, xinv
	return ilgs(A,id(cols(A)));
endfunction

function ievalpoly (t:interval, p:interval vector)
## Guaranteed evaluation of a polynomial p(t).
##
## p contains the coefficients of a polynomial. Euler stores
## polynomials starting with the constant coefficient.
##
## See: polyval, xpolyval
	s=t; si=size(t);
	A=id(cols(p));
	b=flipx(p)';
	loop 1 to prod(si);
		A=setdiag(A,-1,-t{#});
		if isreal(A) then v=ilgs(A,b,lusolve(A,id(cols(p))));
		else v=ilgs(A,b);
		endif;
		s{#}=v[cols(p)];
	end;
	return s;
endfunction

function ipolyval (p:interval vector, t:interval)
## Guaranteed evaluation of a polynomial p(t).
##
## See: ievalpoly
	return ievalpoly(t,p);
endfunction

// * Guaranteed Inclusions for Equations

function ibisect (f:string, a:scalar, b:scalar=none, y:scalar=0)
## Interval bisection algorithm to solve f(x)=y
##
## See: bisect, inewton
    if b==none then b=right(a); a=left(a); endif;
	if f(~b,b~,args())<y then
		b1=a; a1=b;
		if not f(~a,a~,args())>y then 
			error("Possibly no zero in interval"); 
		endif;
	else
		a1=a; b1=b;
		if not f(~a,a~,args())<y then 
			error("Possibly no zero in interval"); 
		endif;
	endif;
	repeat
		m=(a1+b1)/2;
		fy=f(~m,m~,args());
		if fy>y then 
			b1=m; 
		elseif fy<y then 
			a1=m;
		else
			return ~a1,a1~||~b1,b1~
		endif;
	end;
endfunction

function inewton (f:string, df:string , xi: interval, yi:real scalar="", y=0)
## Guaranteed interval inclusion of the zero of f.
##
## df must compute an inclusion of the derivative of f for intervals
## x. f and df must be functions of one scalar variable, or
## expressions in x. Additional parameters after the semicolon are
## passed to both functions.
##
## The initial interval x must already contain a zero. If x is a
## point, and not an interval, the function tries to get an initial
## interval with the usual Newton method.
##
## Returns {x0,f}: the solution and a flag, if the solution is
## guaranteed.
##
## See: inewton2, mxminewton, inewton2
	if !(yi=="") then x=~xi,yi~;
	else x=xi;
	endif;
	if !isinterval(x);
		x=expand(~newton(f,df,x,y;args())~,1000);
	endif
	valid=0;
	repeat
		m=middle(x);
		a=f(~m,m~,args())-y;
		b=df(x,args());
		xnew=(m-a/b);
		if !valid && (xnew << x); valid=1; endif;
		xnew=xnew && x;
		if ! (xnew << x); return {x,valid}; endif;
		x=xnew;
	end;
endfunction

function inewton2 (f:string, Df:string, x:interval, check:integer=false)
## Guaranteed inclusion of the zero of f, a function of several parameters.
##
## Works like newton2, starting from a interval vector x which already
## contains a solution. If x is no interval, the function tries to
## find such an interval.
##
## f and Df must be a function of a row vector x, or an expression in
## x. f must return a row vector, and Df the Jacobi matrix of f.
##
## Returns {x,valid}.
##
## If check is false, the result is not checked for a guaranteed
## inclusion. In this case the return value of valid can be checked
## to learn, if the inclusion is a guaranteed inclusion. If checked
## is true valid=0 will throw an error exception.
##
## See: newton2, inewton 
	if !isinterval(x); 
		x=expand(~newton2(f,Df,x;args())~,1000);
	endif;
	valid=0;
	y=middle(x)';
// Compute an approximative inverse of Df(x)
	R=inv(Df(y',args()));
	yi=x';
	repeat
// Compute xm-R.f(xm)+(R.Df(x)-I).(x-xm)
// where xm is the midpoint of x
		y=middle(yi);
		M=~residuum(R,Df(yi',args()),id(cols(x)))~;
		yn=(y-R.f(~y,y~',args())'+M.(yi-y));
		if !valid and all(yn<<yi) then valid=1; endif;
		if any(!intersects(yn,yi)) then 
			if check then
					error("Nonempty intersection!");
			endif;
			return {yn,0}; 
		endif;
		yn=yn&&yi;
		if not all(yn<<yi) then
			if check and not valid then
					error("No guaranteed inclusion found!");
			endif;
			return {yn',valid}; 
		endif;
		yi=yn;
	end;
endfunction

// * Plot Intervals

function plotintervals (r)
## Adds plots of two dimensional intervals to a given plot.
##
## r is an nx2 vector of intervals.
##
## See: mxmibisectfxy
	rh=r';
	x=left(rh[1]); y=left(rh[2]);
	w=diameter(rh[1]); h=diameter(rh[2]);
	return plotbar(x,y,w,h);
endfunction
