// Elementary File Input and Output

comment
Formatted input and output to files. 
endcomment

// See also the Euler core functions for file input and output in
// @"eulercore".

// For files of unknown sources getmatrix or getvector should be used
// to read the numerical content. For output, writematrix is the
// easiest choice. 

// The functions work either for files opened with open(), or for a
// file name provided as a parameter, or for terminal output.

// For Latex, writelatex can be used.

function getmatrix (n:real scalar, m:real scalar, filename:string="")
## Read a real nxm Matrix from the file.
##
## If the filename is "", the file must be opened before. Else the
## file will open and close for read.
##
## Matrix values must be decimal fix point numbers with a dot (not a
## comma). They must be stored row after row.
##
## See: getvector, open
	if filename<>""; open(filename,"r"); endif;
	{v,N}=getvector(n*m);
	if filename<>""; close(); endif;
	if (N<n*m); error("Incorrect values in file"); endif;
	v=v[1:n*m];
	return redim(v,n,m);
endfunction

function readmatrix (filename:string="", max=1000, expand=0)
## Read a real matrix from the file.
##
## The matrix is stored row by row in lines, with an empty line ending
## the matrix. If the filename is not empty, the function will open
## the file for reading, and close it after reading. Otherwise, it
## will use the provided file name.
##
## max is the maximimal number of columns.
##
## If expand is true incomplete lines will filled with 0.
##
## See: writematrix, open, close, getmatrix
	if filename<>""; open(filename,"r"); endif;
	A=[];
	r=0;
	repeat;
		{s,n}=getvectorline(max);
		if n==0 then break; endif;
		if rows(A)==0 then 
			A=s; r=1;
		elseif n==cols(A) then 
			r=r+1;
			if r>rows(A) then A=A_A; endif;
			A[r]=s;
		elseif expand then
			if n>cols(A) then A=(A|zeros(rows(A),n-cols(A)))_s;
			else A=A_(s|zeros(1,cols(A)-n));
			endif;
			r=r+1;
		else
			"Error in line "|printf("%g",rows(A)+1),
			error("Matrix rows do not agree");
		endif;
	end;
	if filename<>""; close(); endif;
	return A[1:r,:];
endfunction

function getmatrixlines (filename:string="", max=1000, expand=0)
## Obsolete. Use readmatrix.
##
## See: readmatrix
	return readmatrix(filename,max,expand)
endfunction
	

function writematrix (v:real, filename:string="", format:string="%0.16f")
## Write a real nxm Matrix to the file row after row.
##
## If the filename is not "", the file will be created (beware!),
## opened, and closed after writing. If the filename is "", a file
## opened with open(filename,"w") will be used, or the terminal.
##
## See: open
	if typeof(v)<>2; error("Can write only real matrices"); endif;
	if filename<>""; open(filename,"w"); endif;
	f=format|" ";
	for i=1 to rows(v);
		for j=1 to cols(v);
			write(printf(f,v[i,j]));
		end;
		putchar(10);
	end;
	putchar(10);
	if filename<>""; close(); endif;
endfunction

function writef (x, f:string="%g", sep1:string=", ", sep2:string="; ")
## Print the vector x formatted with format f to the open file.
##
## The format is applied to real and imaginary part, or lower and
## upper part of the elements of x.
##
## See: printf, open
	s=size(x);
	loop 1 to s[1];
		i=#;
		loop 1 to s[2]
		if iscomplex(x);
			write(printf(f|sep1,re(x[i,#])));
			write(printf(f|sep2,im(x[i,#])));
		elseif isinterval(x);
			write(printf(f|sep1,left(x[i,#])));
			write(printf(f|sep2,right(x[i,#])));
		else write(printf(f|sep2,x[i,#]));
		endif;
		end;
		putchar(10);
	end;
endfunction

// * Formatted Output

function writeform (x, format="%0.16g")
## Converts x in the specified format to a string
	if isreal(x) then 
		return printf(format,x);
	elseif iscomplex(x) then
		return printf(format,re(x))|"+"|printf(format,im(x));|"i";
	elseif isinterval(x) then
		return "~"|printf(format,left(x))|","|printf(format,right(x))|"~";
	elseif isstring(x) then
		return x;
	endif;
	error("Cannot print this!");
endfunction

function writevar (x,s="",filename="",newlines=0)
## Write a variable Euler syntax. 
##
## If a filename is given, or a file is open, it will be used,
## otherwise the function will write to the command line.
##
## s : the name of the variable
## filename : a file to write to
## newlines : flag for newlines to separate elements in a row
	if filename<>"" then open(filename,"w"); endif;
	if s=="" then s=name(x); endif;
	si=size(x);
	if max(si)==1 then write(s|" = "|writeform(x)|";");
	else
		write(s|" = [ .."); putchar(10);
		for i=1 to si[1];
			for j=1 to si[2]-1;
				write(writeform(x[i,j])|", ");
				if newlines then putchar(10); endif;
			end;
			if i==si[1]; write(writeform(x[i,si[2]])|"];"); putchar(10);
			else; write(writeform(x[i,si[2]])|";"); putchar(10);
			endif;
		end;
	endif;
	putchar(10);
	if filename<>"" then close(); endif;
	return 0
endfunction

function writeformat (x,f="%g",sep1=", ",sep2=";", ..
	writereal=1,ibracket1="~",ibracket2="~",perline=100)
## Print the vector x formatted with format f.
##
## The format is applied to real and imaginary part, or lower and
## upper part of the elements of x. 
##
## writeral : Complex and interval values become two real values. Rows
## are separated with sep2.
##
## If a file is open, the function will write to that file.
##
## See: writevar
	s=size(x);
	loop 1 to s[1];
		i=#;
		count=1;
		loop 1 to s[2]
			sep=sep1;
			if #==s[2] then 
				if i==s[1] then sep=""; else sep=sep2; endif;
			endif;
			if iscomplex(x) then
				if writereal then
					write(printf(f|sep1,re(x[i,#])));
					write(printf(f|sep,im(x[i,#])));
				else
					write(printf(f,re(x[i,#])));
					if !(im(x[i,#])~=0) then
						write(printf("+"|f|"i"|sep,im(x[i,#])));
					else
						write(sep);
					endif;
				endif;
			elseif isinterval(x) then
				if writereal then
					write(printf(f|sep1,left(x[i,#])));
					write(printf(f|sep,right(x[i,#])));
				else
					write(printf(ibracket1|f,left(x[i,#])));
					write(printf(","|f|ibracket2|sep,right(x[i,#])));
				endif;
			else
				write(printf(f|sep,x[i,#]));
			endif;
			count=count+1;
			if count>perline and #<s[2] then 
				count=0; putchar(10);
			endif;
		end;
		putchar(10);
	end;
endfunction

// Latex is a text system for mathematical texts. Sometimes, you may
// wish to output Euler matrices in Latex. Symbolic matrices can be
// printed using the latex command of Maxima.

function writelatex (x,f="%g",environment="pmatrix",perline=100)
## Write x for AMS latex matrix environment
##
## This function can output an Euler matrix for the AMS Latex
## environment pmatrix, or any other environment. The open file is
## used, or the terminal.
##
## perline: Maximal number of items per line
##
## See: writeformat, open, latex
	write("\begin{"|environment|"}"); putchar(10);
	writeformat(x,f,sep1=" & ",sep2=" \\", ..
		writereal=0,ibracket1="[",ibracket2="]",perline=perline);
	write("\end{"|environment|"}"); putchar(10);
endfunction

// Use the dir command to get a listing of the files in the active
// directory. This is the directory, where the notebook is stored. It
// can be changed using the changedir command in @"eulercore".

function overwrite dir (pattern:string=none, dirs:integer=false)
## vector of strings with matching file names or subdirectories.
##
## The pattern can be something like "*.en" or "file?.dat"
## If dirs==true, then only directories will be listed.
## Directories are listed with a "/" at the end.
##
## See: changedir, searchfile
	if pattern==none then pattern="*"; endif;
	s=_dir(pattern);	
	if dirs then
		res=none;
		loop 1 to length(s)
			f=s[#];
			if substring(f,-1,-1)=="/" then
				if res==none then res=[f]
				else res=res|f;
				endif;
			endif;
		end;
		return res;
	endif;
	return s;
endfunction
