// Linear Algebra

comment
Linear algebra and regression routines.
endcomment

// This file contains basic linear algebra routines, like solutions of
// linear systems, linear fit, polynomial fit, eigenvalues and
// eigenspaces, decompositions, singular values, norms, and special
// matrices.

// It does also contain methods to solve systems by hand (pivotize).

// See: ../Programs/04 - Introduction to Linear Algebra
// See: eulercore.html#Vectors_and_Matrices | Euler Core

// * Linear Systems

function id (n:index, m:index=none)
## Creates a nxn identity matrix.
##
## See: diag, eye
	if m==none then return diag([n,n],0,1);
	else return diag([n,m],0,1);
	endif;
endfunction

function eye (n:positive integer,m:index=none)
## Creates a nxm matrix with 1 on the diagonal
##
## If m=none, then it creates an nxn matrix. This is a Matlab
## function.
##
## See: id
	if m==none then 
		if cols(n)>1 then return diag(n,0,1);
		else return diag([n,n],0,1);
		endif;
	else return diag([n,m],0,1);
	endif;
endfunction

function inv (A:complex)
## Computes the inverse of a matrix, using the Gauss algorithm.
##
## See: xinv, svdsolve, xlgs, fit, pinv
	return A\id(cols(A));
endfunction

function invert (A)
## returns A^{-1}
## Alias to inv.
##
## See: inv, pinv
	return inv(A);
endfunction

function transpose (A)
## returns A'
## Alias for Maxima
	return A';
endfunction

function image (A:complex)
## Computes the image of the quadratic matrix A.
##
## Returns a matrix, containing a basis of the image of A in the
## columns.
##
## The image of a matrix is the space spanned by its columns. This
## functions uses an LU decomposition with the Gauss algorithm to
## determine a base of the columns. The function "svdimage" returns an
## orthogonal basis of the image with more effort.
##
## >A=redim(1:9,3,3)
##                    1                   2                   3 
##                    4                   5                   6 
##                    7                   8                   9 
## >rank(A)
##  2
## >image(A)
##      0.0592348877759      0.118469775552 
##       0.236939551104       0.29617443888 
##       0.414644214431      0.473879102207 
## >svdimage(A)
##      -0.214837238368     -0.887230688346 
##      -0.520587389465     -0.249643952988 
##      -0.826337540561       0.38794278237 
## >kernel(A)
##                    1 
##                   -2 
##                    1 
## >svdkernel(A)
##       0.408248290464 
##      -0.816496580928 
##       0.408248290464 
##
## See: svdimage
	if isvar("eps"); localepsilon(eps); endif;
	a=A/norm(A);
	{aa,r,c,d}=lu(a);
	return a[:,nonzeros(c)];
endfunction

function kernel (A:complex)
## Computes the kernel of the matrix A.
##
## Returns a matrix M, containing a basis of the kernel of A. I.e.,
## A.M=0.
##
## This function uses an LU decomposition with the Gauss algorithm. The
## function "svdkernel" does the same with more effort, but it returns
## an orthogonal base of the kernel.
##
## Add eps=..., if A is almost regular. Use a larger epsilon in this
## case than the default epsilon(). 
##
## See: image, svdkernel
	if isvar("eps"); localepsilon(eps); endif;
	no=norm(A);
	if no==0 then return id(cols(A)); endif;
	a=A/no;
	{aa,r,c,d}=lu(a);
	n=size(a); nr=size(r);
	if nr[2]==n[2]; return zeros([1,n[2]])'; endif;
	if nr[2]==0; return id(n[2]); endif;
	c1=nonzeros(c); c2=nonzeros(!c);
	b=lusolve(aa[r,c1],a[r,c2]);
	m=size(b);
	x=zeros([n[2] m[2]]);
	x[c1,:]=-b;
	x[c2,:]=id(cols(c2));
	return x
endfunction

function rank (A:complex)
## Computes the rank of the matrix A.
##
## The rank is the dimension of the image, the space spanned by the
## columns of A.
##
## See: image
	return cols(image(A));
endfunction

function totalsum (A)
## Computes the sum of all elements of A.
	return sum(sum(A)');
endfunction

function seidel (A:real, b:real column, x=0, om=1.2)
## Solve Ax=b using the Gau-Seidel method for large, sparse A.
##
## A must be diagonal dominant, at least not have 0 on the diagonal.
## The method will converge for all positiv definit matrices. However,
## larger dominance in the diagonal speeds the convergence. For thin
## matrices use seidelX.
##
## om is a relaxation parameter between 1 and 2. The default is
## 1.2, and works in many cases. Specify the accuracy with an optional
## parameter eps. The optional column vector x is start value.
##
## See: seidelX
	if isvar("eps") then localepsilon(eps); endif;
	if x==0 then x=zeros(size(b)); endif;
	n=cols(A);
	D=band(A,0,0);
	M1=band(A,1,n-1)+(1-1/om)*D;
	M2=flipx(flipy(band(A,-n+1,-1)+D/om));
	repeat
		xn=flipy(lusolve(M2,flipy(b-M1.x)));
		if all(xn~=x) then return xn; endif;
		x=xn;
	end;
endfunction

function cg (A:real, b:real column, x0:column=none, f:index=10)
## Conjugate Gradent method to solve Ax=b.
##
## This function is the function of choice for large matrices. There
## is a variant "cgX" for large, sparse matrices.
##
## Stops as soon as the error gets larger.
##
## See: cgX, cpxfit
	if isvar("eps") then localepsilon(eps); endif;
	n=cols(A);
	if x0==none then x=zeros(size(b));
	else; x=x0;
	endif;
	loop 1 to 10
		r=b-A.x; p=r; fehler=r'.r;
		loop 1 to f*n
			if sqrt(fehler)~=0 then return x; endif;
			Hp=A.p;
			a=fehler/(p'.Hp);
			x=x+a*p;
			rn=r-a*Hp;
			fehlerneu=rn'.rn;
			p=rn+fehlerneu/fehler*p;
			r=rn; fehler=fehlerneu;
		end;
	end;
	return x;
endfunction

function solvespecial (A:complex, b:complex)
## Special solution of Ax=b using the LU decomposition.
##
## Returns a vector, such that A.x=b, even for singular, or non-square
## matrices A.
##
## This function uses the Gauss algorithm to determine the LU
## decomposition. It then extracts a base from the columns, and
## uses this base to get a special solution. Works only, if the matrix
## has maximal rank, i.e., the rows are linear independent.
##
## For matrices, which have a solution, the functions "svdsolve" or
## "fit" provide the same functionality with more effort.
##
## See: svdsolve, fit, kernel
	{a,r,c,d}=lu(A); x=zeros(cols(A),1); cc=nonzeros(c);
	s=A(:,cc)\b; x(cc)=s;
	return x
endfunction

function det (A:complex)
## Determinant of A.
##
## Uses the Gauss algorithm to compute the determinant of A.
	if isvar("eps"); localepsilon(eps); endif;
	r=maxrowsum(A);
	{aa,rows,c,d}=lu(A/r);
	return d*r^cols(A);
endfunction

function cholesky (A)
## Decompose a real matrix A, such that A=L.L'. Returns L.
##
## A must be a positive definite symmetric and at least 2x2 matrix.
## A is not checked for symmetry,
##
## See: LU, lu
	n=cols(A);
	L=zeros(size(A));
	L[1,1]=sqrt(A[1,1]); L[2,1]=A[1,2]/L[1,1];
	L[2,2]=sqrt(A[2,2]-L[2,1]^2);
	loop 3 to n;
		c=flipy(lusolve(flipx(flipy(L[1:#-1,1:#-1])),flipy(A[1:#-1,#])));
		L[#,1:#-1]=c';
		L[#,#]=sqrt(A[#,#]-c'.c);
	end;
	return L;
endfunction

function lsolve (L,b)
## Solve L.L'.x=b
##
## See: cholesky
	return lusolve(L',flipy(lusolve(flipx(flipy(L)),flipy(b))));
endfunction

function choleskyeigen (A)
## Iterates the Cholesky-iteration, until the diagonal converges.
##
## A must be positive definite symmetric and at least 2x2.
##
## See: cholesky
	if isvar("eps"); localepsilon(eps); endif;
	L=cholesky(A);
	B=L'.L;
	d=diag(B,0);
	repeat
		L=cholesky(B);
		B=L'.L;
		dnew=diag(B,0);
		if all(dnew~=d) then break; endif;
		d=dnew;
	end;
	return dnew;
endfunction


function pivotize (A:numerical, i:index, j:index)
## Make A[i,j]=1 and all other elements in column j equal to 0.
##
## The function is using elementary operations on the rows of A. Use
## this functions for the demonstration of the Gauss algorithm.
##
## The function modifies A, but returns A, so that it can be printed
## easily.
##
## See: normalizeRow, swapRows, gjstep
	k=1:rows(A); k=nonzeros(k<>i);
	A[k]=A[k]-A[k,j]/A[i,j]*A[i];
	A[i]=A[i]/A[i,j];
	return A;
endfunction

function pivotizeRows (A:numerical, i:index, j:index)
## Alias to pivotize
##
## See: pivotize
	return pivotize(A,i,j);
endfunction

function normalizeRow (A:numerical, i:index, j:index)
## Divide row j by A[i,j].
##
## The function is using elementary operations. Use this functions for
## the demontration of the Gauss algorithm.
##
## See: pivotizeRows, swapRows
	A[i]:=A[i]/A[i,j];
	return A;
endfunction

function swapRows (A:numerical, i1:index, i2:index)
## Swap rows i1 and i2 of A.
##
## The function is using elementary operations. Use this functions for
## the demontration of the Gauss algorithm.
##
## See: pivotizeRows, normalizeRow
	A[[i1,i2]] := A[[i2,i1]];
	return A
endfunction

// * QR Decomposition

function qrdecomp (A:real)
## Decompose QA=R using Givens rotations.
##
## The function returns {R,Q,c}. Q is an orthogonal matrix, and R is
## an upper triangle matrix. c is a vector with ones in the basis
## columns of A.
##
## This function uses orthogonal transformations to compute Q and R.
## It works for non-square matrices A.
##
## see: givensqr, givensrot
	B=id(rows(A));
	return givensqr(A,B);
endfunction

// * Fit

function fitnormal (A:complex, b:complex)
## Computes x such that ||A.x-b||_2 is minimal using the normal equation.
##
## A is an nxm matrix, and b is a nx1 vector.
##
## The function uses the normal equation A'.A.x=A'.b. This works only,
## if A has full rank. E.g., if n>m, then the m columns of A must be
## linear independent. The function "fit" provides a faster and better
## solution. "svdsolve" has the additional advantage that it produces
## the solution with minimal norm. For sparse matrices use "cpxfit"
##
## For badly conditioned A, you should use svdsolve instead.
##
## A : real or complex matrix (nxm)
## b : column vector or matrix (mx1 or mxk)
##
## See: fit, svdsolve, cpxfit
	At=conj(A');
	return symmult(At,A)\(At.b)
endfunction

function fit (A:real, b:real)
## Computes x such that ||A.x-b||_2 is minimal using Givens rotations.
##
## A is a nxm matrix, and b is a nxk vector (usually k=1).
##
## This function works even if A has no full rank. Since it uses
## orthogonal transformations, it us also quite stable. The function
## "svdsolve" does the same, but minimizes the norm of the solution
## too.
##
## A : real matrix (nxm)
## b : real column vector or matrix  (mx1 or mxk)
##
## See: fitnormal, svdsolve
	{x,y,c}=givensqr(A,b);
	c=nonzeros(c); n=1:cols(c);
	xr=lusolve(x[n,c],y[n,:]);
	x=zeros(cols(A),cols(b)); x[c,:]=xr;
	return x;
endfunction

function divideinto (A: real, B: real)
## Divide A into B yielding fit(B',A')'
##
## This function is used in Matlab mode for A/B.
##
## See: fit
	return fit(B',A')'
endfunction

function polyfit (x:complex vector, y:complex vector, ..
	n:nonnegative integer scalar, w:real vector=none)
## Fits a polynomial in L_2-norm to data.
##
## Given data x[i] and y[i], this function computes a polynomial p of
## given degree such that the sum over the squared
## errors (p(x[i])-y[1])^2 is minimimal.
##
## The fit uses the normal equation solved with an orthogonal
## decomposition.
##
## w : Optional weights (squared) for the 
##
## See: fit
	A=ones(size(x))_dup(x,n); A=cumprod(A');
	if w!=none then A=w'*A; y=w*y; endif;
	return fit(A,y')';
endfunction

// * Norm

function norm (A:numerical, p:nonnegative real=2)
## Euclidean norm of the vector A.
##
## For p=2, the function computes the square root of the sum of
## squares of all elements of A, if A is a matrix. For p>0 it computes
## the p-norm. For p=0 it computes the sup-norm.
##
## Note: The function does not compute the matrix norm.
##
## See: maxrowsum
	if p==2 then return sqrt(sum(sum(abs(A)^2)'));
	elseif p==0 then return totalmax(A);
	elseif p==1 then return totalsum(abs(A));
	else return totalsum(A^p)^(1/p);
	endif;
endfunction

function maxrowsum (A:numerical)
## Compute the maximal row sum of A
	return max(sum(abs(A))');
endfunction

function crossproduct (v, w)
## Cross product between two 3x1 or 1x3 vectors.
##
## The function will work for two column or row vectors. The result is
## of the same form as v.
	if cols(v)==1 then
		return [v[2]*w[3]-v[3]*w[2];-v[1]*w[3]+w[1]*v[3];v[1]*w[2]-v[2]*w[1]];
	else
		return [v[2]*w[3]-v[3]*w[2],-v[1]*w[3]+w[1]*v[3],v[1]*w[2]-v[2]*w[1]];
	endif;		
endfunction

// * Eigenvalues

function %eigenremove (l,n)
	{ll,i}=sort(abs(l[1]-l)); h=l[i];
	return l[n+1:cols(l)];
endfunction

function eigenvalues (A:complex)
## Eigenvalues of A.
##
## Returns a complex vector of eigenvalues of A.
##
## This function computes the characteristic polynomial using
## orthogonal transformations to an upper diagonal matrix, with
## additional elements below the diagonal. It then computes the zeros
## of this polynomial with "polysolve".
##
## See: charpoly, polysolve, svd, jacobi, svdeigenvalues, mxmeigenvalues
	return polysolve(charpoly(A));
endfunction

function eigenspace (A:complex, l:complex vector)
## Returns the eigenspace of A to the eigenvaue l.	
##
## The eigenspace is computed with kernel(A-l*id(n)). Since the
## eigenvalue may be inaccurate, the function tries several epsilons
## to find a non-zero kernel. A more stable function is
## "svdeigenspace".
##
## See: eigenvalues, svdeigenspace
	k=kernel(A-l*id(cols(A)));
	if all(k==0) then k=kernel(A-l*id(cols(A)),eps=1e-8); endif;
	if all(k==0) then k=kernel(A-l*id(cols(A)),eps=1e-4); endif;
	if all(k==0) then error("No eigenvalue found!"); endif;
	si=size(k);
	loop 1 to si[2];
		x=k[:,index()];
		k[:,index()]=x/norm(x);
	end;
	return k;
endfunction

function eigen (A:complex)
## Eigenvectors and a basis of eigenvectors of A.
##
## Returns {l,x,ll}, where l is a vector of eigenvalues, x is a basis
## of eigenvectors, and ll is a vector of distinct eigenvalues.
##
## See: eigenvalues, eigenspace, mxmeigenvectors
	l=eigenvalues(A);
	s=eigenspace(A,l[1]);
	si=size(s); v=dup(l[1],si[2])'; vv=l[1];
	l=%eigenremove(l,si[2]);
	repeat;
		if min(size(l))==0; break; endif;
		ll=l[1]; sp=eigenspace(A,ll);
		si=size(sp); l=%eigenremove(l,si[2]);
		s=s|sp; v=v|dup(ll,si[2])'; vv=vv|ll;
	end;
	return {v,s,vv}
endfunction

function cjacobi (A:complex, eps=sqrt(epsilon))
## Jacobi method to compute the eigenvalus of a Hermitian matrix A.
##
## Returns a vector of eigenvalues.
##
## This function computes the eigenvalues of a real or complex matrix
## A, using an iteration with orthogonal transformations. It works
## only for Hermitian matrices A, i.e., A=conj(A').
##
## See: jacobi
	if not all(A==conj(A')) then error("A not Hermitian!"); endif;
	A0=A;
	n=cols(A0);
	repeat;
		H=band(A0,-n,-1);
		E=extrema(abs(H));
		J=extrema(E[:,3]');
		i=J[4];
		j=E[i,4];
		if J[3]<eps; break; endif;
		A2=A0[[j,i],[j,i]];
		Q2=%rotation2(A2);
		Q=id(n);
		Q[[j,i],[j,i]]=Q2;
		A0=conj(Q').A0.Q;
	end;
	return diag(A0,0);
endfunction

function overwrite jacobi (A:real)
## Jacobi method to compute the eigenvalus of a symmetric matrix A.
##
## Returns a vector of eigenvalues.
##
## This function computes the eigenvalues of a real matrix A, using an
## iteration with orthogonal transformations. It works only for
## symmetric matrices A, i.e., A=A'.
##
## See: cjacobi
	if isvar("eps") then localepsilon(eps); endif;
	if not all(A~=A') then error("Matrix not symmetric!"); endif;
	return _jacobi((A+A')/2)
endfunction

function %pospower (A,n)
	if n==1 then return A; endif;
	m=floor(n/2); y=%pospower(A,m);
	if (2*m==n) then return y.y;
	else return y.y.A;
	endif;
endfunction

// * Hilbert Matrix

hilbertfactor:=3*3*3*5*5*7*11*13*17*19*23*29;

function hilbert (n:index, f=hilbertfactor)
## Scaled nxn Hilbert matrix.
##
## The Hilbert matrix is the matrix (1/(i+j-1)). This function
## multplies all entries with a factor, so thatthe matrix can
## accurately be stored up to the 30x30 Hilbert matrix.
    {i,j}=field(1:n,1:n);
    return f/(i+j-1);
endfunction

function hilb (n:integer, f=hilbertfactor)
## Scaled nxn Hilbert matrix.
##
## Alias for hilbert(n).
##
## See: hilbert
	return hilbert(n,f);
endfunction

// * Matrix Functions

function power (A:numerical, n:integer scalar)
## A^n for integer n for square matrices A.
##
## The function uses a binary recursive algorithm. For n<0, it will
## compute the inverse matrix first. For n<0, the matrix must be real
## or complex, not interval.
##
## For scalar values, use the faster x^n. Moreover, power(x,n) will
## not map to the elements of x or n.
##
## See: matrixpower
	if n>0 then return %pospower(A,n);
	elseif n==0 then return id(cols(A)); 
	elseif not isinterval(A) then return %pospower(inv(A),-n);
	else error("power(A,n) does not work for interval matrices and n<0"); 
	endif;
endfunction

function matrixpospower (A:numerical, n:natural)
## Returns A^n for natural n>=0 and a matrix A.
##
## Alias for power(A,n)
	return %pospower(A,n)
endfunction

function matrixpower (A:complex, n:integer scalar)
## Returns A^n for integer n and a matrix A.
##
## Alias for power(A,n)
	return power(A,n)
endfunction

function matrixfunction (f:string, A:complex)
## Evaluates the function f for a matrix A.
##
## The function f must be an analytic function in the spectrum of A.
## A must have a basis of eigenvectors. This function will decompose A
## into M.D.inv(M), where D is a diagonal matrix, and apply f to the
## diagonal of D.
##
## See: matrixpower
	{d,M}=eigen(A);
	R=M.diag(size(A),0,f(d;args())).inv(M);
	if all(R~=re(R)) then return real(R)
	else return R;
	endif;
endfunction

function LU (A)
## Returns {L,R,P} such that L.R=P.A
##
## This function uses the Gau-Algorithm in lr(A) to find the LR
## decomposition of a regular square matrix A. L is a lower triangle
## matrix with 1 on the diagonal, R an upper right triangle matrix,
## and P a permutation matrix.
##
## See: lu
	{B,r,c,det}=lu(A);
	if det~=0 then error("Determinant too small!"); endif;
	n=cols(A);
	return {band(B[r],-n+1,-1)+id(n),band(B[r],0,n-1),id(n)[r]}
endfunction

function echelon (A)
## Forms A into echelon form using the Gau algorithm
##
## See: LU, lu
	{B,r,c,det}=lu(A);
	j=nonzeros(c);
	i=1:length(j);
	C=A[r];
	return inv(C[i,j]).C[i];
endfunction

// * Filter

function overwrite filter (b:complex vector, a:complex vector, ..
	x:complex vector, y:complex vector=none, zeros:integer=true)
## Apply a filter with a and b to start values x and y
##
## Computes y[n] recursively using
##
## a[1]*y[n]+...+a[m]*y[n-m+1]=b[1]*x[n]+...+b[k]*x[n-k+1]
##
## a[1] must not be zero. 
##
## The start values on the right hand side are either 0 or the first k
## values of x, depending on the flag zeros (true by default). The
## start values on the left hand side are either 0 (the default), or
## the values of the parameter y.
##
## The size of a must exceed the size of x by 1.
## All input vectors must be real or complex row vectors or scalars.
##
## See: filter, fold, fftfold
	if zeros and cols(b)>1 then x=zeros(1,cols(b)-1)|x; endif;
	if y==none then y=zeros(1,cols(a)); endif;
	return _filter(b,a,x,y);
endfunction

// * Singular Values

// The singular value decomposition of Euler is based on the builtin
// function svd. It is used to compute an orthogonal basis of the
// kernel and the image of a matrix, the condition of a matrix, or the
// pseudo-inverse.

function svdkernel (A:real)
## Computes the kernel of the quadratic matrix A
##
## This function is using the singular value decomposition, and works
## for real matrices only. 
##
## Returns an orthogonal basis of the kernel as columns of a matrix.
##
## See: kernel
	{B,w,V}=svd(A);
	i=nonzeros(w~=0);
	if cols(i)==0; return zeros(cols(V),1);
	else return V[:,i];
	endif;
endfunction

function svdimage (A:real)
## Computes the image of the quadratic matrix A
##
## This function is using the singular value decomposition, and works
## for real matrices only.
##
## Returns an orthogonal basis of the image as columns of a matrix.
##
## See: kernel, image, svdkernel
	{B,w,V}=svd(A);
	i=nonzeros(!(w~=0));
	if cols(i)==0; return zeros(1,cols(A));
	else return B[:,i];
	endif;
endfunction

function svdcondition (A:real)
## Condition number based on a singular value decompostion of A
##
## Returns the qutient of the largest singular value divided by the
## smallest. 0 means singularity.
##
## A : real matrix
	{B,w,V}=svd(A);
	if any(w~=0); return 0; endif;
	return max(abs(w))/min(abs(w));
endfunction

function svddet (A:real)
## Determinant based on a singular value decomposition of A
##
## A : real matrix
##
## See: det
	{B,w,V}=svd(A);
	return prod(w);
endfunction

function svdeigenvalues (A:real)
## Eigenvalues or singular values of A
##
## For a symmetric A, this returns the eigenvalues of A For a
## non-symmetric A, the singular values.
##
## A : real matrix
##
## See: eigenvalues
	{B,w,V}=svd(A);
	return w;
endfunction

function svdeigenspace (A:real,l:real)
## Returns the eigenspace of A to the eigenvaue l
##
## See: eigenspace
	k=svdkernel(A-l*id(cols(A)));
	if k==0; error("No eigenvalue found!"); endif;
	si=size(k);
	loop 1 to si[2];
		x=k[:,index()];
		k[:,index()]=x/sqrt(x'.x);
	end;
	return k;
endfunction

function svdsolve (A:real,b:real)
## Minimize |A.x-b| with smallest norm for x
##
## The singular value decomposition is one way to solve the fit
## problem. It has the advantage, that the result will be the result
## with smallest norm, if there is more than one solution.
##
## A : real matrix
##
## See: fit
	{B,w,V}=svd(A);
	i=nonzeros(!(w~=0));
	if (cols(i)>0); w[i]=1/w[i]; endif;
	return V.diag(size(V),0,w).(B'.b);
endfunction

function svdinv (A:real)
## The pseudo-inverse of A.
##
## The pseudo-inverse B of a matrix solves the fit problem to minimize
## |Ax-b| by x=B.b. It is computed in this function using an svd
## decomposition.
##
## See: pseudoinv
	return svdsolve(A,id(rows(A)))
endfunction

function pinv (A:real)
## The pseudo-inverse of A.
##
## Alias to svdinv
##
## See: svdinv
	return svdsolve(A,id(rows(A)))
endfunction

function ginv (A:real)
## Gilbert inverse of a matrix A
##
## This inverse has the property A.G.A=A
	i=shuffle(1:rows(A));
	H=A[i];
	j=shuffle(1:cols(A));
	H=H[:,j];
	{B,C,c}=qrdecomp(A);
	H=H[:,nonzeros(c)]';
	{B,C,c}=qrdecomp(A);
	H=H[:,nonzeros(c)]';
	n=min(size(H));
	H=inv(H[1:n,1:n])';
	Z=zeros(size(A));
	Z[i[1:n],j[1:n]]=H;
	return Z';
endfunction

// * Gau-Jordan Scheme

// Some functions for the demonstration of the Gau-Jordan algorithm.
// These functions can be used for linear systems or for the simplex
// algorithm.

// See: ../Programs/17 - Introduction to Optimization.html#The_Gauss-Jordan_Scheme | Example

defaultgjdigits:=3;
defaultgjlength:=10;

function gjprint (A : real, n : positive integer vector,
	v : string vector, digits:nonnegative integer=none,
	length:index=none)
## Print the matrix A in Gau-Jordan form.
##
## The scheme A is printed in the form
##
##    x  y ...
## a  1  2 ...
## b  3  4 ...
## ...
##
## n : index vector with a permumatin of the variables
## v : variable names with the column variables first
## digits : number of digits for each number
## length : output length of each number
##
## File: gaussjordan
## See: gjstep
	global defaultgjdigits,defaultgjlength;
	if digits==none then digits=defaultgjdigits; endif;
	if length==none then length=defaultgjlength; endif;
	ma=rows(A); na=cols(A); nv=cols(n)-ma;
	s=printstr("",length);
	for j=1 to nv;
		s=s|printstr(v[n[ma+j]],length);
	end;
	s,
	for i=1 to ma;
		s=printstr(v[n[i]],length);
		for j=1 to na;
			s=s|print(A[i,j],digits,length);
		end;
		s,
	end;
endfunction

function gjstep (A:numerical, i:index, j:index,
	n:positive integer vector=none,
	v:string vector=none, digits:nonnegative integer=none,
	length:index=none)
## Make A[i,j]=1 and all other elements in column j equal to 0.
##
## The function is using elementary operations on the rows of A. Use
## this functions for the demonstration of the Gauss algorithm.
##
## The function modifies A and n.
##
## n : A row vector of indices, the function assumes the Gau-Jordan
## form. n contains the indices of the variables, first the indices of
## the variables in the rows, and then the indices of the variables in
## the columns. The row variable i is exchanged with the column
## variable j. This is the same as makeing the j-th column to an
## canonical vector e[i], and inserting in the j-th column the result
## of this operation applied to e[i].
##
## If n is present, you can add a vector v of strings, which contains
## the names of the variables. The problem will then be printed using
## gjprint.
##
## File: gaussjordan
## See: pivotize, gjprint
	global defaultgjdigits,defaultgjlength;
	if digits==none then digits=defaultgjdigits; endif;
	if length==none then length=defaultgjlength; endif;
	m=rows(A);
	b=zeros(m,1); b[i]=1;
	k=1:m; k=nonzeros(k<>i);
	b[k]=b[k]-A[k,j]/A[i,j]*b[i];
	A[k]=A[k]-A[k,j]/A[i,j]*A[i];
	b[i]=b[i]/A[i,j];
	A[i]=A[i]/A[i,j];
	A[:,j]=b;
	h=n[i]; n[i]=n[m+j]; n[m+j]=h;
	R=(0|n[(m+1):cols(n)])_(n[1:m]'|A);
	if v<>none then
		gjprint(A,n,v,digits,length);
	endif
endfunction

function gjsolution (M : real, n : positive integer vector)
## Read the values of the solution from the Gau-Jordan scheme.
##
## For this, we assume that the last column contains the values, and
## the variables in the top row are set to zero.
##
## See: gjstep
	m=rows(M);
	x=M[:,-1]'|zeros(1,cols(n)-m); x=x[n];
	return x[(m+1):cols(n)];
endfunction

function gjvars (M : real, simplex=false)
## Generate variable names for M
##
## simplex : If true the last row variable will be named ""
##           and the last column will name will be missing.
##
## Returns {v,n}
##    where n=1:(rows+cols), v=["s1"...,"x1",...]
##
## See: gjprint
	n=rows(M);
	m=cols(M); if simplex then m=m-1; endif;
	nres=1:(n+m);
	v=["s1"];
	loop 2 to n-1; v=v|("s"+#); end;
	if simplex then v=v|""; else v=v|("s"+n); endif;
	loop 1 to m; v=v|("x"+#); end;
	return {v,nres}
endfunction

function gjaddcondition (M : real, 
	n:positive integer vector, v:string vector,
	line:real vector, varname:string)
## Add a condition to the Gau-Jordan Form
##
## The line for the condition (left side <= right side) and the
## variable name must be provided. The condition uses the current top
## row variables.
##
## Return {M,n,v}
	r=rows(M)-1; c=cols(M);
	M=M[1:r]_line_M[-1];
	v=v|varname;
	n=n[1:r]|(cols(n)+1)|n[r+1:cols(n)];
	return {M,n,v};
endfunction
