// Integer Arithmetic

comment
Integer factorization and modulo functions.
endcomment

function map gcd (a:integer, b:integer)
## Computes the greatest common divisor of a and b.
## See also: primes, isprime, factor, lcm, mod, invmod, ggt.
	if a>b;
		u=a; v=b;
	else
		u=b; v=a;
	endif;
	repeat;
		if v==0; break; endif;
		r=mod(u,v);
		u=v;
		v=r;
	end;
	return u;
endfunction

function lcm (a,b)
## Returns least common multiple of a and b.
## See also: primes, isprime, factor, gcd, mod, invmod, ggt.
	return a*b/gcd(a,b);
endfunction;

function ggt (x:integer, y:integer)
## Return {g,a,b} such that ax+by=g.
##
## >p=23423429; isprime(p)
##  1
## >x=3423; {g,a,b}=ggt(x,p);> a,
##  4831125
## >mod(a*x,p)
##  1
##
## See also: primes, isprime, factor, gcd, lcm, mod, invmod.
	if x<y; {g,a,b}=ggt(y,x); return {g,b,a}; endif;
	if y~=1; return {1,0,1}; endif;
	m=mod(x,y);
	if m~=0; return {y,0,1}; endif;
	{g,aa,bb}=ggt(y,m);
	return {g,bb,aa-bb*floor(x/y)};
endfunction

function map invmod (x:integer, m:integer)
## Return y such that x*y is 1 modulo m.
##
## >p=23423429; invmod(3423,p)
##  4831125
## See also: primes, isprime, factor, gcd, lcm, mod, ggt.
	{g,a,b}=ggt(x,m);
	if !(g~=1); error("no inverse!"); endif;
	if a<0; return m+a; endif;
	return a;
endfunction

function primes (n:integer)
## Return all primes up to n using the Sieve of Erastothenes.
##
## >length(primes(100000))
##  9592
##
## For this, I have demonstrated a native implementation in C.
##
## See also: isprime, factor, gcd, lcm, mod, invmod, ggt.
	if n>=3
		len = floor((n-1)/2);
		sieve = ones ([1,len]);
		for i=1 to (sqrt(n)-1)/2;
			if (sieve[i]) then
				sieve[3*i+1:2*i+1:len] = 0; .. do it
			endif
		end
		return [2, 1+2*nonzeros(sieve)];
	elseif n>=2
		return 2;
	else
		return [];
	endif
endfunction

function map isprime (n:integer)
## Returns true if n is a prime number, false otherwise.
## See also: primes, factor, gcd, lcm, mod, invmod, ggt.
	if (n != ceil(n) || n < 2)
		return 0;
	elseif n < 4
		return 1;
	else
		q = n/[2:sqrt(n)];
		return !any(q == ceil(q));
	endif
endfunction

function factor (n:integer, p=none)
## Returns prime factorization of n. If n==1, returns 1.
## See also: primes, isprime, gcd, lcm, mod, invmod, ggt.
	if n < 4
		return n;
	else
		if p==none then p = primes(floor(sqrt(n))+1); endif;
		q = n/p;
		idx = nonzeros (q == ceil(q));
		if cols(idx)==0 then
			return [n];
		else
			r = p[idx];
			q = factor(n/prod(r),p);
			if all(q != 1) then
				return (sort([r, q]));
			else
				return r;
			endif
		endif
	endif
endfunction
