// Numerical Algorithms

comment
Various numerical methods.
endcomment

// For demos of some of these algorithms refer to the following
// notebook.

// See: ../Programs/06 - Introduction to Numerical Analysis

// * Solve Equations

function bisect (f:string, a:number, b:number, y:number=0)
## Uses the bisection method to solve f(x)=y in [a,b]
##
## f is either a function of one real variable, or an expression in x.
## Addtional parameters to "bisect" after a semicolon will be passed
## to the function f.
##
## The bisection routine assumes a zero in the interval, i.e., it
## assumes different signs of f(a) and f(b). The routine bisects the
## interval until the desired accuracy is reached. It is a very stable
## function.
##
## It is possible to specify the accuracy epsilon with eps=... as last
## parameter.
##
## f : function or expression in x
## a,b : interval bounds
## y : target value
##
## >bisect("x^2",0,2,y=2)
##  1.41421356237
##
## See: newton, secant, ibisect
	if (isvar("eps")); localepsilon(eps); endif;
	if f(b,args())<y;
		b1=a; a1=b;
		if f(a,args())<y error("No zero in interval"); endif;
	else;
		a1=a; b1=b;
		if f(a,args())>y error("No zero in interval"); endif;
	endif;
	repeat
		m=(a1+b1)/2;
		if a1~=b1; break; endif;
		if f(m,args())>y; b1=m; else a1=m; endif;
	end;
	return m;
endfunction

function secant (f:string, a:complex scalar, b=none, y:number=0)
## Uses the secant method to solve f(x)=y in [a,b]
##
## f is either a function of one real variable, or an expression in x.
## Additional parameters to "secant" after a semicolon will be passed
## to the function f.
##
## The secant routine uses an approximation of the derivative to find
## a better approximation to the solution. This function is almost as
## good as the Newton method, but does not require the derivative of
## f.
##
## Note that the result is not guaranteed to be within the interval
## [a,b]. The function may fail, if the approximated derivative is too
## close to 0.
##
## The parameter b is optional. In this case, the function starts with
## an interval around a.
##
## It is possible to specify the accuracy epsilon with eps=... as last
## parameter.
##
## f : function or expression in x
## a,b : interval bounds, b is optional
## y : target value
##
## >bisect("x^2",0,2,y=2)
##  1.41421356237
## >
## >secant("x^2",1,y=2)
##  1.41421356237
## >function f(x,a) := x^2-a
## >secant("f",5;3), %^2
##  1.73205080757
##  3
##
## See: bisect, newton
	if (isvar("eps")); localepsilon(eps); endif;
	if b==none then x0=a-sqrt(epsilon); x1=a+sqrt(epsilon);
	else x0=a; x1=b; 
	endif;
	y0=f(x0,args())-y; y1=f(x1,args())-y;
	repeat
		x2=x1-y1*(x1-x0)/(y1-y0);
		if x2~=x1; break; endif;
		x0=x1; y0=y1; x1=x2; y1=f(x2,args())-y;
	end;
	return x2
endfunction

function solve (f:string, a:complex scalar, b=none, y:number=0)
## Solve f(x)=y near a.
##
## Alias to "secant".
##
## See: secant
	if (isvar("eps")) then return secant(f,a,b,y;args(),eps=eps);
	else return secant(f,a,b,y;args())
	endif;
endfunction

function %evalf (expression)
### Evaluate the expression f with global parameters.
	useglobal;
	return evaluate(expression);
endfunction

function root (f:string, %x)
## Find the root of an expression f by changing the variable x. 
##
## This is a function for interactive use. f should be an expression
## in several variables. All variables but x are constant, and only x
## is changed to solve f(x,...)=0. Other variables must be set
## globally. Note that the actual name of the variable x may be
## different from "x", so you can solve for any variable in the
## expression f.
##
## All variables in the expression must be global!
##
## f : expression in several variables
## %x : variable to bo solved for
##
## >a=2; x=2; root("x^x-a",x); "a = "+a, "x = "+x,
##  a = 2
##  x = 1.55961046946
## >a=2; x=2; root("x^x-a",a); "a = "+a, "x = "+x,
##  a = 4
##  x = 2
##
## See: secant
	if (isvar("eps")); localepsilon(eps); endif;
	if (%x==0) x1=0.00001;
	else x1=1.00001*%x;
	endif;
	x0=%x;
	a=%evalf(f);
	repeat
		%x=x1; b=%evalf(f);
		xn=(x0*b-x1*a)/(b-a);
		if xn~=%x; break; endif;
		x0=x1; x1=xn; a=b;
	end;
	return %x;
endfunction

// * Integrals

function simpson (f:string, a:number, b:number, n:natural=50, maps:integer=0)
## Integrates f in [a,b] using the Simpson method
##
## The Simpson method is stable, but not fast. The results are not
## accurate. The method is exact for polynomials of degree 3. Use
## "gauss" for better results.
##
## f is either a function of one real variable, or an expression in x.
## Addtional parameters to "bisect" after a semicolon will be passed
## to the function f.
##
## f : function or expression in x
## a,b : interval bounds
## n : number of points to use
## maps : flag, if the expression needs to be mapped (0 or 1).
##        Functions are always mapped.
##
## See: romberg, gauss
	t=linspace(a,b,2*n);
	s=%evalexpression(f,t,maps;args());
	ff=4-mod(1:2*n+1,2)*2; ff[1]=1; ff[2*n+1]=1;
	return sum(ff*s)/3*(t[2]-t[1]);
endfunction

function simpsonfactors (n)
## Returns the vector 1,4,2,...,2,4,1.
##
## See: simpson
	ff=4-mod(1:2*n+1,2)*2; ff[1]=1; ff[2*n+1]=1;
	return ff;
endfunction

function romberg (f:string, a:number, b:number, m:natural=10, maps:integer=0)
## Romberg integral of f in [a,b]
##
## This method gets very accurate results for many functions. The
## Romberg algorithm uses an extrapolation of the trapezoidal rule for
## decreasing step sizes. However, the Gauss method is to be preferred
## for functions, which are close to polynomials. The function may
## fail, if f is not smooth enough.
##
## f is either a function of one real variable, or an expression in x.
## Addtional parameters to "bisect" after a semicolon will be passed
## to the function f.
##
## f : function or expression in x
## a,b : interval bounds
## m : number of points to use for the start of the algorithm
## maps : flag, if the experssion needs to be mapped (0 or 1).
##        Functions are always mapped.
##
## See: simpson, gauss, integrate
	if (isvar("eps")); localepsilon(eps); endif;
	y=%evalexpression(f,linspace(a,b,m),maps;args());
	if cols(y)==1 then return y*(b-a); endif;
	h=(b-a)/m;
	y[1]=y[1]/2; y[m+1]=y[m+1]/2; I=sum(y);
	S=I*h; H=h^2; Intalt=S;
	repeat;
		I=I+sum(%evalexpression(f,a+h/2:h:b,maps;args())); h=h/2;
		S=S|I*h;
		H=H|h^2;
		Int=interpval(H,interp(H,S),0);
		if Int~=Intalt; break; endif;
		Intalt=Int;
	end;
	return Intalt
endfunction

// * Iterations

function iterate (f:string, x:numerical, n:natural=0)
## Iterates f starting from x to convergence.
##
## This function iterates xnew=f(x), until xnew~=x, or the maximal
## number of iterations n is exceeded. It returns the limit for n=0
## (the default), or n column vectors.
##
## f is either a function of a scalar returning a scalar, or a
## function of a vector returning a vector. The function will work
## with column or row vectors, and even for matrices, if n==0. f can
## also be an expression of a variable x.
##
## f : function or expression
## x : start point for the algorithm 
## n : optional number of steps
	if n>0 then
		return niterate(f,x,n;args());
	endif;
	if (isvar("eps")) then localepsilon(eps); endif;
	if isinterval(x) then
		repeat
			x=f(x,args());
			if all(left(x)~=right(x)) then return x; endif;
		end;
	else;
		repeat
			xn=f(x,args());
			if all(x~=xn) then return xn; endif;
			x=xn;
		end;
	endif;
endfunction

function niterate (f:string, x:numerical, n:natural)
## Iterates f n times starting from x.
##
## Works like iterate with n>0. 
##
## f : function or expression
## x : start point for the algorithm 
## n : number of steps
##
## See: iterate
	c=(cols(x)==1);
	if c then y=zeros(rows(x),n); else y=zeros(n,cols(x)); endif
	loop 1 to n
		x=f(x,args());
		if c then y[:,#]=x; else y[#]=x; endif;
	end;
	return y;
endfunction

function %seqhelp (expression,x,n)
	useglobal;
	return evaluate(expression);
endfunction

function sequence (f:string, x:numerical, n:natural)
## Computes sequences recursively.
##
## This functions is used to compute x[n]=f(x[1],...,x[n-1])
## recursively. It will work for scalar x[i], or for column vectors
## x[i]. Depending on the iteration function, more than one start
## value x[1] to x[k] is needed. These start values are stored in a
## row vector x0, or for vector iteration in a matrix x0 as columns.
## The iteration count n determines the highest x[n], which is
## computed.
##
## f is a function f(x,n), or an expression in x an n. The parameter x
## contains the values x[1],...,x[n-1] computed so far as
## elements of a row vector, or as columns in a matrix x. In case of a
## vector sequence, the function can also work with row vectors,
## depending on the result of the function f. Note, that the start
## value must fit to the result of the function.
##
## f : function f(x,n) (x : mx(n-1) or (n-1)xm, result: mx1 or 1xm)
## x : start values (mxk or kxm)
## n : desired number of values
##
## See: iterate
	loop cols(x)+1 to n;
		if isfunction(f) then xnew=f(x,#;args());
		else xnew=%seqhelp(f,x,#);
		endif;
		if cols(xnew)==1 then x=x|xnew;
		else x=x_xnew;
		endif;
	end;
	return x;
endfunction	

// * Newton

function newton (f:string, df:string, x:complex scalar, y:number=0)
## Solves f(x)=y with the Newton method.
##
## The Newton method needs the derivative of the function f. This
## derivative must be computed by the function df. The Newton method
## will start in x, and stop if the desired accuracy is reached. It is
## possible to set this accuracy with eps=...
##
## The function will only work for real or complex variables. For
## systems of equations, see "newton2". For intervals, see "inewton".
##
## f and df must be a function of a real or complex scalar, returning a
## scalar, or an expression of x.
##
## f : function or expression in x (real or complex, scalar)
## df : function or expression in x (real or complex, scalar)
## x : start value
## y : target value
##
## See: secant, bisect, newton2, inewton, mxminewton
	if (isvar("eps")); localepsilon(eps); endif;
	repeat
		a=f(x,args())-y;
		b=df(x,args());
		xnew=x-a/b;
		if all(xnew~=x) then return xnew; endif;
		x=xnew;
	end;
endfunction

function newton2 (f:string, df:string, x:numerical)
## Newton methods to solve a system of equations.
##
## The mulitdimensional Newton method can solve the equation f(v)=0
## for row vectors v. Here, f must map row vectors to row vectors.
## This function needs the Jacobian matrix of f, which is provided
## throught the function df. The Newton method will fail, if the
## Jacobian gets singular during the computation.
##
## f must be a function of a row vector, returning a row vector,
## or an expression of the row vector x. df must be a function
## of a row vector, returning the Jacobian matrix of f. Additional
## parameters after a semicolon are passed to the functions f and df.
## The function f can also use column vectors. In this case, the start
## value must be a column vector.
##
## f : function f(v) (v: 1xn or nx1, result 1xn or nx1)
## df : function df(v) (v: 1xn or nx1, result: nxn)
## x : start point (1xn or nx1)
##
## See: broyden, neldermin, inewton2
	if (isvar("eps")); localepsilon(eps); endif;
	c=(cols(x)==1);
	repeat
		if c then d=df(x,args())\f(x,args());
		else d=(df(x,args())\f(x,args())')';
		endif; 
		xn=x-d;
		if all(xn~=x) then return xn; endif;
		x=xn;
	end
endfunction

function steffenson (f:string, x:scalar, n:natural=0)
## Does n steps of the Steffenson operator, starting from x.
##
## This is a similar function as iterate. However, the function
## assumes an asymptotic expansion of the convergence, and uses the
## Steffenson operator to speed up the convergence.
##
## f is either a function of a scalar returning a scalar, or a
## function of a column vector returning a column vector. f can also
## be an expression of a variable x.
##
## f : function or expression of a scalar x
## x : start point
## n : optional number of iterations
##
## See: iterate
	if (isvar("eps")); localepsilon(eps); endif;
	if n>0 then
		return nsteffenson(f,x,n;args())
	endif;
	repeat
		a=f(x,args()); b=f(a,args());
		c=b-2*a+x;
		if c~=0; return x; endif;
		xn=(x*b-a*a)/c;
		if xn~=x; return xn; endif;
		x=xn;
	end
endfunction

function nsteffenson (ggg:string, x0:scalar, n:natural)
## Does n steps of the Steffenson operator, starting from x0.
##
## Works like "steffenson", but returns the history of the iteration.
##
## See: steffenson, niterate
	if (isvar("eps")); localepsilon(eps); endif;
	y=zeros(rows(x0),n);
	x=x0;
	loop 1 to n;
		a=ggg(x,args); b=ggg(a,args);
		c=b-2*a+x;
		if c~=0 then i=#; return y[:,1:i-1]; endif;
		x=(x*b-a*a)/c;
		y[:,#]=x;
	end
	return y;
endfunction

function aitkens (x: vector)
## Improves the convergence of the sequence x.
##
## The Aitkens operator assumes that a-x[n] has a geometric
## convergence 1/c^n to 0. With this information, it is easy to compute
## the limit a.
##
## x : row vector (1xn, n>2)
##
## Return a row vector (1x(n-2)) with faster convergence.
##
## See: steffenson
	n=length(x);
	return x[1:n-2]-(x[2:n-1]-x[1:n-2])^2/(x[3:n]-2*x[2:n-1]+x[1:n-2]);
endfunction

// * Differentiation

function %setupdif (n)
	A=zeros([n,2*n+1]);
	loop 1 to n;
	b=zeros([2*#+1,1]); b[#+1]=fac(#);
	a=((-#:#)^(0:2*#)')\b;
	A[#,1:2*#+1]=a';
	end;
	return A;
endfunction

DifMatrix:=%setupdif(5);

function %dif (x, f, n=1, h=epsilon()^(1/3))
	global DifMatrix;
	if n==0;
		if isfunction(f); return f(x,args()); 
		else return expreval(f,x); 
		endif;
	elseif (n==1)
		return (-f(x+2*h,args())/12+2*f(x+h,args())/3 ..
			-2*f(x-h,args())/3+f(x-2*h,args())/12)/h
	elseif n<=5;
		eps=h^(3/(2*n+1));
		h=(-n:n)*eps; xh=h'+x; a=DifMatrix[n,1:2*n+1];
		if isfunction(f); 
			return (a.map(f,xh,args()))/eps^n;
		else
			return (a.f(xh,args()))/eps^n;
		endif;
	else
		error("Too high a derivative!");
	endif;
endfunction

function diff (f:string, x:numerical, n:natural=1, h:number=epsilon()^(1/3))
## Compute the n-th (n<6) derivative of f in the points x.
##
## Numerical differentiation is somewhat inaccurate. To get a good
## approximation, the first derivative uses 4 function evaluations.
##
## f is either a function in a scalar, or an expression in x.
## Additional parameters after a semicolon are passed to f.
##
## f : function or expression a scalar x
## x : the point, where the derivative should be computed
## n : degree of derivative
## h : optional delta
	return %difmap(x;f,n,h,args())
endfunction

function dif (f, x, n=1, h=epsilon()^(1/3))
## Compute the n-th (n<6) derivative of f in the points x.
##
## Alias to diff.
	return %difmap(x;f,n,h,args())
endfunction

// * Folding

function fftfold (v:vector, w:vector)
## fold v with w using fast Fourier transformation.
##
## The length of the vector v should have small prime factors. For
## large v, this is much faster than fold(v,w). However, the vector v
## is folded as a periodic vector. To get the same result as
## fold(v,w), delete the first cols(w)-1 coefficients.
##
## v : row vector (1xn)
## w : row vector (1xm, m<=n)
##
## See: fold
	v1=zeros(size(v)); v1[1:cols(w)]=w;
	z=ifft(fft(v)*fft(v1));
	if isreal(v) then return real(z); 
	else return z;
	endif;
endfunction

// * Broyden Algorithm

// The Broyden algorithm is a Quasi-Newton method for solving F(x)=0
// with vectors x. It does not need a derivative. Instead, it uses an
// approximation of the Jacobian Df to start with, and updates it at
// each step.

function broyden (f:string, xstart:real, A:real=none)
## Finds a zero of f with the Broyden algorithm.
##
## The Broyden algorithm is an iterative algorithm just like the
## Newton method, which solves the equation f(v)=0 for vectors v.
## It tries to approximate the Jacobian of the function f using the
## previous iteration steps. The algorithm will fail, if the Jacobian
## of f in the zero is singular.
##
## The function f must take a vector v, and return a vector.
## Additional parameters to "broyden" after the semicolon are passed
## to f. The function can work with column or row vectors. The start
## vector must be of the same form.
##
## x is the start value, and the optional matrix A is a approximation
## of the Jacobian matrix of f.
##
## To change the accuracy, you can specify an optional eps=... as last
## parameter.
##
## f : function of one vector
## xstart : start point, a row vector
## A : optional start for the Jacobian
##
## Returns the solution as a vector.
##
## See: nbroyden
	if (isvar("eps")); localepsilon(eps); endif;
	c=(cols(xstart)==1);
	x=xstart; n=length(x);
	if c then x=x'; endif;
	delta=sqrt(epsilon());
	if A==none then
		A=zeros(n,n);
		y=f(x,args());
		loop 1 to n;
			x0=x; x0[#]=x0[#]+delta;
			x1=x; x1[#]=x1[#]-delta;
			if c then
				A[:,#]=(f(x0,args())-f(x1,args()))/(2*delta);
			else
				A[:,#]=(f(x0,args())-f(x1,args()))'/(2*delta);
			endif;
		end;
	endif;
	y=f(x,args());
	if c then y=y'; endif;
	repeat
		d=-A\y'; x=x+d';
		y1=f(x,args()); 
		if c then y1=y1'; endif;
		q=y1-y;
		if all(d~=0) then break; endif;
		A=A+((q'-A.d).d')/(d'.d);
		y=y1;
	end;
	if c then return x'; else return x; endif;
endfunction

function nbroyden (f:string, xstart:real vector, nmax:natural, A:real=none)
## Do nmax steps of the Broyden algorithm.
##
## This function works like "broyden", but returns the steps of the
## algorithm. Moreover, it does at most nmax steps.
##
## f : function of one row vector
## xstart : start point, a row vector
## nmax : maximal number of steps
## A : optional start for the Jacobian
##
## Returns a matrix, with one step of the algorithm in each row,
## and the current approximation of the Jacobian.
##
## See: broyden
	if (isvar("eps")); localepsilon(eps); endif;
	x=xstart; n=cols(x); r=x;
	delta=epsilon();
	if A==none then
		A=zeros([n n]);
		y=f(x,args());
		loop 1 to n;
			x0=x; x0[#]=x0[#]+delta;
			A[:,#]=(f(x0,args())-y)'/delta;
		end;
	endif;
	y=f(x,args());
	count=0;
	repeat
		count=count+1;
		d=-A\y'; xn=x+d';
		if all(xn~=x) || count>nmax; r=r_xn; break; endif;
		x=xn;
		y1=f(x,args()); q=y1-y;
		A=A+((q'-A.d).d')/(d'.d);
		y=y1;
		r=r_xn;
	end;
	return {r,A};
endfunction

// * Hermite Interpolation

function hermiteinterp (x:vector, y)
## Divided differences for Hermite interpolation.
##
## x : row vector of interpolation points. x can contain double
##     points in immediate succession. These points use derivatives
##     for the interpolation.
## y : Either a function df(x,n), which computes the n-th derivatives
##     of f (including the 0-th derivative), or a vector of values
##     and derivatives in the form f(x),f'(x),f''(x),... for a
##     multiple interpolation point x.
##
## Returns the divided differences, which can be used in interpval
## (divdifeval) as usual.
##
## The multiplicities in x can be generated with multdup.
##
## >xp=multdup(1:3,2)
##  [ 1  1  2  2  3  3 ]
## >yp=[1,0,0,0,-1,0]
##  [ 1  0  0  0  -1  0 ]
## >d=hermiteinterp(xp,yp)
##  [ 1  0  -1  2  -1.5  1.5 ]
## >plot2d("interpval(xp,d,x)",0.9,3.1);
##
## >function fh (x,n,f,fd) ...
## $  if n==0 then return f(x);
## $  elseif n==1 then return fd(x);
## $  else error("n=0 or n=1!");
## $  endfunction
## >xp=multdup(chebzeros(0,1,5),2);
## >expr &= sqrt(x);
## >d=hermiteinterp(xp,"fh";expr,&diff(expr,x));
## >plot2d("sqrt(x)-interpval(xp,d,x)",0,1);
##
## See: divdif, divdifeval, multdup
	n=length(x);
	d=zeros(1,n);
	if isstring(y) then
		df=y;
		for i=1 to n;
			d[i]=df(x[i],0,args());
		end;
		havevalue=false; value=0;
		for i=1 to n-1;
			for j=n to i+1 step -1;
		    	if x[j]==x[j-i] then 
		    		if not havevalue then
		    			value=df(x[j],i,args())/i!;
		    			havevalue=true;
		    		endif;
		    		d[j]=value;
		    	else
		    		havevalue=false; 
		    		d[j]=(d[j]-d[j-1])/(x[j]-x[j-i]);
		    	endif;
			end;
		end;
	else
		k=1; ind=ones(1,n);
	    for i=2 to n;
	    	if not x[i]==x[i-1] then k=i; endif;
	    	ind[i]=k;
	    end;
		for i=1 to n;
			d[i]=y[ind[i]];
		end;
		havevalue=false; value=0;
		for i=1 to n-1;
			for j=n to i+1 step -1;
		    	if x[j]==x[j-i] then 
		    		if not havevalue then
		    			value=y[ind[j]+i]/i!;
		    			havevalue=true;
		    		endif;
		    		d[j]=value;
		    	else
		    		havevalue=false; 
		    		d[j]=(d[j]-d[j-1])/(x[j]-x[j-i]);
		    	endif;
			end;
		end;
	endif;
	return d
endfunction

function hermitedivdif (x:vector, y)
## Divided differences for Hermite interpolation.
##
## Alias to hermiteinterp
##
## See: hermiteinterp
	return hermiteinterp(x,y);
endfunction

function multdup (x:numerical, n:nonnegative integer)
## Repeat the rows or columns of x with multiplicities in n
##
## E.g., if n[1]=2, and n is a row vector, the first column of x is
## duplicated.
##
## If n is a column vector, the rows of x are duplicated, if it is a
## row vector, the columns are duplicated. n can be shorter than the
## number of rows resp. columns of x. 
##
## If n is scalar, it acts on the rows, duplicating the first row n
## times.
##
## See: dup, hermite
	if typeof(n)==0 then n=n*ones(1,cols(x)); endif;
	if rows(n)==1 or cols(n)==1 then
	    m=totalsum(n);
		ind=ones(1,m);
		k=1;
		for i=1 to length(n);
			if n[i]>0 then
				ind[k:k+n[i]-1]=i;
				k=k+n[i];
			endif;
		end;
		if cols(n)==1 then return x[ind,:];
		else return x[:,ind];
		endif;
	else
		error("Multiplicities n must be a row or column vector");
	endif;
endfunction

// * Remez Algorithm

function %remezhelp (x,y,n)
## {t,d,h}=%remezhelp(x,y,deg) is the case, where x are exactly deg+2
## points.
	d1=interp(x,y); d2=interp(x,mod(1:n+2,2)*2-1);
	h=d1[n+2]/d2[n+2];
	d=d1-d2*h;
	return {x[1:n+1],d[1:n+1],h};
endfunction

function remez (x:vector, y:vector, deg:index, tracing:integer=0, 
	remezeps=0.00001)
## Best polynomial approximation.
##
## The Remez algorithm computes the best polynomial approximation to
## the data (x[i],y[1]) of degree deg. This algorithm uses a
## simultanous exchange, which requires the points x[i] to be sorted.
##
## x : vector of points on the real line, sorted.
## y : vector of values at these points.
## deg : degree of the polynomial.
## tracing : if non-zero, the steps will be plotted.
##
## Returns {t,d,h,r}
##
## t : the alternation points
## d : the divided difference form of the best approximant
## h : the discrete error (with sign)
## r : the rightmost alternation point, which is missing in t. 
##
## To evaluate the polynomial in v use interpval(t,d,v).
	n=cols(x);
	ind=round(linspace(1,n,deg+1),0); // indices of reference points
	h=0; // discrete error on reference points
	repeat
		// compute new reference
		{t,d,hnew}=%remezhelp(x[ind],y[ind],deg);
		// compute reference error
		r=y-interpval(t,d,x); hh=hnew;
		// plot, if wanted
		if tracing then
			xplot(x,r); 
			color(15);
			xgrid(x[ind],ticks=0,color=9);
			ygrid([-hnew,0,hnew],ticks=0);
			color(1);
			title("Weiter mit Taste"); wait(180);
		endif;
		indnew=ind;
		// simultanuous exchange
		// extrema between first point of x and second reference point
		e=extrema(r[1:ind[2]]);
		if (hh<0) then
			indnew[1]=e[2];
		else
			indnew[1]=e[4];
		endif;
		// extrema between two reference points
		hh1=hh;
		loop 2 to deg+1;
			e=extrema(r[ind[#-1]:ind[#+1]]);
			if (hh>0);
				indnew[#]=e[2]+ind[#-1]-1;
			else
				indnew[#]=e[4]+ind[#-1]-1;
			endif;
			hh=-hh;
		end;
		// extrema on right interval end
		e=extrema(r[ind[deg+1]:cols(r)]);
		if (hh>0) then
			indnew[deg+2]=e[2]+ind[deg+1]-1;
		else
			indnew[deg+2]=e[4]+ind[deg+1]-1;
		endif;
		hh=-hh;
		// total maximum
		mm=max(abs(r[indnew]));
		if ind[deg+2]<cols(r) then
			e=extrema(r[ind[deg+2]:cols(r)]);
			if (hh>0) then
				if abs(r[e[2]])>mm then
					indnew=indnew[2:deg+2]|(e[2]+ind[deg+2]-1);
				endif;
			else
				if abs(r[e[4]])>mm then
					indnew=indnew[2:deg+2]|(e[4]+ind[deg+2]-1);
				endif;
			endif;
		endif;
		mm=max(abs(r[indnew]));
		if (ind[1]>1) then
			e=extrema(r[1:ind[1]]);
			if (hh1>0) then
				if abs(r[e[2]])>mm then
					indnew=e[2]|indnew[1:deg+1]; 
				endif;
			else
				if abs(r[e[4]])>mm then
					indnew=e[4]|indnew[1:deg+1]; 
				endif;
			endif;
		endif;
		ind=indnew;
		if (abs(hnew)<(1+remezeps)*abs(h)) then break; endif;
		h=hnew;
	end;
	{t,d,h}=%remezhelp(x[ind],y[ind],deg);
	if tracing then
		xplot(x,y-interpval(t,d,x));
		color(15);
		xgrid(x[ind],ticks=0,color=9);
		ygrid([-hnew,0,hnew],ticks=0);
		color(1);
		title("Weiter mit Taste"); wait(180);
	endif;
	return {t,d,h,x[ind[deg+2]]};
endfunction

// * Nonlinear Optimization

function fmin (f:string, a:real scalar, b:real scalar)
## Compute the minimum of the convex function f in [a,b].
##
## Uses the golden cut method, starting with the interval [a,b]. The
## method takes about 60*(b-a) function evalations for full accuracy.
##
## f is either an expression in x, or a function of x. Additional
## parameters are passed to a function f.
##
## You can specify an epsilon eps with eps=... as last parameter.
##
## See: fmax
	if (isvar("eps")); localepsilon(eps); endif;
	x0=a; x3=b;
	y0=f(x0,args()); y3=f(x3,args());
	l=(3-sqrt(5))/2;
	x1=x0+l*(x3-x0); x2=x3-l*(x3-x0);
	y1=f(x1,args()); y2=f(x2,args());
	repeat
		if y1>y2;
			x0=x1; x1=x2; x2=x3-l*(x3-x0);
			y0=y1; y1=y2;
			y2=f(x2,args());
		else
			x3=x2; x2=x1; x1=x0+l*(x3-x0);
			y3=y2; y2=y1;
			y1=f(x1,args());
		endif;
		if x0~=x3; return x0; endif;
	end;
endfunction

function fmax (f:string, a:real scalar, b:real scalar)
## Compute the maximum of the concave function f in [a,b].
##
## Uses the golden cut method, starting with the interval [a,b]. The
## method takes about 60*(b-a) function evalations for full accuracy.
##
## f is either an expression in x, or a function of x. Additional
## parameters after a semicolon are passed to a function f.
##
## You can specify an epsilon eps with eps=... as last parameter.
##
## See: fmin
	if (isvar("eps")); localepsilon(eps); endif;
	x0=a; x3=b;
	y0=f(x0,args()); y3=f(x3,args());
	l=(3-sqrt(5))/2;
	x1=x0+l*(x3-x0); x2=x3-l*(x3-x0);
	y1=f(x1,args()); y2=f(x2,args());
	repeat
		if y1<y2;
			x0=x1; x1=x2; x2=x3-l*(x3-x0);
			y0=y1; y1=y2;
			y2=f(x2,args());
		else
			x3=x2; x2=x1; x1=x0+l*(x3-x0);
			y3=y2; y2=y1;
			y1=f(x1,args());
		endif;
		if x0~=x3; return x0; endif;
	end;
endfunction

function fextrema (f:string, a:real scalar, b:real scalar, n:integer=100)
## Find all internal extrema of f in [a,b].
##
## f may be an expression in x or a function. Additional
## parameters after a semicolon are passed to a function f.
##
## You can specify an epsilon eps with eps=... as last parameter.
##
## Returns {minima,maxima} (vectors, possibly of length 0)
##
## See: fmax, fmin
	if (isvar("eps")); localepsilon(eps); endif;
	x=linspace(a,b,n);
	mi=[]; ma=[];
	y=f(x,args());
	loop 2 to n;
		if y[#]>=y[#-1] && y[#]>y[#+1];
			ma=ma|fmax(f,x[#-1],x[#+1];args());
		elseif y[#]<=y[#-1] && y[#]<y[#+1]
			mi=mi|fmin(f,x[#-1],x[#+1];args());
		endif;
	end;
	return {mi,ma}
endfunction

function %fexprv (x,expr)
	useglobal;
	return evaluate(expr);
endfunction

function brentmin (f:string, a:real scalar, d=0.1, eps=epsilon())
## Compute the minimum of f around a with Brent's method.
##
## d is an initial step size to look for a starting interval.
## eps is the final accuracy.
##
## Returns the point of minimal value.
##
## f is an expression in x, or a function in f. Additional parameters
## after a semicolon are passed to a function f.
##
## Return the point of minimum.
##
## See: fmin, fmax
	if isfunction(f);
		return brent(f,a,d,eps;args());
	else;
		return brent("%fexprv",a,d,eps,f);
	endif;
endfunction

function neldermin (f:string, v:real, d=0.1, eps=epsilon(), tries=2)
## Compute the minimum of f around v with the Nelder-Mead method.
##
## The Nelder-Mead method is a stable, but slow method to search for a
## local minimum of a function without using any derivative. It should
## not be used for high dimensions. Of course, it can also be applied
## to solve a system equations by minimizing the norm of the errors.
##
## f must be function of a row or column vector x, returning a real
## value. Addditional parameters after the semicolon are passed to f.
## f can also be an expression depending on a vector x.
##
## d is an optional initial step size for a starting simplex.
## eps is the final accuracy.
##
## f : function f(v) (v : 1xn or nx1, result: scalar)
## v : start point for the search (1xn or nx1)
## d : optional size of start simplex
## eps : optional accuracy
## tries : number of restarts of the algorithm
##
## Return the point of minimum (1xn vector).
##
## See: nelder
	m=f(v,args());
	loop 1 to tries
		if isfunction(f);
			v1=_nelder(f,v,d,eps;args());
		else;
			v1=_nelder("%fexprv",v,d,eps,f);
		endif;
		m1=f(v,args());
		if m1>m then return v; endif;
		v=v1;
	end;
	return v1;
endfunction

function overwrite nelder (f:string, v:real, d=0.1, eps=epsilon())
## Compute the minimum of f around v with the Nelder-Mead method.
##
## Alias to neldermin.
##
## See: neldermin
	return neldermin(f,v,d,eps;args());
endfunction

function nlfit (f,Df,v)
## Minimizes f(x) from start vector v.
##
## This method is named after Levenberg-Marquardt.
##
## f(x) maps 1xn vectors to 1xm vectors (m>n)
## Df(x) is the Jacobian of f.
	fn=norm(f(v));
	repeat
		M=Df(v);
		vnew=-fit(M,f(v)'-M.v')';
		fnnew=norm(f(vnew));
		if fn~=fnnew then return vnew; endif;
		fn=fnnew;
		v=vnew;
	end;
endfunction

function map %nbeval (lambda;f,x,v,c,A,b)
    diff = (b-A.(x+lambda*v)')';
    if all(diff>0) then return f(x+lambda*v)-c*sum(log(diff));
    else return NAN;
    endif;
endfunction

function %nbdescent (f:string, x:real vector, v:real vector, 
    A:real, b:real column,
	c:real,	lambda:positive real=1)
## Search minimum of f(x+lambda*v)
##
## Determine x+lambda*v such that h is minimized. It is assumed that v
## is indeed a direction of minimization. The function first
## searches for an ascent into direction v, doubling the step size. If
## the initial lambda delivers an ascent, the function divides lambda
## until there is a descent. Then fmin is used to find the minimum.
## The function returns the minimum x+lambda*v and the lambda it took
## to start the fmin algorithm.
##
## Returns: {xmin,lambda}
	y=%nbeval(0,f,x,v,c,A,b);
	ynew=%nbeval(lambda,f,x,v,c,A,b);
	if isNAN(ynew) then
		repeat
			lambda=lambda/2;
			ynew=%nbeval(lambda,f,x,v,c,A,b);
			until !isNAN(ynew) || lambda~=0;
		end
		if lambda~=0 then return {x,lambda}; endif;
	endif;
	if ynew>=y then
		repeat
			lambda=lambda/2;
			ynew=%nbeval(lambda,f,x,v,c,A,b);
			until ynew<y || lambda~=0;
			y=ynew;
		end
		if lambda~=0 then return {x,lambda}; endif;
		lambda=lambda*2;
	else
		repeat
			lambda=lambda*2;
			ynew=%nbeval(lambda,f,x,v,c,A,b);
			until isNAN(ynew) || ynew>=y;
			y=ynew;
		end
		if isNAN(ynew) then
			a1=lambda/2; b1=lambda;
			repeat
				d=(a1+b1)/2;
				ynew=%nbeval(d,f,x,v,c,A,b);
				if isNAN(ynew) then b1=d;
				elseif ynew<y then a1=d; y=ynew;
				else lambda=d; break;
				endif;
				if b1~=a1 then 
					error("Could not determine the descent. "+ ..
					"Problem unbounded?");
				endif;
			end;
		endif;
	endif
	lmin=fmin("%nbeval",0,lambda;f,x,v,c,A,b);
	return {x+lmin*v,lambda}
endfunction

function newtonbarrier (f:string, df:string, Hf:string,
	A:real, b:real column,
	x:real vector, lambda:positive real=1, 
	c:positive real=0.1, cfactor:positive real=0.1,
	history=false, eps=epsilon())
## Newton-Barrier method starting from interior x.
##
## This function can minimize a convex function function f(x) subject
## to conditions Ax <= b. It needs the gradient df and the Hessian Hf
## of f as functions. The start point x must be an interior point with
## Ax < b. 
##
## Note: f must be convex.
##
## f,df,Hf : Functions taking a vector as a parameter.
## A,b : Conditions Ax<=b
## x : Start point with Ax<b
##
## lambda,c,cfactor : Variables for the algorithms. You can take the
## default.
##
## history : If true, the function returns a matrix with one step x
## in each row. Else it returns the last x only.
##
## eps : The absolute accuracy of the solution. The algorithm returns,
## when |x-xnew|<epsilon.
	if history then X=x; endif;
	repeat
		d=1/(b-A.x');
		H=A'.(d^2*A);
		v=-fit(Hf(x)+c*H,df(x)'-c*sum((A*d)'))';
		{xnew,lambda}=%nbdescent("f",x,v,A,b,c,lambda);
		until norm(x-xnew)<eps;
		x=xnew;
		if history then X=X_x; endif;
		c=c*cfactor;
	end;
	if history then return X;
	else return x;
	endif;
endfunction

// For an example of the Newton Barrier Method see the following
// introduction notebook.

// See: ../Programs/17 - Introduction to Optimization.html#Newton-Barrier_Method | Newton-Barrier Method

// * Splines

function spline (x,y)
## Defines the natural Spline at points x(i) with values y(i).
## 
## The natural spline is the spline using cubic polynomials
## between the points x[i], smooth second derivative, and linear
## outside the interval x[1] to x[n]. The points x[i] must be sorted.
##
## The function returns the second derivatives at these points. With
## this information, the spline can be evaluated using "splineval".
##
## >xp=1:10; yp=intrandom(1,10,3); s=spline(xp,yp);
## >plot2d("splineval(x,xp,yp,s)",0,11);
## >plot2d(xp,yp,>points,>add);
##
## See: splineval
	n=cols(x);
	h=x[2:n]-x[1:n-1];
	s=h[2:n-1]+h[1:n-2];
	l=h[2:n-1]/s;
	A=diag([n,n],0,2);
	A=setdiag(A,1,0|l);
	A=setdiag(A,-1,1-l|0);
	b=6/s*((y[3:n]-y[2:n-1])/h[2:n-1]-(y[2:n-1]-y[1:n-2])/h[1:n-2]);
	return (A\(0|b|0)')';
endfunction

function map splineval (t:number; x:vector, y:vector, h:vector)
## Evaluates the cubic interpolation spline for (x(i),y(i)) 
## with second derivatives h(i) at t
	n=cols(x);
	if t>=x[n] then
		x1=x[n]; x2=x[n-1];
		y1=y[n]; y2=y[n-1];
		f=(x2-x1)^2;
		h1=h[n]*f; h2=h[n-1]*f;
		b=h1/2; c=(h2-h1)/6;
		return y1+(y2-y1-b-c)*(t-x1)/(x2-x1);
	endif;
	if t<x[1] then
		x1=x[1]; x2=x[2];
		y1=y[1]; y2=y[2];
		f=(x2-x1)^2;
		h1=h[1]*f; h2=h[2]*f;
		b=h1/2; c=(h2-h1)/6;
		return y1+(y2-y1-b-c)*(t-x1)/(x2-x1);
	endif;
	p1=find(x,t); p2=p1+1;
	y1=y[p1]; y2=y[p2];
	x1=x[p1]; x2=x[p2]; f=(x2-x1)^2;
	h1=h[p1]*f; h2=h[p2]*f;
	b=h1/2; c=(h2-h1)/6;
	a=y2-y1-b-c;
	d=(t-x1)/(x2-x1);
	return y1+d*(a+d*(b+c*d));
endfunction

function evalspline (t:number, x:vector, y:vector, s:vector)
## Evaluates the cubic interpolation spline for (x(i),y(i)) 
## with second derivatives s(i) at t
## 
## See: spline, splineval
	return splineval(t,x,y,s);
endfunction

function map linsplineval (x:number; xp:vector, yp:vector,
	constant:integer=1)
## Evaluates the linear interpolating spline for (xp[i],yp[i]) at x
##
## xp must be sorted. Outside of the points of xp, the spline is
## continued as in the closest interval to x, or as a constant
## function.
##
## >xp=1:10; yp=intrandom(1,10,3);
## >plot2d("linsplineval(x,xp,yp,<constant)",0,11);
## >plot2d(xp,yp,>points,>add);
## 
	n=cols(xp);
	if (x>=xp[n]) then
		if constant then return yp[n]; endif;
		x1=xp[n]; x2=xp[n-1];
		y1=yp[n]; y2=yp[n-1];
		return y1+(y2-y1)/(x2-x1)*(x-x1);		
	endif;
	if (x<xp[1]) then
		if constant then return yp[1]; endif;
		x1=xp[1]; x2=xp[2];
		y1=yp[1]; y2=yp[2];
		return y1+(y2-y1)/(x2-x1)*(x-x1);		
	endif;	
	p1=find(xp,x); p2=p1+1;
	y1=yp[p1]; y2=yp[p2];
	x1=xp[p1]; x2=xp[p2];
	return y1+(x-x1)*(y2-y1)/(x2-x1);
endfunction

function evallinspline (x:number, xp:vector, yp:vector,
	constant:integer=1)
## Evaluates the linear interpolating spline for (xp[i],yp[i]) at x
##
## See: linsplineval
	linsplineval (x,xp,yp,constant);
endfunction