// Plot Functions plot2d and plot3d

comment
The main plot functions plot2d and plot3d.
endcomment

// plot2d handles all plotting in the plane. This includes plots of
// funtions of one variable, implicit plots of functions of two
// variables, plots of curves and polygons in the plane, data and bar
// plots of two vectors, or clouds of points in the plane.

// plot3d handles all plotting in 3D stereographic view. This includes
// plots of functions of two variables, implicit plots of functions of
// three variables, plots of surfaces, bar plots of matrices, or clouds
// of points in the space. There is an option to view the plot in
// anaglyph mode with red/cyan glasses.

// For a demonstration of 2D plots in Euler, see the following
// introduction notebook.

// See: ../Programs/02 - Introduction to 2D Graphics | 2D Plots
// See: ../Programs/03 - Introduction to 3D Graphics | 3D Plots

defaultanaglyphseparation=0.2;
anaglyphseparation=0.2;

function plot2d (xv,yv=none,btest=none, ..
	a=none,b=2,c=none,d=2, ..
	xmin=none,xmax=1, r=0, n=100, logplot=0, ..
	grid=2, frame=3, framecolor=none, ..
	square=0, color=none, thickness=1, style=none, auto=1, add=0, ..
	user=0, delta=0.1, points=0, ..
	bar=0, histogram=0, distribution=0, even=0, steps=0,..
	own=0, adaptive=1, ..
	hue=0, level=none, contour=0, nc=40, ..
	filled=0, fillcolor=none, outline=1, ..
	title="",xl=none,yl=none, ..
	maps=0, ..
	contourcolor=none, contourwidth=1, ..
	ticks=1, margin=none, clipping=true, ..
	cx=0, cy=0, ..
	insimg=0, spectral=0)
## Multipurpose plot function for plots in the plane.
##
## Overview: The function can plot 
##
## - expressions or functions of one variable, 
## - parametric curves,
## - x data against y data,
## - implicit functions,
## - bar plots,
## - polygons.
##
## If a function or expression for xv is given, plot2d will compute
## values in the given range using this function or expression. The
## expression must be a string expression in the variable x. The range
## must be defined in the parameters a and b unless the default range
## [-2,2] should be used. The y-range will be computed automatically,
## unless c and d are specified, or a radius r, which yields the range
## [-r,r] for x and y. For plots of functions, plot2d will use an
## adaptive evaluation of the function by default. To speed up the
## plot for complicated functions, switch this off with adpative=0,
## and otionally decrease the number of intervals n. Moreover, plot2d
## will by default use mapping. I.e., it will compute the plot element
## for element. If your expression or your functions can handle a
## vector x, you can switch that off with maps=0. Note that adaptive
## plots are always computed element for element.
##
## If functions or expressions for both xv and for yv are specified,
## plot2d will compute a curve with the xv values as x-coordinates and
## the yv values as y-coordinates. In this case, a range should be
## defined for the parameter using xmin, xmax. Expressions contained
## in strings must always be expressions in the parameter variable x.
##
## If xv and yv are data vectors, these data will be used as x- and
## y-coordinates of a curve. In this case, a, b, c, and d, or a radius
## r can be specified, or the plot window will adjust automatically to
## the data. Alternatively, square=1 can be set to keep a square
## aspect ratio.
##
## The function plot2d can also plot curves or data as filled
## polygons with filled=1. The fill color is determined by
## fillcolor=1, and on optional outline=0 prevents drawing the
## boundary for all styles but the default one.
##
## If xv is a sorted vector, and yv is a vector of intervals, then
## plot2d will plot the filled ranges in the plane. The fill styles
## are the same as the styles of polygons.
##
## Implicit plots show level lines solving f(x,y)=level, where
## "level" can be a single value or a vector of values. If
## level="auto", there will be nc level lines, which will spread
## between the minimum and the maximum of the function evenly. Darker
## or lighter color can be added with hue=1 to indicate value of the
## function. For implicit functions, xv must be a function or an
## experssion of the parameters x and y, or, alternatively, xv can be
## a matrix of values.
##
## The data for bar plots (bar=1) and histograms (histogram=1) can
## either be explicitely given in xv and yv, or can be computed from
## an empirical distribution in xv (with distribution=1). Histograms
## of xv values will be computed automatically with histogram=1. If
## even=1 is specified, the xv values will be counted in integer
## intervals.
##
## Logarithmic plots can be plotted either using a logarithmic scale
## in y with logarithmic=1, or using logarithmic scales in x and y
## with logarithmic=2.
##
## If you wish to show more than one function in a single plot use
## several calls to plot2d, and add=1 in all, but the first call, or
## provide a vector of expressions and functions, or use a matrix of
## yv values, with one function in each row.
##
## If xv is a function or an expression, user=1 allows the user to
## zoom (+/-), set the plot window with the mouse, or move the
## plot (cursor keys). The space key will reset the plot to the
## original plot window. If xv is a data, then user=1 will simply wait
## for key stroke.
##
## title="string" can be used to describe the plot. The title will
## appear above the plot. Moreover, a label for the x and y axis can
## be added with xl="string" or yl="string". Other labels can be added
## in a separate call to the function "label".
##
## Note: expressions and functions are mapped automatically in plot2d.
##
## Parameters
##
## x,y : equations, functions or data vectors
##
## a,b,c,d : Plot area (default a=-2,b=2)
## r : if r>0 then a=cx-r, b=cx+r, c=cy-r, d=cy+r
## xmin,xmax : range of the parameter for curves
## auto : Determine y-range automatically
## square : if 1, try to keep square x-y-ranges
## n : number of intervals
## grid : 0 = no grid and labels, 
##        1 = axis only, 
##        2 = normal grid, 
##        3 = inside axis
##        4 = no grid
##        5 = full grid including margin
## frame: 0 = no frame
## framecolor: color of the frame and the grid (default 3)
## margin : number between 0 and 1 for the margin around the plot
##
## color : plot color for curves (default 1)
## thickness : line thickness for curves (default 1)
## style : plot style for lines, markers, and fills
##         for points use 
##                "[]", "<>", ".", "..", "...",
##                "*", "+", "|", "-", "o"
##                "[]#", "<>#", "o#" (filled shapes)
##                "[]w", "<>w", "ow" (non-transparent)
##         for lines use 
##                "-", "--", "-.", ".", ".-.", "-.-", "->"
##         for filled polygons or bar plots use 
##                "#", "#O", "O", "/", "\", "\/",
##                "+", "|", "-", "t"
## points : plot single points instead of line segments
##
## add : add the plot to the existing plot
## user : enable user interaction for functions
## delta : step size for user interaction
##
## bar : bar plot (x are the interval bounds, y the interval values)
## histogram : plots the frequencies of x in n subintervals
## distribution=n : plots the distribution of x with n subintervals
## even : use inter values for automatic histograms.
##
## steps : plots the function as a step function (steps=1,2)
## adaptive : use adaptive plots (n is the minimal number of steps)
##
## level : plot level lines of an implicit function of two variables
## hue : add hue color to the level plot to indicate the function
##       value 
## contour : Use level plot with automatic levels 
## nc : number of automatic level lines
##
## title : plot title (default "")
## xl, yl : labels for the x- and y-axis
##
## filled : fill the plot of a curve
## fillcolor : fill color for bar and filled curves
## outline : boundary for filled polygons
##
## logplot : if 1, logplot in y, if 2 logplot in xy
##
## own : a string, which points to an own plot routine. With user=1,
##       you get the same user interaction as in plot2d. The range
##       will be set before each call to your function.
##
## maps : map expressions (0 is faster), functions are always mapped.
##
## contourcolor : color of contour lines
## contourwidth : width of contour lines
##
## clipping : toggles the clipping (default is true)
##
## Available index colors are
##
## white=0,black=1,red=2,green=3,blue=4,
## cyan=5, olive=6, lightgray=7, gray=8,
## darkgray=9, orange=10, lightgreen=11, turquoise=12,
## lightblue=13, lightorange=14, yellow=15.
##
## or use rgb(red,green,blue).
##
## Some settings can be changed globally only.
##
## setgrid("--",".",3) : Grid style and color
##
## Examples:
##
## >plot2d("x^3-x",-1,2,title="y=x^3-x",yl="y",xl="x");
## >plot2d("x^3-x",a=0,b=2,c=-1,d=2,grid=3,<frame);
## >plot2d("x^3-x",>user,title="Press any key!");
##
## >function f(x) &= x^x;
## >plot2d(f,r=1,cx=1,cy=1,color=blue,thickness=2);
## >plot2d(&diff(f(x),x),>add,color=red,style="-.-");
##
## >x=linspace(0,2pi,1000);
## >plot2d(sin(5x),cos(7x));
## >plot2d(sin(x),cos(x)*0.5,r=1,>filled,style="/");
##
## >k=0:10; 
## >plot2d(k,bin(10,k),>bar);
## >plot2d(k,bin(10,k)); plot2d(k,bin(10,k),>points,>add);
##
## >plot2d(normal(1,1000),>distribution,style="O");
## >plot2d("qnormal",0,5;2.5,0.5,>filled);
##
## >plot2d("x^2",0,1,steps=1,color=red,n=10);
## >plot2d("x^2",>add,steps=2,color=blue,n=10);
##
## >plot2d("x^3-y^2",>contour,>hue,>spectral);
## >plot2d("x^3-y^2",level=0,contourwidth=3,>add,contourcolor=red);
##
## See: plot3d, label, rgb
	
	if margin<>none then marginold=_margin(margin); endif;
	if fillcolor==none then fillcolor=fillcolor()[2]; endif;
	if framecolor==none then framecolor=framecolor(); endif;
	if color==none then color=color(); endif;
	oldclipping=clipping(clipping);
	if isvar("niveau") then level=niveau; endif;

	// allow plot areas, s.t. the function is not defined everywhere
	errors off;
	global usesubwindows;

	// contour or level plots (calls fcontour or datacontour)
	if contour or level!=none then
		if spectral then color=-2; contourcolor=1; endif;
		oldframe=_framecolor(framecolor);
		if r==0 then r=1; endif;
		if a==none then a=cx-r; b=cx+r; endif;
		if c==none then c=cy-r; d=cy+r; endif;
		if typeof(xv)==8 then
			fcontour(xv,a,b,c,d;args(), ..
				hue=hue,niveau=level,title=title,n=n,nc=nc, ..
				add=add,color=color,thickness=thickness,maps=maps,grid=grid, ..
				contourcolor=contourcolor,contourwidth=contourwidth);
		else
			datacontour(xv,a,b,c,d,hue=hue,niveau=level,title=title,nc=nc, ..
				add=add,color=color,thickness=thickness,grid=grid, ..
				contourcolor=contourcolor,contourwidth=contourwidth);
		endif;
		errors on;
		_framecolor(oldframe);
		if margin<>none then _margin(marginold); endif;
		clipping(oldclipping);
		if insimg then insimg(); endif;
		return;
	endif;
	
	oldhold=holding();
	
	// hold, if add=1 or subwindows are used
	if add then hold on; 
	elseif !oldhold then hold off; 
	endif;
	
	aa=a; bb=b; cc=c; dd=d; xx=xv; yy=yv;
	
	// take care of plot("f(x)",a,b)
	if typeof(btest)==0 and typeof(yy)==0 then
		aa=yv; bb=btest; yy=none;
	endif;
	
	// set the range if r is specified
	if r>0 then
		aa=cx-r; bb=cx+r; cc=cy-r; dd=cy+r;
	endif;
	
	// plot a complex matrix
	if iscomplex(xx) then
		if points then
			plot2d(re(xx),im(xx),a=aa,b=bb,c=cc,d=dd,points=1, ..
				grid=grid,frame=frame,framecolor=framecolor, ..
				square=square,color=color,thickness=thickness, ..
				style=style,add=add,title=title,user=user,maps=maps, ..
				xl=xl,yl=yl,r=r,ticks=ticks,margin=margin,grid=grid);
		else
			plot2d(re(xx),im(xx),a=aa,b=bb,c=cc,d=dd,grid=grid, ..
				frame=frame,framecolor=framecolor, ..
				square=square,color=color,thickness=thickness,style=style, ..
				add=add,title=title,user=user,thickness=thickness, ..
				maps=maps,xl=xl,yl=yl,r=r,ticks=ticks,margin=margin);
			plot2d(re(xx'),im(xx'),add=1,grid=grid, ..
				frame=frame,framecolor=framecolor, ..
				square=square,color=color,thickness=thickness,style=style, ..
				add=add,title=title,user=user,thickness=thickness, ..
				maps=maps,xl=xl,yl=yl,r=r,ticks=ticks,margin=margin);
		endif;
		ret=plot();
		errors on;
		if margin<>none then _margin(marginold); endif;
		clipping(oldclipping);
		if insimg then insimg(); endif;
		holding(oldhold);
		return ret;
	endif;
	
	// use old plot range, if add=1
	if add then
		p=_plot();
		if aa==none then aa=p[1]; bb=p[2]; endif;
		cc=p[3]; dd=p[4];
	// if a,b are not specified, and c,d not, compute range
	// automatically
	elseif typeof(aa)<>8 and typeof(cc)==8 then
		auto=1;
	// if both are specified, use them
	elseif typeof(aa)<>8 and typeof(cc)<>8 then
		auto=0;
	endif;
	
	// if histogram>0 then compute the xx,yy from the data
	if histogram then
		if histogram==1 then histogram=20; endif;
		{xx,yy}=histo(xx,histogram,even=even);
		bar=1;
	endif;
	
	// if distribution>0 then compute xx, yy from the data
	if distribution then
		if distribution==1 then distribution=20; endif;
		ny=cols(xx);
		{xx,yy}=histo(xx,distribution,even=even);
		yy=yy/(ny*(xx[2]-xx[1]));
		bar=1;
	endif;
	
	if !add then unclip(); endif;
	if typeof(aa)==8 then aa=-2; endif;
	if typeof(cc)==8 then cc=-2; endif;
	
	// determine xmin, xmax from a and b
	allx=0;
	if typeof(xmin)==8 then xmin=aa; xmax=bb; allx=1; endif;
	ks=keepsquare(square);
	
	olds=[aa,bb,cc,dd];
	
	oldframe=_framecolor(framecolor);
	
	// loop for user interaction
	repeat
		// set the range
		if !add then
			if !auto then setplot(aa,bb,cc,dd); 
			else setplot();
			endif;
		else
			hold on;
		endif;
		
		if allx then xmin=aa; xmax=bb; endif;
		
		// own plotting routine
		if own then
			if (!add) then clg; endif;
			ccc=_color(color); l=_linewidth(thickness);
			xx(args());
			_color(ccc); _linewidth(1);
		elseif typeof(xx)==8 and typeof(yy)==8
		// curve with one or two parameter functions
			if yy<>none then
			// two functions specified
				if adaptive then
					dw=abs(xmax-xmin);
					{s,u}=adaptiveeval(xx,yy,xmin,xmax, ..
						dw/n,dw/n^2,dw/n;args());
				else
					t=linspace(xmin,xmax,n);
					s=%ploteval(xx,t;args());
					u=%ploteval(yy,t;args());
				endif;
				ccc=_color(color); l=_linewidth(thickness);
				if style!=none then oldst=_linestyle(style); endif;
				if logplot then 
					if logplot==2 then xylogplot(t,u,color=color,points=points,frame=frame);
					elseif logplot==3 then xlogplot(t,u,points=points,color=color,frame=frame);
					else ylogplot(t,u,color=color,points=points,frame=frame);
					endif;
				elseif filled then
					if (!add) then 
						if auto then 
							setplot(totalmin(s),totalmax(s),totalmin(u),totalmax(u));
						endif;
						clg; 
					endif;
					if grid and !add then 
						frame(); 
						xplot(grid=grid,ticks=ticks,frame=frame); 
					endif;
					ccf=_barcolor(fillcolor);
					ccc=_color(color);
					if style!=none then oldst=_barstyle(style); endif;
					if !add then
						if auto then plotarea(s,u); endif;
						if !holding() then clg; endif;
						if grid then 
							xplot(grid=grid,ticks=ticks,frame=frame); 
						endif;
					endif;
					h1=holding(1); polygon(s,u,outline); holding(h1);
					if style!=none then _barstyle(oldst); endif;
					_barcolor(ccf);
				else 
					if steps then {s,u}=%makesteps(s,u,steps==2); endif;
					if !add then
						if auto then plotarea(s,u); endif;
						if !holding() then clg; endif;
						if grid then 
							xplot(grid=grid,ticks=ticks,frame=frame); 
						endif;
					endif;
					h1=holding(1); plot(s,u); holding(h1);
				endif;
				if style!=none then _linestyle(oldst); endif;
				_color(ccc); _linewidth(1);
			elseif xv==none then
			// function xv is none
				if grid and !add then 
					if !auto then clg; endif;
					xplot(grid=grid,ticks=ticks,frame=frame); 
				endif;
			else
			// one function specified
				if adaptive then
					dw=abs(xmax-xmin);
					{t,u}=adaptiveevalone(xx,xmin,xmax, ..
						dw/n,dw/n^2,dw/n,auto;args());
				else
					t=linspace(xmin,xmax,n);
					u=%ploteval(xx,t;args());
					if cols(u)<>cols(t) then
						error("Cannot plot this result!"); 
					endif;
				endif;
				ccc=_color(color); l=_linewidth(thickness); 
				if style!=none then oldst=_linestyle(style); endif;
				if logplot then 
					if logplot==2 then xylogplot(t,u,points=points,color=color,frame=frame);
					elseif logplot==3 then xlogplot(t,u,points=points,color=color,frame=frame);
					else ylogplot(t,u,points=points,color=color,frame=frame);
					endif;
				else 
					if steps then {t,u}=%makesteps(t,u,steps==2); endif;
					if !add then
						if auto then plotarea(t,u); endif;
						if !holding() then clg; endif;
						if grid then 
							xplot(grid=grid,ticks=ticks,frame=frame); 
						endif;
					endif;
					h1=holding(1); 
					if filled then 
						bst=barstyle(style);
						bcl=barcolor(fillcolor);
						polygon(t[1]|t|t[-1],0|u|0,1); 
						barcolor(bcl);
						barstyle(bst);
					else plot(t,u); 
					endif;
					holding(h1);
				endif;
				if style!=none then _linestyle(oldst); endif;
				_color(ccc); _linewidth(l);
			endif;
		elseif typeof(xx)==13 then
			if adaptive then
				dw=abs(xmax-xmin);
				{t,u}=adaptiveevalone(xx[1],xmin,xmax, ..
					dw/n,dw/n^2,dw/n,auto;args());
			else
				t=linspace(xmin,xmax,n);
				u=%ploteval(xx[1],t;args());
				if cols(u)<>cols(t) then
					error("Cannot plot this result!"); 
				endif;
			endif;
			loop 2 to length(xx)
				u=u_(%ploteval(xx[#],t;args()));
			end;
			ccc=_color(color); l=_linewidth(thickness); 
			if style!=none then oldst=_linestyle(style); endif;
			if steps then {t,u}=%makesteps(t,u,steps==2); endif;
			if !add then
				if auto then plotarea(t,u); endif;
				if !holding() then clg; endif;
				if grid then 
					xplot(grid=grid,ticks=ticks,frame=frame); 
				endif;
			endif;
			h1=holding(1); plot(t,u); holding(h1);
			if style!=none then _linestyle(oldst); endif;
			_color(ccc); _linewidth(l);
		// data plots:
		else
			if typeof(yy)==8 then
				yy=xx; xx=1:cols(yy);
			elseif typeof(yy)==11 then
				xx=xx|flipx(xx); yy=left(yy)|flipx(right(yy));
				filled=1; polygon=1;
			endif;
			if points then
				if style!=none then oldst=_markerstyle(style); endif;
				if square then setplot(aa,bb,totalmin(yy),totalmax(yy)); endif;
				if logplot then
					ccc=_color(color);
					if logplot==2 then xylogplot(xx,yy,points=points,frame=frame);
					elseif logplot==3 then xlogplot(xx,yy,points=points,frame=frame);
					else ylogplot(xx,yy,points=points,frame=frame);
					endif;
					_color(ccc);
				else
					if !add then
						if auto then plotarea(xx,yy); endif;
						if !holding() then clg; endif;
						if grid then 
							xplot(grid=grid,ticks=ticks,frame=frame); 
						endif;
					endif;
					h1=holding(1); 
					if cols(color)>0 then
						markwithcolor(xx,yy,color);
					else
						ccc=_color(color);
						mark(xx,yy); 
						_color(ccc);
					endif;
					holding(h1);
				endif;
				if style!=none then _markerstyle(oldst); endif;
			elseif bar then
				ccc=_barcolor(fillcolor);
				if style!=none then oldst=_barstyle(style); endif;
				if cols(xx)==cols(yy);
					xx=xx|(2*xx[cols(xx)]-xx[cols(xx)-1]);
				endif;
				if auto and !add then
					setplot(min(xx),max(xx),1.1*min(0|yy),1.1*max(0|yy)); 
				endif;
				xxx=xx[1:cols(xx)-1];
				if !add then
					if !holding() then clg; endif;
					if grid then 
						xplot(grid=grid,ticks=ticks,frame=frame); 
					endif;
				endif;				
				h=holding(1);
				plotbar(xxx,0,xx[2:cols(xx)]-xxx,yy[1:cols(xx)-1]);
				holding(h);
				if style!=none then _barstyle(oldst); endif;
				_barcolor(ccc);
			elseif filled then
				if !add then
					if auto then plotarea(xx,yy); endif;
					if !holding() then clg; endif;
					if grid then 
						xplot(grid=grid,ticks=ticks,frame=frame); 
					endif;
				endif;
				ccf=_barcolor(fillcolor);
				ccc=_color(color);
				if style!=none then oldst=_barstyle(style); endif;
				h1=holding(1); polygon(xx,yy,outline); holding(h1);
				if style!=none then _barstyle(oldst); endif;
				_barcolor(ccf);
				_color(ccc);
			else
				ccc=_color(color); l=_linewidth(thickness); 
				if style!=none then oldst=_linestyle(style); endif;
				if logplot then
					if logplot==2 then xylogplot(xx,yy,points=points,frame=frame);
					elseif logplot==3 then xlogplot(xx,yy,points=points,frame=frame);
					else ylogplot(xx,yy,points=points,frame=frame);
					endif;
				else 
					if square then setplot(aa,bb,totalmin(yy),totalmax(yy)); endif;
					if steps then {xx,yy}=%makesteps(xx,yy,steps==2); endif;
					if !add then
						if auto then plotarea(xx,yy); endif;
						if !holding() then clg; endif;
						if grid then 
							xplot(grid=grid,ticks=ticks,frame=frame); 
						endif;
					endif;
					h1=holding(1); plot(xx,yy); holding(h1);
				endif;
				if style!=none then _linestyle(oldst); endif;
				_color(ccc); _linewidth(l);
			endif;
		endif;
		settitle(title);
		if xl<>none then xlabel(xl); endif;
		if yl<>none then ylabel(yl); endif;
		if add then holding(oldhold); endif;
		if !user then break; endif;
		gotkey=0;
		k1=_mouse("Press return, cursor key, +/-, space, click with mouse");
		if cols(k1)==1 then
			gotkey=k1;
		elseif add then
			gotkey=13;
		else
			hl=holding(1); mark(k1[1],k1[2]); holding(hl);
			k2=_mouse("Click with mouse");
			if auto then k2[2]=k1[2]; endif;
			if (cols(k2)==1) then
				gotkey=k2;
			else
				hl=holding(1); 
				mark(k2[1],k2[2]);
				{kl,k2}={min(k1,k2),max(k1,k2)};
				c=color(2);
				plot([kl[1],kl[1],k2[1],k2[1],kl[1]],[kl[2],k2[2],k2[2],kl[2],kl[2]]);
				color(c);
				holding(hl);			
				wait(0.5);
				aa=kl[1]; bb=k2[1]; cc=kl[2]; dd=k2[2];
			endif;
		endif;
		if gotkey then
			k=gotkey;
			if k==13 or k==9 then break;
			elseif k==27 then error("Interrupt"); break;
			elseif !add then
				if k==43 then
					{aa,bb}={(1-delta)*aa+delta*bb,(1-delta)*bb+delta*aa};
					{cc,dd}={(1-delta)*cc+delta*dd,(1-delta)*dd+delta*cc};
				elseif k==45 then
					{aa,bb}={(1+delta)*aa-delta*bb,(1+delta)*bb-delta*aa};
					{cc,dd}={(1+delta)*cc-delta*dd,(1+delta)*dd-delta*cc};
				elseif k==3 then
					{aa,bb}={(1+delta)*aa-delta*bb,(1-delta)*bb+delta*aa};
				elseif k==4 then
					{aa,bb}={(1-delta)*aa+delta*bb,(1+delta)*bb-delta*aa};
				elseif k==2 then
					{cc,dd}={(1+delta)*cc-delta*dd,(1-delta)*dd+delta*cc};
				elseif k==1 then
					{cc,dd}={(1-delta)*cc+delta*dd,(1+delta)*dd-delta*cc};
				elseif k==32 then
					aa=olds[1]; bb=olds[2]; cc=olds[3]; dd=olds[4];
				else
					"Press return to end, cursorkeys to pan,",
					"+, - to zoom, or space to reset.",
				endif;
			endif;
		endif;
	end;
	_framecolor(oldframe);
	if margin<>none then _margin(marginold); endif;
	clipping(oldclipping);
	keepsquare(0);
	ret=plot();
	errors on;
	if insimg then insimg(); endif;
	return ret;
endfunction

function plot3d (x, y=none, z=none, disconnect=none, ..
	xmin=-1,xmax=1,ymin=-1,ymax=1, ..
	a=none,b=none,c=none,d=none, ..
	n=40,scale=1,fscale=-1,frame=1,r=0, ..
	xlabel="x",ylabel="y",zlabel="z",polar=0, ..
	sliced=0, ..
	hue=0,light=[2,3,4],amb=0.1,max=0.9, ..
	anaglyph=0, ..
	user=0,duser=0.1, ..
	wire=0, wirecolor=none, points=0, lines=0, contour=0, .. 
	rotate=0,title="",style=".", ..
	level=none,add=0, ..
	implicit=0,zmin=-1,zmax=1, ..
	angle=none,height=none,zoom=none,distance=none,view=none, ..
	center=none, ..
	bar=0, values=none,own=0, ..
	maps=0, ..
	contourcolor=none, contourwidth=1, ..
	color=1, framecolor=none, ..
	cx=0, cy=0, cz=0, ..
	insimg=0, spectral=0, xhue=0, yhue=0, zhue=0, hues=none, ..
	sframe:real vector=none)
## General function for 3D plots.
##
## Overview: This function can plot 
##
## - the graph of a function in two variables,
##   - as a wire plot, 
##   - a solid plot, 
##   - optionally with shading, 
##   - or with level lines. 
## - surfaces, described by three matrices of x-, y-, and
##   z-coordinates, or by three functions of two parameters. 
## - curves in space, 
## - a cloud of points,
## - implicit functions,
## - analgyphs.
##
## The plot3d function uses the convention, that the x axis goes to
## the right, the y axis points into the screen, and the z axis goes
## vertically upwards. The plot can be turned, however, to the left
## and right around the z axis, and up and down. The point the plot
## looks at is the origin by default.
##
## To plot a function of x and y, use an expression string or a
## function name for the first parameter x. "plot3d" will evaluate
## the function on a grid of x and y values, and plot the graph of the
## function. The parameter n determines the number of grid lines in
## both directions. The function values will be scaled in z direction,
## but not in x or y direction. By default (fscale=-1), the scaling
## will depend on the x and y range to make the graph look good. But
## an own scaling factor can be used by setting fscale to some
## positive value. fscale=0 will use no scaling. All 3D plots can be
## scaled in all directions with the parameter scale. Instead of
## rectangle coordinates, polar coordinates can be used with polar=1.
##
## To plot a surface from data, plot3d needs the x, y and z
## coordinates of the surface. These coordinates must be stored in
## three matrices. So the parameterization of the surface is always
## rectangular. Note, that three matrices with compatible size must be
## provided, one for each coordinate of the surface. Row and column
## vectors will be expanded using the rules of the Euler matrix
## language.
##
## If three function names, or three expressions of the variables x
## and y are provided then plot3d will compute the coordinates of
## the surface using the ranges [xmin,xmax] for x or [ymin,ymax] for y.
## Again, n will be the number of partitions. 
##
## To add shading to a solid plot, use hue=1. The shading will be
## computed assuming a light from a given direction, and ambient light
## of a given brightness. Contour lines can be added to a shaded plot
## with contour=1. Contour lines at specific levels can be added with
## level=v, where v is a vector of level values. These values for
## contour and level refer to the z value of the surface. However, if
## a matrix of values is provided with values=w, then these values
## will be used to plot the contour lines at the specific levels.
## This form of level lines will work only for surfaces given by data,
## and if level=v is provided additionally. 
##
## To plot a cloud of points, use three vectors of x, y and z
## coordinates, and set points=1. You can set the point style with
## style, which must be one of "[]", "<>", ".", "..", "*", "+", or
## "o". For a big lot of points, it looks good to use ".", and
## optionally view the cloud in anaglyph mode (anaglyph=1). For a
## point cloud with points in different colors, the color parameter
## can be a row vector of colors. In this case, the points in the back
## are drawn before the points in the front.
##
## A surface can also be drawn as a wire frame plot with wire=1. To
## plot one curve or several curves in 3D, either provide three
## vectors of data containing the x, y and z parameters of the curve,
## or three functions or expressions in the variable x, and set
## lines=1. In the case of three functions, the range [xmin,xmax] is
## used. To plot several curves at once, use three matrices, with one
## row for each curve, and specify lines=1.
##
## Functions of one variable rotated around the x axis can be plotted
## with rotate=1. In this case, plot3d needs an expression in x, or a
## function f(x), and the range is [xmin,xmax]. The function can also
## be rotated around the z axis (rotate=2). In this case use a
## non-negative xmin.
##
## Implicit plots show the solutions of f(x,y,z)=0 with implicit=1.
## The function needs an expression in x, y, and z, or the name of a
## function f. Cuts trough this set orthogonal to an axis will be
## drawn. Use 1 for the x-axis, 2 for the y-axis and 4 for the z-axis.
## You can add these values to show cuts in several directions.
## Implicit plots are the slowest of the 3D plots. You can speed up
## the plot using expressions, which can handle matrices, with
## maps=0.
##
## Plots can be interactive with user=1. In this case the user can
## turn the plot with the cursor keys, and zoom the plot with +/-. The
## space key will reset the default view. For solid plots with light,
## the key l will toggle to the movement of the light. The center of
## the plot can also be moved in four directions. You can toggle the
## movement of the plot center with the key c.
##
## All plots can be anaglyphs. Use red/cyan glasses to see the rather
## nice 3D effect. Either enter the paparemter analgyph=1, or press a
## in interactive plots.
##
## Note: Functions are mapped automatically in plot2d. Expressions
## must be able to map to their arguments. So use plot("f",...) not
## plot("f(x,y)",...), if "f" does not map.
##
## Parameters
##
## x : expression in x and y
## x,y,z : matrices of the coordinates of a surface
## x,y,z : expressions in x and y for a parametric surface
## x,y,z : expressions in x to plot a line
## xmin,xmax,ymin,ymax : for plots over rectangles
## a,b,c,d : alternative for xmin,xmax,ymin,ymax
## r : a radius can be used instead of xmin,xmax,ymin,ymax
## cx, cy, cr : center of plot, if r is used
## r : also used for polar plots
## n : accuracy
## scale : scale plot to this size (0 = no scaling, 1 = default)
##         if scale is a 1x3 vector, it will scale in each direction
## fscale : scales the values for function to this range (0=no, -1=auto)
##
## wire : wire plot
## polar : polar plot
## points : cloud of points
##
## sliced : plot a scliced version (0=no, 1=x-direction, 2=y-direction)
## hue : compute shading
## light, amb, max : controls the shading
## contour : set automatic level (1=solid hue, 2=only contour lines)
## spectral : use spectral colors instead of monochrome hue
## xhue,yhue,zhue : use these coordinates instead of default lighting
## hues : a matrix of hue values from 0 to 1
## level : 2D level plot of these levels
## add : adds a level plot to another
##
## user : user can turn the plot around with the keyboard
## duser : controls the user action
##
## lines : a line given by three expressions
## rotate : rotation plot, the function must be of one variable
##
## anaglyph : 3d plot
##
## viewangle : changes the default angle of view
## viewheight : changes the height of view
## zoom : changes the zoom of view
## distance : changes the distance of view
## view : sets the default view
##
## implicit : for implict plots of f(x,y,z)
##            0=off, 1=x, 2=y, 4=z, can be combined by adding
## values : values to be used for contour plots (default: z-values)
##
## style : plot style for markers (see plot2d for available styles)
##
## maps : expressions should be mapped (faster, if 0), functions
##        are always mapped.
##
## contourcolor : color of contour lines
## contourwidth : width of contour lines
## color : color for markers and for shaded surfaces
## wirecolor : color for wire plots
##
## frame : If 0, no frame is drawn. If this is a vector, it must be of
## the form [xmin,xmax,ymin,ymax,zmin,zmax] and will be used as a
## frame for plots. The plot must fit into the frame then.
## framecolor : color of the 3D frame
##
## sframe : Sets the tick values for the axes. This is a vector of the
## same form as frame.
##
## Some settings are global
##
## fillcolor(11,3) : fill color for 3d surfaces (no hue).
## twosides(1) : show two different colors.
##
## Examples:
##
## >plot3d("x*y",r=1,title="z=x*y");
## >plot3d("x*y^2",>user,r=1,title="Press cursor keys or return!");
## >plot3d("x^2*y^3",r=0.9,zlabel="x^2*y^3",angle=30,height=20,zoom=3);
## >plot3d("x^2+y^3",angle=0,>contour,>spectral);
## >plot3d("x^y-y^x",a=0,b=4,c=0,d=4,angle=40,level=0,n=100);
## >plot3d("x^2+y^2",>wire,>anaglyph,title="Use Red/Cyan Glasses!");
##
## >plot3d("x^2+y^3+sin(z)^2-1",r=pi,implicit=4,zoom=3);
##
## >x=-1:0.05:1; y=x';
## >plot3d(x,x*y^2,y,>hue,angle=20);
## >h=x^2+y^2;
## >plot3d(x,y,(y-x)/2,level=-2:0.1:2,values=h,hues=h/2,>spectral);
##
## >X=normal(3,1000);
## >plot3d(X[1],X[2],X[3],>points,zoom=3,>user,title="Press return or cursor key!");
## >Y=cumsum(X);
## >plot3d(Y[1],Y[2],Y[3],>wire,>user,zoom=3,title="Press return or cursor key!");
##
## >t=linspace(0,2,1000);
## >plot3d(sin(2pi*t),cos(2pi*t),t,>wire,>anaglyph);
##
## See: plot2d
	if wirecolor==none then wirecolor=_wirecolor(); endif;
	if contourcolor==none then contourcolor=_contourcolor(); endif;
	if framecolor<>none then fc=frame3dcolor(framecolor); endif;
	if isvar("niveau") then level=niveau; endif;
	
	errors off; // allow plot areas out of the definition set of the function
	global defaultanaglyphseparation;
	oldcenter=_center();
	if center!=none then center(center); endif;
	resetcenter=center();
	xx=x; yy=y; zz=z; wwire=wire;
	if a<>none then xmin=a; 
		if b<>none then xmax=b;
			if c<>none then ymin=c;
				if d<>none then ymax=d;
				endif;
			endif;
		endif;
	endif;
	if r!=0 then 
		xmin=cx-r; xmax=cx+r; ymin=cy-r; ymax=cy+r; zmin=cz-r; zmax=cz+r;
	endif;
	if polar and r==0 then r=1; endif;
	if fscale==-1 then 
		if r>0 then fscale=r;
		else fscale=min(xmax-xmin,xmax-xmin)*0.5; 
		endif;
	endif
	if contour and level==none then level="auto"; endif;
	if !(level==none) then hue=1; endif;
	if contour==2 then hue=-1; endif;
	if user==1 then hold off; endif;
	if typeof(x)==8 and typeof(y)==8 and y<>none and typeof(z)==8 and z<>none then
		if lines then
			t=linspace(xmin,xmax,n);
			xx=%ploteval(xx,t;args());
			yy=%ploteval(yy,t;args());
			zz=%ploteval(zz,t;args());
			wwire=1;
		else
			lx=linspace(xmin,xmax,n);
			ly=linspace(ymin,ymax,n);
			{xa,ya}=field(lx,ly);
			xx=%ploteval2(xx,xa,ya;args());
			yy=%ploteval2(yy,xa,ya;args());
			zz=%ploteval2(zz,xa,ya;args());
		endif;	
	endif
	if typeof(xx)==8;
		f=xx;
	else
		f=none;
	endif;
	if rotate then
		xxt=linspace(xmin,xmax,n);
		r=%ploteval(f,xxt;args());
		t=linspace(0,2*pi,n)';
		if rotate==1 then 
			xx=xxt; yy=cos(t)*r; zz=sin(t)*r;
		else
			zz=r; xx=cos(t)*xxt; yy=sin(t)*xxt;
		endif;
		f=none;
	endif;
	vold=view();
	if view<>none then view(view); endif;
	if zoom<>none then zoom(zoom); endif;
	if distance<>none then viewdistance(distance); endif;
	if angle<>none then viewangle(angle); endif;
	if height<>none then viewheight(height); endif;
	vreset=view();
	v=view();	
	movelight=0;
	movecenter=0;
	if hue then
		{phil,psil,rl}=polar(light[1],light[2],light[3]);
		philorg=phil; psilorg=psil; rlorg=rl;
	endif;
	if anaglyph then 
		agmode=1; deletepages(); addpage(); addpage();
	else 
		agmode=0; 
	endif;
	if spectral then
		hue=-4; color=-2;
	endif;
	if xhue then hue=-2; endif;
	if yhue then hue=-3; endif;
	if zhue then hue=-4; endif;
	if hues!=none then hue=1; endif;
	xyzframe=none;
	if length(frame)>1 then xyzframe=frame; frame=1; endif;
	repeat
		if own then
			f(args());
		elseif f==none // no function for x provided
			if yy==none and zz==none then // only one matrix
				zz=xx; xx=1:cols(zz); yy=(1:rows(zz))';
			endif;
			if hue
				hc=huecolor(color);
				framedsolidhue(xx,yy,zz,hues, ..
					scale,1,frame,1,xlabel,ylabel,zlabel,level,values, ..
					light,amb,max,hue,contourcolor,contourwidth, ..
					frame=xyzframe,sframe=sframe);
				huecolor(hc);
			else
				if wwire or lines then
					framedwire(xx,yy,zz,scale,frame,1,xlabel,ylabel,zlabel, ..
						wirecolor,lines, ..
						frame=xyzframe,sframe=sframe);
				elseif points then
					ms=markerstyle(style);
					framedmark3(xx,yy,zz,scale,frame,1,xlabel,ylabel,zlabel,color, ..
						frame=xyzframe,sframe=sframe);
					markerstyle(ms);
				else
					framedsolid(xx,yy,zz,scale,frame,1,disconnect, ..
						xlabel,ylabel,zlabel,sliced,bar, ..
						frame=xyzframe,sframe=sframe);
				endif;
			endif
		else // function for x provided
			if hue
				if polar
					hc=huecolor(color);
					f3dplotpolarl(f,r,floor(n/2),scale,fscale,light, ..
						amb,max,frame,xlabel,ylabel,zlabel,level,hue,maps;args());
					huecolor(hc);
				else
					hc=huecolor(color);
					f3dplotlight(f,xmin,xmax,ymin,ymax,n,scale,fscale,light, ..
						amb,max,frame,xlabel,ylabel,zlabel,level,hue,maps, ..
						contourcolor,contourwidth;args());
					huecolor(hc);
				endif;
			else
				if polar
					f3dplotpolar(f,r,floor(n/2),scale,fscale,frame, ..
						wire,xlabel,ylabel,zlabel,maps;args());
				elseif implicit then
					xv=linspace(xmin,xmax,n);
					yv=linspace(ymin,ymax,n);
					zv=linspace(zmin,zmax,n);
					f3dimplicit(f,xv,yv,zv,frame, ..
						xlabel,ylabel,zlabel,scale,implicit,maps;args());
				else
					f3dplot(f,xmin,xmax,ymin,ymax,n,scale,fscale,frame, ..
						wire,xlabel,ylabel,zlabel,sliced,maps,wirecolor;args());
				endif;
			endif;
		endif;
		settitle(title);
		v=view();
		if agmode==1 then
			copytopage(1);
			setanaglyph(defaultanaglyphseparation/max(v[1],4));
			agmode=2;
		else
			if agmode==2 then
				copytopage(2); showpage(-1);
				setanaglyph(0);
				if anaglyph then agmode=1; else agmode=0; endif;
			endif;
			if !user then break; endif;
			k=key("Press return, cursor keys, +/-, space, l (light), c (center), a (anaglyph)");
			if k==13 or k==9 then break; endif;
			oc=center();
			if k==2 then
				if movelight then psil=psil-duser;
				elseif movecenter then oc[3]=oc[3]-duser;
				else v[4]=v[4]-duser;
				endif;
			elseif k==1 then 
				if movelight then psil=psil+duser;
				elseif movecenter then oc[3]=oc[3]+duser;
				else v[4]=v[4]+duser;
				endif;
			elseif k==4 then 
				if movelight then phil=phil+duser;
				elseif movecenter then
					oc[1]=oc[1]+cos(v[3])*duser;
					oc[2]=oc[2]+sin(v[3])*duser;
				else v[3]=v[3]+duser;
				endif
			elseif k==3 then
				if movelight then phil=phil-duser;
				elseif movecenter then
					oc[1]=oc[1]-cos(v[3])*duser;
					oc[2]=oc[2]-sin(v[3])*duser;
				else v[3]=v[3]-duser;
				endif
			elseif k==32 then 
				v=vreset;
				if hue then
					phil=philorg; psil=psilorg; rl=rlorg;
					{xl,yl,zl}=rect(phil,psil,rl);
					light=[xl,yl,zl];
				endif;
				movelight=0;
				movecenter=0;
				oc=resetcenter; _center(resetcenter);
			elseif k==7 then v[3]=0;
			elseif k==6 then v[4]=0;
			elseif k==43 then 
				v[1]=v[1]-duser; 
				if (v[1]<1) then v[1]=1; endif;
			elseif k==45 then v[1]=v[1]+duser;
			elseif k==ascii("a") and !anaglyph then
				agmode=1; deletepages(); addpage(); addpage(); 
			elseif k==ascii("l") then
				if hue then movelight=!movelight; endif;
			elseif k==ascii("c") then
				movecenter=!movecenter;
			else
				"Press return to end, cursorkeys to turn,",
				"+, - to zoom, a for anaglyph, space to reset,",
				"l to toggle moving the light,",
				"c to toggle moving the center of the plot.",
				if key()==13; break; endif;
			endif;
			if movelight then
				{xl,yl,zl}=rect(phil,psil,rl);
				light=[xl,yl,zl];
			elseif movecenter then
				center(oc);
			endif
		endif;
		view(v);
	end;
	if framecolor<>none then frame3dcolor(fc); endif;
	if agmode then setanaglyph(0); endif;
	_center(oldcenter);
	view(vold);
	errors on;
	if insimg then insimg(); endif;
endfunction
