// Basic Plot Functions

comment
Help functions for 2D and 3D plots.
endcomment

// It should not be necessary to call these functions directly, unless
// a special effect is needed. You can use plot2d or plot3d instead.
// Have a look at the following notebook.

// See: plot | Functions plot2d and plot3d

// There are also some core functions, which are not explained here.
// Look at the following documenation, if you need those.

// See: eulercore.html#Plots | Core Functions for Plots

// Moreover, some of the following functions link to core functions
// for further details.

// * Ultility Functions

logredraw=1;
subwindows=zeros([100,4]);
usesubwindows=0;

function overwrite window
## window(c1,r1,c2,r2) sets a plotting window. 
## The coordinates must be screen coordinates.
## window() returns the active window coordinates.
## See: coordinates
	global logredraw;
	if argn()==4; logredraw=1; return _window([arg1,arg2,arg3,arg4]);
	else if argn()==1; logredraw=1; return _window(arg1);
	else return _window();
	endif;
endfunction

function fullwindow
## Takes the full size (besides a title) for the plots
## Should only be used for plotRGB
## See: plotRGB, shrinkwindow, reset
	global logredraw;
	h=textheight(); b=textwidth();
	logredraw=1;
	return _window([b*2,h*1.5,1011,1011-b*2]);
endfunction

function shrinkwindow
## shrinkwindow() shrinks the window to allow labels.
## Should only be used for plotRGB after fullwindow
## See: plotRGB, fullwindow, reset
	global logredraw;
	h=textheight(); b=textwidth();
	logredraw=1;
	return _window([7*b,1.5*h,1023-0.5*b,1023-3*h]);
endfunction

function overwrite setplot
## setplot(xmin xmax ymin ymax) sets the plot coordinates and holds the plot 
## Also setplot([xmin,xmax,ymin,ymax]).
## setplot() resets it. 
	if argn()==4; return _setplot([arg1,arg2,arg3,arg4]);
	else;
		if argn()==0; scaling(1); 
		else; _setplot(arg1);
		endif;
	endif;
	return plot();
endfunction

function unclip
## Clips to the complete graphics screen
	return _clip([0,0,1023,1023]);
endfunction	

function overwrite clip
## clip(c1,r1,c2,r2) sets the clipping window. 
## The coordinates must be screen coordinates.
## clip() returns the active clipping window coordinates.
## See: clip
	if argn()==4; return _clip([arg1,arg2,arg3,arg4]);
	else if argn()==1; return _clip(arg1);
	else return _clip();
	endif;
endfunction

function clipmore (x,more=0)
## Sets the clipping window. 
## The coordinates must be screen coordinates.
## There is some extra space around the area x
## clip() returns the active clipping window coordinates.
## See: clip
	return _clip([x[1]-more,x[2]-more,x[3]+more,x[4]+more]);
endfunction

function overwrite args toscreen (x,y=none)
## Convert the x-y-coordinates screen coordinates,
## from 0..1024. Useful for Labels.
	if y==none then return _toscreen(x);
	else
		c=x; r=y;
		loop 1 to prod(size(x))
			cr=_toscreen([x{#},y{#}]);
			c{#}=cr[1]; r{#}=cr[2];
		end;
		return {c,r};
	else error("Need x,y for toscreen");
	endif;
endfunction

function overwrite args fromscreen (c,r=none)
## Convert the screen coordinates v[1],v[2] to x-y-coordinates,
## Returns {x,y}
	if r==none then return _fromscreen(c);
	else
		x=c; y=c;
		loop 1 to prod(size(c))
			xy=_fromscreen([c{#},r{#}]);
			x{#}=xy[1]; y{#}=xy[2];
		end;
		return {x,y};
	else error("Need c,r for fromscreen");
	endif;
endfunction

function label (t:string, x:number, y:number, offset:number=10, ..
	color:number=none)
## Label the x-y-value with the text t.
## offset is in screen coordinates 0..1024.
	if color==none then color=textcolor(); endif;
	c=_toscreen([x,y]); 
	c[1]=c[1]+textwidth()/4+offset; c[2]=c[2]+offset;
	old=textcolor(color);
	ret=text(t,c);
	textcolor(old);
	return ret
endfunction

function labelbox (labels, styles="-", colors:real=1,
	x=0.98, y=0.02,	w=none, points=0, tcolor=red)
## Plot a box with labels and line styles for the functions.
##
## A label box is a list of labels for each function in the plot. The
## box shows a string and a line in the style and color of the
## function for each function. For point plots the box can show point
## styles.
##
## labels : string, or vector of strings
## styles : string or string of vector, styles of the functions
## colors : real vector, colors of the functions
## w : width of the box (fraction of plot window)
## x, y : upper right corner of label box (fractions)
## points : flag or vector of flags.
##
## Example:
## >plot2d("x+x^3",-1,1); plot2d("x^4",color=blue,style="--",>add);
## >labelbox(["x+x^3","x^4"],colors=[black,blue],styles=["-","--"]):
##
## See: label
	lmax=0;
	loop 1 to length(labels); lmax=max(lmax,strlen(labels{#})); end;
	ts=textsize(); twidth=(ts[1]+0.5)*lmax;
	
	win=window(); ww=win[3]-win[1]; wh=win[4]-win[2];
	th=ts[2]; h=length(labels)*(th*1.2)+th*0.8;
	
	if w==none then
		if all(points) then w=twidth+ts[1]*6;
		else w=twidth+ts[1]*15;
		endif; 
	else
		w=ww*w;
		if all(points) then twidth=w-6*ts[1];
		else twidth=w/2;
		endif;
	endif;
		
	x=win[1]+ww*x-w; y=win[2]+ww*y;
	
	bst=barstyle("O#"); bcol=barcolor(0);
	bar([x,y,w,h]);
	barstyle(bst); barcolor(bcol);
	
	xt=x+w-ww/60-twidth; yt=y+th/2;
	loop 1 to length(labels);
		tc=textcolor(tcolor{#});
		text(labels{#},[xt,yt]);
		textcolor(tc);
		yt=yt+th*1.2;
	end;
	
	xt=x+ww/60; yt=y+th;
	loop 1 to length(labels);
		if points{#} then
			lc=color(colors{#}); ms=markerstyle(styles{#});
			v=fromscreen([x+w-ww/30-twidth-ts[1],yt]); 
			hold on; mark(v[1],v[2]); hold off; 
			color(lc); markerstyle(ms); 
			yt=yt+th*1.2;
		else
			lc=color(colors{#}); ls=linestyle(styles{#});
			v1=fromscreen([xt,yt]);
			v2=fromscreen([x+w-ww/30-twidth-ts[1],yt]); 
			hold on; plot([v1[1],v2[1]],[v1[2],v2[2]]); hold off; 
			color(lc); linestyle(ls); 
			yt=yt+th*1.2;
		endif;
	end;
endfunction

function textbox (text, x=0.98, y=0.02, tcolor=red, w=none)
## Plot a box with text.
##
## x,y : An anchor for the upper left corner of the box (fractions of
## plot area)
## text : A string or a vector of strings, one for each line of text
## tcolor : a color or an array of colors for the text lines
## w : an optional width of the box (fraction of plot area)
##
## See: label
	lmax=0;
	loop 1 to length(text); lmax=max(lmax,strlen(text{#})); end;
	ts=textsize(); twidth=(ts[1]+2)*lmax;
	
	win=window(); ww=win[3]-win[1]; wh=win[4]-win[2];
	th=ts[2]; h=length(text)*(th*1.2)+th*0.8;
	
	if w==none then w=twidth; 
	else w=ww*w;
	endif;
	
	x=win[1]+ww*x-w; y=win[2]+ww*y;
	
	bst=barstyle("O#"); bcol=barcolor(0);
	bar([x,y,w,h]);
	barstyle(bst); barcolor(bcol);
	
	xt=x+ts[1]; yt=y+th/2;
	loop 1 to length(text);
		tc=textcolor(tcolor{#});
		text(text{#},[xt,yt]);
		textcolor(tc);
		yt=yt+th*1.2;
	end;
	
endfunction

// * Bars

function plotbar1 (x:number,y:number,w:number,h:number)
## Plots a single bar with x,y,w,h in plot coordinates.
## use barstyle and barcolor to modify the bar.
## The parameters may be vectors.
	a=_toscreen([x,y+h]);
	b=_toscreen([x+w,y]);
	bar(a|(b-a));
	return 0;
endfunction

function plotbar (x,y,w,h)
## Plots a bars with x,y,w,h in plot coordinates.
## use barstyle and barcolor to modify the bar.
## The parameters may be vectors.
	if !holding(); clg; endif;
	oldclip=clipmore(window());
	map("plotbar1",x,y,w,h);
	clip(oldclip);
	return plot();
endfunction

function xplotbar (x,y,w,h,st:string="#O")
## Plots a bar with x,y,w,h in plot coordinates.
## Uses the plot coordinates from the last plot!
## Use setplot to set new plot coordinates.
## Draws coordinates below the bar.
## use barstyle and barcolor to modify the bar.
## See: plot2d
	if !holding() then clg; endif;
	frame(); xplot();
	stold=barstyle(st);
	hold; plotbar(x,y,w,h); hold;
	barstyle(stold);
	return plot();
endfunction;

function xplotrange (r:real vector, v:real vector)
## Plots a barplot of the multiplicities v[i] in the ranges r[i],r[i+1]. 
##
## Obsolete. Use plot2d with the histogram=1 option.
##
## See: plot2d
	s=scaling(1); 
	if s;	
		setplot(min(r),max(r),0,max(v));
	endif;
	scaling(s);
	n=cols(v);
	xplotbar(r[1:n],0,r[2:n+1]-r[1:n],v);
	return plot();
endfunction

function scalematrix(A)
## Scales a matrix A, so that its value are in the range from 0 to 1.
	e=extrema(A)'; mi=min(e[1]); ma=max(e[3]);
	if ma~=mi; ma=mi+1; endif;
	return (A-mi)/(ma-mi);
endfunction	

function args select
## Coordinates {x,y} of mouse clicks, stops if user clicks above the window.
## until the user clicked above the plot window.
## See: mouse
	p=plot();
	x=[];
	repeat
		m=mouse();
		if m[2]>p[4]; break; endif;
		h=holding(1); mark(m[1],m[2]); holding(h);
		x=x_m;
	end;
	x=x'; return {x[1],x[2]}
endfunction

// * Text

function settitle (text:string)
## settitle(text) plots a title to the grafik window.
	return title(text);
endfunction

function textheight
## Height of a letter.
	h=textsize();
	return h[2];
endfunction

function textwidth
## Width of a letter.
	h=textsize();
	return h[1];
endfunction

function ticks (aa=0,bb=0)
## Returns the proper ticks to be used for intervals [a,b]
## and the factor f of the ticks.
## See: xplot
	if argn==2 then a=aa; b=bb;
	elseif argn==1 then a=min(aa); b=max(aa);
	else error("Wrong arguments for ticks");
	endif;
	if (b>1e30); b=1e30; endif;
	if (a<-1e30); a=-1e30; endif;
	if (a>=b); b=a+1; endif;
	w=window();
	// if norm([w[3]-w[1],w[4]-w[2]])<500 then f=0.
	tick=10^floor(log(b-a)/log(10)-0.3);
	tick1=tick;
	repeat
		while b-a > 5*tick1;
		tick1=tick1*2; 
	end;
	repeat
		while b-a < 2*tick1;
		tick1=tick1/2; 
	end;
	if abs(a)>0.00001 or abs(b)>0.00001 then tick=1; endif;
	res=(round(a/tick1,0):round(b/tick1,0))*tick1;
	return {res[nonzeros(res>=a-tick1*0.1 && res<=b+tick1*0.1)],tick}
endfunction

function xplot (x=0,y=0,grid=2,ticks=1,frame=1)
## Works like plot, but shows axis ticks.
## xplot() shows only axis ticks and the grid.
## See: plot, plot2d
	if argn()>0 then
		if argn()==1 then
			if iscomplex(x) then y=im(x); xh=re(x);
			else y=x; xh=1:cols(y);
			endif;
		else; xh=x;
		endif;
		p=plotarea(xh,y);
		if !holding() and frame then clg; frame(); endif;
	else
		p=plot();
	endif;
	lw=linewidth(1);
	{t,f}=ticks(p[1],p[2]);
	ylevel=none;
	xlevel=none;
	if grid==3 then
		grid=1;
		plevel=_toscreen([0,0]);
		xlevel=plevel[1];
		ylevel=plevel[2];
		w=window();
		if xlevel<w[1] or xlevel>w[3] then xlevel=none; endif;
		if ylevel<w[2] or ylevel>w[4] then ylevel=none; endif;
	endif;
	xgrid(t,f,grid,ticks,ylevel=ylevel);
	{t,f}=ticks(p[3],p[4]);
	ygrid(t,f,grid,ticks,xlevel=xlevel);
	linewidth(lw);
	if argn()>0 then
		ho=holding(1); plot(xh,y); holding(ho);
	endif;
	if frame then frame(); endif;
	return p;
endfunction

function xmark (x=0,y=0,grid=2,ticks=1)
## Works like mark, but shows axis ticks.
## xmark() shows only axis ticks and the grid.
## See: mark, plot2d
	if !holding(); clg; frame(); endif;
	if argn()==1;
		if iscomplex(x); y=im(x); xh=re(x);
		else; y=x; xh=1:cols(y);
		endif;
	else; xh=x;
	endif;
	p=plotarea(xh,y);
	{t,f}=ticks(p[1],p[2]);
	xgrid(t,f,grid,ticks);
	{t,f}=ticks(p[3],p[4]);
	ygrid(t,f,grid,ticks);
	ho=holding(1); p=mark(xh,y); holding(ho);
	return p;
endfunction

// * Mouse

function setplotm
## The user can choose the plotting coordinates with the mouse
## Returns plot coordinates.
	h=holding(1);
	k1=mouse(); mark(k1[1],k1[2]);
	k2=mouse(); mark(k2[1],k2[2]);
	kl=min(k1,k2); ku=max(k1,k2);
	c=color(2);
	plot([kl[1],kl[1],ku[1],ku[1],kl[1]],[kl[2],ku[2],ku[2],kl[2],kl[2]]);
	color(c);
	setplot(kl[1],ku[1],kl[2],ku[2]);
	holding(h);
	return plot();
endfunction

function %printscale (x)
	if (abs(x)>10000) || (abs(x)<0.00001);
		return printf("%12.5e",x);
	else
		return printf("%10.5f",x);
	endif;
endfunction

function niceform (x,n=10)
## Return a string, containing a nicely formatted of x
	y=round(x,n);
	if abs(y)<10^(-10); y=0; endif;
	return printf("%g",y);
endfunction

defaultgrid1="--";
defaultgrid2=".";
defaultgridcolor=3;

function gridstyle (st1:string="--", st2:string=".", color=3)
## Set the grid styles and color.
## Returns both styles and the old color.
## See: linestyle, rgb
	global defaultgrid1,defaultgrid2,defaultgridcolor;
	res1=defaultgrid1;
	defaultgrid1=st1;
	res2=defaultgrid2;
	defaultgrid2=st2;
	res3=defaultgridcolor;
	defaultgridcolor=color;
	return {res1,res2,res3}
endfunction

function gridcolor (c:integer=3)
## Set the grid color.
## Returns the current color
## See plot2d,rgb
	global defaultgridcolor;
	res=defaultgridcolor;
	defaultgridcolor=c;
	return res;
endfunction

function xgrid (xx, f=1, grid=2, ticks=1, color=none, ..
	xt=none, ylevel=none)
## Draws vertical grid lines on the plot window at x0,x1,...
## xgrid([x0,x1,...],f) additionally writes x0/f to the axis.
##
## f : factor (like 10^5)
## grid : 2 for normal grid (see plot2d)
## ticks : write labels below the axis
## color : tick color
## xt : use these numbers for the grid
## ylevel : draw the grid at this y level
##
## The default values for the gridstyles are defaultgrid1,
## defaultgrid2,defaultgridcolor
##
## See: ygrid, plot2d
	global defaultgrid1,defaultgrid2,defaultgridcolor;
	if color==none then color=defaultgridcolor; endif;
	if (grid==5) then c=_fullplotarea();
	else c=_plot(); 
	endif;
	n=cols(xx); s=scaling(0); h=holding(1);
	w=window();
	st=linestyle(defaultgrid2); 
	wi=linewidth(1); oldcolor=color(color);
	ht=textheight();
	if xt==none then xt=xx; endif;
	str="";
	ee=(c[2]-c[1])/1000;
	loop 1 to n;
		x=xx[#];
		col=toscreen([x,c[3]])[1];
		if x<=c[2]+ee and x>=c[1]-ee then
			if grid<>4 then
				if abs(x)<1e-10 then
					linestyle(defaultgrid1); 
					_plot([x,x],[c[3],c[4]]); 
					linestyle(defaultgrid2);
				elseif grid==2 or grid==5 then
					_plot([x,x],[c[3],c[4]]); 					
				endif
			endif;		
			if ticks and grid then
				stra=niceform(xt[#]/f);
				if stra!=str or #==n then
					if ylevel==none	then
						_ctext(stra,[col,w[4]+0.4*ht]);
					else
						_ctext(stra,[col+0.5*ht,ylevel+0.4*ht]);
					endif;
					str=stra;
				endif
			endif;
		endif;
	end;
	if ticks and grid==2 and !(f~=1) then
		_rtext("* "|%printscale(f),[w[3],w[4]+1.5*ht]);
	endif;
	linestyle(st); linewidth(wi); color(1); holding(h); scaling(s);
	color(oldcolor);
	return 0;
endfunction

function ygrid (yy, f=1, grid=2, ticks=1, color=none, ..
	yt=none, xlevel=none)
## Draws horizontal grid lines on the plot window at y0,y1,...
## ygrid([x0,x1,...],f) additionally writes x0/f to the axis.
##
## Parameters similar to xgrid
##
## See: xgtrid, plot2d
	global defaultgrid1,defaultgrid2,defaultgridcolor;
	if color==none then color=defaultgridcolor; endif;
	global verticallabels
	if (grid==5) then c=_fullplotarea();
	else c=_plot(); 
	endif;
	n=cols(yy); s=scaling(0); h=holding(1);
	st=linestyle(defaultgrid2); 
	wi=linewidth(1); oldcolor=color(color);
	w=window(); wd=textwidth(); ht=textheight();
	if yt==none then yt=yy; endif;
	ee=(c[4]-c[3])/1000;
	loop 1 to n;
		y=yy[#];
		row=toscreen([c[1],y])[2];
		if (y>=c[3]-ee)&&(y<=c[4]+ee);
			if grid<>4 then
				if abs(y)<1e-10 then
					linestyle(defaultgrid1); 
					_plot([c[1],c[2]],[y,y]); 
					linestyle(defaultgrid2);
				elseif grid==2 or grid==5 then
					_plot([c[1],c[2]],[y,y]); 					
				endif
			endif;		
			if ticks and grid then
				if typeof(yt)==8 or typeof(yt)==13 then str=yt{#};
				else str=niceform(yt[#]/f);
				endif;
				if xlevel==none then
					if verticallabels then
						_vctext(str,[w[1]-0.5*ht,row]);
					else
						_rtext(str,[w[1]-wd,row-ht/2]);
					endif;
				elseif not y~=0 then
					if verticallabels then
						_vctext(str,[xlevel+ht,row]);
					else
						_text(str,[xlevel+0.5*ht,row-ht/2]);
					endif;
				endif
			endif;
		endif;
	end;
	if ticks and grid==2 and !(f~=1) then
		_text("* "|%printscale(f),[w[1]-6*wd,0]);
	endif;
	linestyle(st); linewidth(wi); color(1); holding(h); scaling(s);
	color(oldcolor);
	return 0;
endfunction

function xrange()
## Writes the range of x below the x axis
	c=plot(); s=scaling(0); h=holding(1);
	w=window();
	ht=textheight();
	text(printf("%0.10g",c[1]),[w[1],w[4]+0.2*ht]);
	rtext(printf("%0.10g",c[2]),[w[3],w[4]+0.2*ht]);
	holding(h); scaling(s);
	return 0;
endfunction

function xlabel (text,l=1)
## Puts the label text at the x-axis at point l in [0,1]
	w=window();
	ht=textheight();
	_ctext(text,[(w[1]+w[3])/2,w[4]+1.5*ht]);
	return 0;
endfunction

function yrange()
## Writes the range of y besides the y axis.
	c=plot(); s=scaling(0); h=holding(1);
	w=window();
	wd=textwidth();
	ht=textheight();
	rtext(niceform(c[4]),[w[1]-wd/2,w[2]]);
	rtext(niceform(c[3]),[w[1]-wd/2,w[4]-ht]);
	holding(h); scaling(s);
	return 0;
endfunction

function ylabel (text,l=1)
## Puts the label text at the x-axis at point l in [0,1]
	w=window();
	wd=textwidth();
	ht=textheight();
	_vcutext(text,[w[1]-8*wd+5,(w[2]+w[4])/2]);
	return 0;
endfunction

function plotwindow
## Sets the plot window to the screen coordinates.
	w=window();
	setplot(w[1],w[3],w[2],w[4]);
	return plot()
endfunction

// Windows

function upperwindow (title="")
## Select the upper half of the plot window for a plot
## See: figure
	window(150,860,75,500);
	title(title);
	return title
endfunction

function lowerwindow (title="")
## Select the lower half of the plot window for a plot
## See: figure
	window(150,860,575,1010);
	ctext(title,512,510);
	return title
endfunction

// * Adaptive Functions

function args adaptiveeval(f,g,a,b,eps=0.01,amin=0.00001,amax=0.1)
## Compute f(x), g(x) for t in [a,b] with adaptive step size.
## eps is the target accuracy.
## amin and amax are the minimal and maximal step size.
## amax is also the initial step size.
## f and g can be expressions in x or user defined functions.
## Returns {x,y}
##
## Used by plot2d with the adaptive=1 parameter.
## See: plot2d
// other direction
	if b<a then return adaptiveeval(f,g,b,a,eps,amin,amax); endif;
	t=a;
	sx=f(t;args());
	sy=g(t;args());
	x=zeros(rows(sx),1000); y=zeros(rows(sy),1000); 
	if cols(sx)>1 then 
		error(f|" does not produce a real or column vector"); 
	endif
	if cols(sy)>1 then 
		error(g|" does not produce a real or column vector"); 
	endif
	n=1; h=amax;
	x[:,1]=sx; y[:,1]=sy;
	repeat
		repeat
			tn=t+h;
			if tn>b then tn=b; endif;
			sxn=f(tn;args());
			syn=g(tn;args());
			d=sqrt((sxn-sx)^2+(syn-sy)^2);
			if any(isNAN(d)) then break; endif;
			if d>eps and h>amin; then h=max(h/2,amin);
			elseif d<eps/4;	then h=min(h*2,amax); break;
			else break;
			endif;
		end;
		n=n+1;
		if n>cols(x) then
			x=x|x;
			y=y|y;
		endif;
		x[:,n]=sxn; y[:,n]=syn;
		sx=sxn; sy=syn;
		t=tn;
		if t>=b; then break; endif;
	end;
	return {x[:,1:n],y[:,1:n]};
endfunction

function %ploteval (f:string, x:vector)
	y0=f(x[1],args());
	if cols(y0)>1 then 
		error(f|" does not produce a real or column vector"); 
	endif
	y=zeros(rows(y0),cols(x));
	y[:,1]=y0;
	loop 2 to cols(x)
		y[:,#]=f(x[#],args());
	end
	return y;
endfunction
	
function map %mapexpression2 (x:scalar, y:scalar; expr:string)
	return expr(x,y;args());
endfunction
	
function %ploteval2(f:string, x:numerical, y:numerical, maps=0)
	if isfunction(f) then 
		return map(f,x,y;args());
	else 
		if maps then return %mapexpression2(x,y,f;args());
		else return f(x,y;args());
		endif;
	endif;
endfunction

function map %mapexpression3 (x:scalar, y:scalar, z:scalar; expr:string)
	return expr(x,y,z;args());
endfunction

function %ploteval3 (f:string, ..
	x:numerical, y:numerical, z:numerical, maps=0)
	if isfunction(f) then 
		return map(f,x,y,z;args());
	else 
		if maps then return %mapexpression3(x,y,z,f;args());
		else return f(x,y,z;args());
		endif;
	endif;
endfunction
	
function args adaptiveevalone (g:string, a:number, b:number, ..
	eps=0.01, amin=0.00001, amax=0.01, square=1)
## Compute g(x) for t in [a,b] with adaptive step size.
## eps is the target accuracy.
## amin and amax are the minimal and maximal step size (rel to b-a).
## amax is also the initial step size.
## f and g can be expressions in x or user defined functions.
## Returns {x,y}
##
## Used by plot2d with the adaptive=1 parameter.
## See: plot2d
	// other direction
	if b<a then return
		adaptiveevalone(g,b,a,eps,amin,amax,square;args()); 
	endif;
	// determine saling of g-values (assume a nearly square plot)
	scale=1;
	if square and b<>a then
		t=linspace(a,b,20);
		s=%ploteval(g,t;args());
		scale=(totalmax(s)-totalmin(s))/(b-a);
		if scale==0 then scale=1; endif;
	else scale=1;
	endif;
	// walk through [a,b]
	t=a;
	sx=t;
	sy=g(t,args());
	if cols(sy)>1 then 
		error(g|" does not produce a real or column vector"); 
	endif
	x=zeros(rows(sx),1000); y=zeros(rows(sy),1000); 
	n=1; h=amax;
	x[:,1]=sx; y[:,1]=sy;
	if b<>a then amin=amin*abs(b-a); amax=amax*abs(b-a); endif;
	repeat
		repeat
			tn=t+h;
			if tn>b then tn=b; endif;
			sxn=tn;
			syn=g(tn,args());
			d=sqrt((sxn-sx)^2+((syn-sy)/scale)^2)[1];
			if isNAN(d) then break; endif;
			if d>eps and h>amin; then h=max(h/2,amin);
			elseif d<eps/4;	then h=min(h*2,amax); break;
			else break;
			endif;
		end;
		n=n+1;
		if n>cols(x) then
			x=x|x;
			y=y|y;
		endif;
		x[:,n]=sxn; y[:,n]=syn;
		sx=sxn; sy=syn;
		t=tn;
		if t>=b; then break; endif;
	end;
	return {x[:,1:n],y[:,1:n]};
endfunction

// * Color

function rgb (r,g,b)
## Computes a RGB color from [0,1]-values
##
## Returns an integer encryption of the color.
## Use to set rgb colors instead of a color index,
## and to define an image to be saved on disk.
##
## See: plot2d, putred, putgreen, putblue, savergb
	return putred(r)+putgreen(g)+putblue(b);
endfunction

function args getrgb (x)
## Computes the red, green and blue parts from an rgb color.
##
## Used to decompose image pixels into colors.
## Returns {r,g,b}
##
## See: rgb, getred, getblue, getgreen, loadrgb
	return {getred(x),getgreen(x),getblue(x)};
endfunction

function computehue (x,y,z,L=[0,0,1],amb=0.1,max=0.9,hue=1)
## Computes the hue value for the vector.
##
## This is used in various plot functions
	if hue==-1; return 0; endif;
	if hue==-2; return (x-totalmin(x))/(totalmax(x)-totalmin(x))*0.999; endif;
	if hue==-3; return (y-totalmin(y))/(totalmax(y)-totalmin(y))*0.999; endif;
	if hue==-4; return (z-totalmin(z))/(totalmax(z)-totalmin(z))*0.999; endif;
	O=ones(size(x,y,z));
	X=x*O; Y=y*O; Z=z*O;
	m=rows(O); n=cols(O);
	X1=X[1:m-1,1:n-1]; Y1=Y[1:m-1,1:n-1]; Z1=Z[1:m-1,1:n-1];
	DX1=(X[2:m,1:n-1])-X1;
	DY1=(Y[2:m,1:n-1])-Y1;
	DZ1=(Z[2:m,1:n-1])-Z1;
	DX2=(X[1:m-1,2:n])-X1;
	DY2=(Y[1:m-1,2:n])-Y1;
	DZ2=(Z[1:m-1,2:n])-Z1;
	HX=DY1*DZ2-DY2*DZ1;
	HY=-DX1*DZ2+DX2*DZ1;
	HZ=DX1*DY2-DY1*DX2;
	R=sqrt(HX*HX+HY*HY+HZ*HZ)+epsilon;
	H=-(HX*L[1]+HY*L[2]+HZ*L[3])/R;
	if hue>1; H=abs(H); endif;
	H=(H-totalmin(H))/(totalmax(H)-totalmin(H))*(max-amb)+amb;
	return H;
endfunction

// Image Files

function overwrite savepng (filename:string, w:integer=0, h:integer=0, ..
	antialias:integer=1)
## Save the current graphics as PNG file in current directory.
##
## Saves the current graphics in PNG format. If antialias=1, then
## the function takes more time, and needs more memory, but the output
## looks smoother. If w=0, "savepng" uses the screen width. If h=0,
## the current aspect ratio with the current width is used.
##
## The current directory is set by the "cd" command, or, if a
## notebook is saved or opened.
##
## See: savesvg, saveps
	_savepng(filename,w,h,antialias);
endfunction

function overwrite savesvg (filename:string, w:integer=0, h:integer=0)
## Save the current graphics as SVG file in current directory.
##
## Saves the current graphics in the format SVG (scalable vector
## format). If h=0, the current aspect ratio with the current width is
## used.
##
## The current directory is set by the "cd" command, or, if a
## notebook is saved or opened.
##
## See: savepng, saveps
	_savesvg(filename,w,h);
endfunction

function overwrite saveps (filename:string)
## Save the current graphics as EPS file in current directory.
##
## Saves the current graphics in EPS (encapsulated postscript) format.
##
## The current directory is set by the "cd" command, or, if a
## notebook is saved or opened.
##
## See: savepng, savesvg
	_saveps(filename);
endfunction

// * Obsolete

function fcd (f,xmin=-1,xmax=1,ymin=-1,ymax=1,n=50,nc=50,maps=0)
## Plots a function of two variables or expression of x and y
##
## This function uses density and contour.
## fcd uses map for the evaluation of the function "f".
##
## Obsolete, since plot2d should be used now.
##
## See: plot2d
	x=linspace(xmin,xmax,n); y=linspace(ymin,ymax,n);
	{X,Y}=field(x,y);
	Z=%ploteval2(f,X,Y,maps;args());
	z1=totalmin(Z); z2=totalmax(Z);
	density(Z,1);
	h=holding(1); contour(Z,linspace(z1,z2,nc)); holding(h);
	setplot(xmin,xmax,ymin,ymax);
endfunction

function fcontour (f,xmin=-1,xmax=1,ymin=-1,ymax=1, ..
	n=50,nc=50,hue=0,niveau="auto", ..
	grid=2,title="",add=0,color=1,hcolor=0,maps=0, ..
	contourcolor=none, contourwidth=1)
## Draw contour lines of a function or expression in x and y
##
## 2n is the spacing.
##
## Obsolete. Use plot3d.
##
## See: plot3d
	if add; p=plot(); xmin=p[1]; xmax=p[2]; ymin=p[3]; ymax=p[4]; endif;
	x=linspace(xmin,xmax,n); y=linspace(ymin,ymax,n);
	{X,Y}=field(x,y);
	Z=%ploteval2(f,X,Y,maps;args());
	z1=totalmin(Z); z2=totalmax(Z);
	if add then setplot(_plot()); hold on; endif;
	if hue then
		col=huecolor(color);
		density(Z,0.99); 
		hold on; 
		if contourcolor<>none then oldcol=contourcolor(contourcolor); 
		else oldcol=contourcolor(0);
		endif;
		oldcw=linewidth(contourwidth);
		if typeof(niveau)==8
			contour(Z,linspace(z1,z2,nc)); 
		else
			contour(Z,niveau); 
		endif;
		hold off;
		huecolor(col);
	else
		if contourcolor!=none then oldcol=contourcolor(contourcolor);
		else oldcol=contourcolor(1);
		endif;
		oldcw=linewidth(contourwidth);
		if typeof(niveau)==8
			contour(Z,linspace(z1,z2,nc)); 
		else
			contour(Z,niveau); 
		endif;
	endif;
	contourcolor(oldcol);
	linewidth(oldcw);
	setplot(xmin,xmax,ymin,ymax);
	if !add && grid then xplot(grid=grid); endif;
	if add then hold off; endif;
	setplot();
	settitle(title);
endfunction

function datacontour (Z,a=0,b=1,c=0,d=1, ..
	nc=50,hue=0,niveau="auto",grid=2,title="",add=0,color=1,hcolor=0, ..
	contourcolor=none, contourwidth=1)
## Draw contour lines of a data matrix.
##
## fcontour uses map for the evaluation of the function "f".
##
## Obsolete. Use plot3d.
##
## See: plot3d
	if add; p=plot(); a=p[1]; b=p[2]; c=p[3]; d=p[4]; endif;
	z1=totalmin(Z); z2=totalmax(Z);
	if add; setplot(_plot()); hold on; endif;
	if hue then
		col=huecolor(color);
		density(Z,0.9); 
		hold on; 
		if contourcolor!=none then oldcol=contourcolor(contourcolor);
		else oldcol=contourcolor(0);
		endif;
		oldcw=linewidth(contourwidth);
		if typeof(niveau)==8
			contour(Z,linspace(z1,z2,nc)); 
		else
			contour(Z,niveau); 
		endif;
		huecolor(col);
		hold off;
	else
		if contourcolor!=none then oldcol=contourcolor(contourcolor);
		else oldcol=contourcolor(1);
		endif;
		oldcw=linewidth(contourwidth);
		if typeof(niveau)==8
			contour(Z,linspace(z1,z2,nc)); 
		else
			contour(Z,niveau); 
		endif;
	endif;
	contourcolor(oldcol);
	linewidth(oldcw);
	setplot(a,b,c,d);
	if !add && grid then xplot(grid=grid); endif;
	if add; hold off; endif;
	setplot();
	settitle(title);
endfunction

function fniveau (f,niveau=0,xmin=-1,xmax=1,ymin=-1,ymax=1,n=50,maps=0)
## Draw contour lines of a function or expression in x and y
##
## Obsolete, since plot3d should be used now.
##
## See: plot3d
	x=linspace(xmin,xmax,n); y=linspace(ymin,ymax,n);
	{X,Y}=field(x,y);
	Z=%ploteval2(f,X,Y,maps;args());
	z1=totalmin(Z); z2=totalmax(Z);
	contour(Z,niveau);
	setplot(xmin,xmax,ymin,ymax);
endfunction

function frotate (f,a,b,n=40,nt=40,scale=1,frame=1, ..
	xlabel="x",ylabel="y",zlabel="z")
## Show a rotated plot of a function
##
## Obsolete, since plot3d with user=1 should be used now.
##
## See: plot3d
	x=linspace(a,b,n);
	if isfunction(f);
		r=map(f,x;args);
	else
		r=f(x,args());
	endif;
	t=linspace(0,2*pi,nt)';
	framedsolid(x,cos(t)*r,sin(t)*r,scale,frame);
endfunction

function triangle
## Obsolete function
	return 	1
endfunction

function triangles
## Obsolete Function.
	return 1
endfunction

function args %makesteps (x,y,lower=0)
## Expand {x,y}, such that it plots in steps.
	m=cols(x);
	i=redim(dup(1:m,2)',[1,2*m]);
	if lower then return {x[:,i[2:2*m]],y[:,i[1:2*m-1]]};
	else return {x[:,i[1:2*m-1]],y[:,i[2:2*m]]};
	endif;
endfunction

function args %makesteps3d (x,y,z)
## Expand {x,y}, such that it plots in steps.
	m=max(cols(x),cols(y));
	i=redim(dup(1:m,2)',[1,2*m]);
	if cols(x)>1 then x=x[:,i[2:2*m]]; x=x|(2*x[:,-1]-x[:,-3]); endif;
	if cols(y)>1 then y=y[:,i[2:2*m]]; y=y|y[:,-1]; endif;
	if cols(z)>1 then z=z[:,i[1:2*m]]; endif;
	n=max(rows(x),rows(y));
	i=redim(dup(1:n,2)',[1,2*n]);
	if rows(x)>1 then x=x[i[2:2*m]]; x=x_x[-1]; endif;
	if rows(y)>1 then y=y[i[2:2*m]]; y=y_(2*y[-1]-y[-3]); endif;
	if cols(z)>1 then z=z[i[1:2*m]]; endif;
	return {x,y,z}
endfunction

function f3dimplicit (f:string, x:real vector, y:real vector, z:real vector, ..
	fr:integer=1, xlabel:string="x", ylabel:string="y", zlabel:string="z", ..
	scale:real=1, direction:integer=4, maps=0, ..
	contourcolor=8, contourwidth=1)
## Implicit 3D plot contours at z-values.
	frame=getframe(x,y,z);
	if !holding() then clg; endif;
	h=holding(1);
	oldframe=frame;
	if scale then
		s=max([frame[2]-frame[1],frame[4]-frame[3],frame[6]-frame[5]]);
		xm=(frame[2]+frame[1])/2;
		ym=(frame[4]+frame[3])/2;
		zm=(frame[6]+frame[5])/2;
		ff=scale/s*2;
		frame=[frame[1]-xm,frame[2]-xm,frame[3]-ym, ..
			frame[4]-ym,frame[5]-zm,frame[6]-zm]*ff;
	else
		xm=0; ym=0; zm=0; ff=1;
	endif;
	if fr then color(2); %frame1(frame); color(1); endif;
	if bitand(direction,1) then
		loop 1 to cols(z)
			W=%ploteval3(f,x[#],y,z',maps;args())+x[#];
			oldc=contourcolor(contourcolor);
			lw=linewidth(contourwidth);
			solidhuecontour((W-xm)*ff,(y-ym)*ff,(z'-zm)*ff,0,(x[#]-xm)*ff,(W-xm)*ff);
			linewidth(lw);
			contourcolor(oldc);
		end;
	endif;
	if bitand(direction,2) then
		loop 1 to cols(z)
			W=%ploteval3(f,x,y[#],z',maps;args())+y[#];
			oldc=contourcolor(contourcolor);
			lw=linewidth(contourwidth);
			solidhuecontour((x-xm)*ff,(W-ym)*ff,(z'-zm)*ff,0,(y[#]-xm)*ff,(W-ym)*ff);
			linewidth(lw);
			contourcolor(oldc);
		end;
	endif;
	if bitand(direction,4) then
		loop 1 to cols(z)
			W=%ploteval3(f,x,y',z[#],maps;args())+z[#];
			oldc=contourcolor(contourcolor);
			lw=linewidth(contourwidth);
			solidhuecontour((x-xm)*ff,(y'-ym)*ff,(W-zm)*ff,0,(z[#]-zm)*ff,(W-zm)*ff);
			linewidth(lw);
			contourcolor(oldc);
		end;
	endif;
	if fr then 
		color(2); %frame2(frame);
		if fr<2 then %frameticks(frame,oldframe,xlabel,ylabel,zlabel); endif;
		color(1); 
	endif;
	holding(h);
endfunction

function overwrite solidhue (x:real, y:real, z:real, h:real, ..
	f:number=1, niveau=none, nc:index=40, values=none, ..
	contourcolor=8, contourwidth=1)
## Makes a shaded solid 3D-plot of x,y,z
## h is the shading and should run between 0 and 1.
## f determines, if h is scaled to fit in between 0 and f.
##
## See: plot3d
	if argn()==4 then return _solidhue(x,y,z,h); endif;
	if niveau==none then
		return _solidhue(x,y,z,h);
	else 
		if niveau=="auto" then
			niveau=linspace(totalmin(z),totalmax(z),nc);
		endif;
		if values==none then values=z; endif;
		oldc=contourcolor(contourcolor);
		lw=linewidth(contourwidth);
		res=_solidhuecontour(x,y,z,h,niveau,values);
		linewidth(lw);
		contourcolor(oldc);
		return res;
	endif;
endfunction

function drawbutton (text:string, x:real scalar, y:real scalar, w:real scalar)
## Draw a button with the text at x,y with w character width. x
## centered and y top of the label. Return the button rectangle. This
## function can be used to program user interaction with clicks onto
## the screen.
## See: inbutton
	st=barstyle("#O");
	bc=barcolor(0);
	tw=textwidth(); th=textheight();
	b=[x-tw*w/2,y-th/3,tw*w,th*5/3];
	bar(b);
	ctext(text,[x,y]);
	barstyle(st);
	barcolor(bc);
	return b
endfunction

function inbutton (b:real vector, s:real vector)
## Test if the screen coordinates s are with in the button rectangle b. 
## Get s with toscreen(mouse()) to ask the user for a click.
## See: drawbutton, mouse
	return s[1]>b[1] && s[1]<b[1]+b[3] && s[2]>b[2] && s[2]<b[2]+b[4];
endfunction

%currentaspect=1;

function aspect (w:positive number=none, h:positive number=none)
## Set the window aspect to w times h.
##
## This function reduces the plot window to a new window with
## proportions w times h. The font size remains the same as in the
## full window. To export such a plot, use the crop feature.
##
## If an aspect with larger width is active, insimg will insert only
## the active portion of the window.
##
## Without parameters, the function resets to the default window.
##
## Returns the current window ratio.
##
## See: insimg
	global %currentaspect;
	setplot([0,1,0,1]);
	if w==none then w=1; endif;
	if h==none then h=1; endif;
	if w==1 and h==1 then 
		shrinkwindow();
	elseif w>0 and h>0 then
		shrinkwindow();
		wi=window();
		p=pixel();
		oldrat=(wi[3]-wi[1])/(wi[4]-wi[2])*p[2]/p[1];
		newrat=abs(w)/abs(h)*p[2]/p[1];
		if oldrat>newrat then
			wi[3]=wi[1]+newrat*(wi[4]-wi[2]);
		else
			wi[4]=wi[2]+(wi[3]-wi[1])/newrat;
		endif;
		window(wi);
	endif;
	wi=window();
	%currentaspect=w/h;
	return %currentaspect;
endfunction

function fplot (f,a=0,b=0,n=200,grid=2)
## Plots the function f in [a,b]
##
## fplot("f") or fplot("f",,,n,...) plots f in the old interval.
## fplot uses map for the evaluation of the function "f".
## ffunction may be an expression in x.
##
## Deprecated!
##
## See: plot2d
	if a~=b; s=plot(); a=s[1]; b=s[2]; endif;
	t=linspace(a,b,n);
	if isfunction(f) then 
		s=map(f,t;args());
	else 
		s=f(t);
		if all(size(s)==[1,1]) then s=s[1,1]*ones(size(t)); endif;
	endif;
	if holding || !grid; return plot(t,s);
	else return xplot(t,s);
	endif;
endfunction

function f3dplotlight (f,xmin=-1,xmax=1,ymin=-1,ymax=1,n=40, ..
	scale=1,fscale=1,light=[0,0,1],amb=0.1,max=0.9,fr=1, ..
	xlabel="x",ylabel="y",zlabel="z",niveau=none,hue=1, ..
	maps=0,contourcolor=8,contourwidth=1)
## Plots a function f(x,y,...) in a square.
##
## Also f3dplot("f",xmin,xmax,ymin,ymax,n;...).
## f3dplot uses map for the evaluation of the function "f".
## "f" may be an expression in x and y.
##
## Obsolete. Use plot3d.
## See: plot3d
	lx=linspace(xmin,xmax,n);
	ly=linspace(ymin,ymax,n);
	{x,y}=field(lx,ly);
	z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0 then {z,h,z0}=scalez0(z,fscale); endif;
	ret=framedsolidhue(x,y,z,computehue(x,y,z,light,amb,max,hue), ..
		scale,1,fr,h,xlabel,ylabel,zlabel,niveau, ..
		contourcolor=contourcolor,contourwidth=contourwidth,z0=z0);
	return ret;
endfunction

function f3dplotpolarh (f,rmax=1,n=20,scale=1,fscale=1,frame=1, ..
	xlabel="x",ylabel="y",zlabel="z",niveau=none,hue=1, ..
	maps=0)
## Draw a function defined on the circle with radius rmax.
##
## f3d uses map for the evaluation of the function "f".
## n is the spacing.
##
## Obsolete. Use plot3d.
## See: plot3d
	t=linspace(0,2*pi,3*n); r=linspace(rmax/100,rmax,n)'; x=cos(t)*r; y=-sin(t)*r;
	Z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0; {Z,h,z0}=scalez0(Z,fscale); endif;
	if hue==-1 then h=0; else h=Z; endif;
	ret=framedsolidhue(x,y,Z,h,scale,1,frame,h,xlabel,ylabel,zlabel,niveau,z0=z0);
	return ret;
endfunction

function f3dplotpolar (f,rmax=1,n=20,scale=1,fscale=1, ..
	frame=1,wire=0,xlabel="x",ylabel="y",zlabel="z",maps=0)
## Draw a function defined on the circle with radius rmax.
##
## f3d uses map for the evaluation of the function "f".
## n is the spacing.
##
## Obsolete. Use plot3d.
## See: plot3d
	t=linspace(0,2*pi,3*n); r=linspace(0,rmax,n)'; x=cos(t)*r; y=-sin(t)*r;
	Z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0 then {Z,h,z0}=scalez0(Z,fscale); endif;
	if wire then
		ret=framedwire(x,y,Z,scale,frame,h,xlabel,ylabel,zlabel,z0=z0);
	else
		ret=framedsolid(x,y,Z,scale,frame,h,,xlabel,ylabel,zlabel,z0=z0);
	endif;	
	return ret;
endfunction

function f3dplotpolarl (f,rmax=1,n=20,scale=1,fscale=1, ..
	light=[0,0,1],amb=0.2,max=0.8,frame=1,xlabel="x",ylabel="y",zlabel="z", ..
	niveau="auto",hue=1,maps=0)
## Draw a function defined on the circle with radius rmax.
##
## f3d uses map for the evaluation of the function "f".
## n is the spacing.
##
## Obsolete. Use plot3d.
## See: plot3d
	t=linspace(0,2*pi,3*n); r=linspace(rmax/100,rmax,n)'; x=cos(t)*r; y=-sin(t)*r;
	Z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0 then {Z,h,z0}=scalez0(Z,fscale); endif;
	ret=framedsolidhue(x,y,Z,computehue(x,y,Z,light,amb,max,hue), ..
		scale,1,frame,h,xlabel,ylabel,zlabel,niveau,z0=z0);
endfunction

function overwrite fillcolor (c1=11,c2=3)
## Set the fillcolor for the fore- and background of 3D plots.
##
## fillcolor(c1,c2) or fillcolor() returns the colors [c1,c2].
##
## See: rgb
	return _fillcolor([c1,c2]);
endfunction

function mark3 (x,y,z)
## Plots points in three dimensions. 
##
## x,y,z must be 1xn vectors.
##
## Obsolete. Use plot3d.
##
## See: plot3d
	{c0,r0}=project(x,y,z);
	xy=fromscreen(c0,r0);
	return mark(xy[1],xy[2]);
endfunction

defaultclipping=true;

function clipping (flag:integer=true)
## Sets clipping off or on globally for 2D plot functions.
##
## See: plot2d
	global defaultclipping;
	res=defaultclipping;
	defaultclipping=flag;
	return res;
endfunction

function overwrite plot (x=0, y=0)
## Plots the values (x(i),y(i)) with the current style.
## If x is a matrix, y must be a matrix of the same size.
## The plot is then drawn for all rows of x and y.
## The plot is scaled automatically, unless hold is on.
## plot(x,y) and plot() return [x1,x2,y1,y2], where [x1,x2] is the range
## of the x-values and [y1,y2] of the y-values.
## plot(x) is the same as plot(1:cols(x),x).
##
## To turn off the clipping set defaultclipping=false
##
## See: plot2d
	global defaultclipping;
	if !holding() && argn()>0; clg; endif;
	if argn()==1;
		if iscomplex(x);
			if defaultclipping then oldclip=clipmore(window()); endif;
			res = _plot(re(x),im(x));
			if defaultclipping then clip(oldclip); endif;
			return res;
		elseif isreal(x);
			if defaultclipping then oldclip=clipmore(window()); endif;
			res = _plot(1:cols(x),x);
			if defaultclipping then clip(oldclip); endif;
			return res;
		else
			error("Illegal x for plot");
		endif;
	else if argn()==2;
		if isreal(x) && isreal(y);
			if defaultclipping then oldclip=clipmore(window()); endif;
			res = _plot(x,y);
			if defaultclipping then clip(oldclip); endif;
			return res;
		else
			error("Illegal x or y in plot");
		endif;			
	else return _plot();
	endif;
endfunction

function overwrite mark (x=0,y=0,c=1)
## Plots markers at (x(i),y(i)) according the the actual marker style.
## Works like plot.
## Obsolete. Use plot2d instead.
##
## See: plot2d
	if !holding() && argn()>0; clg; endif;
	if argn()==1;
		if iscomplex(x);
			oldclip=clipmore(window(),15);
			res = _mark(re(x),im(x));
			clip(oldclip);
			return res;
		else
			oldclip=clipmore(window(),15);
			res = _mark(1:cols(x),x);
			clip(oldclip);
			return res;
		endif;
	else if argn()==2;
			oldclip=clipmore(window(),15);
			res = _mark(x,y);
			clip(oldclip);
			return res;
	else if argn()==3;
			oldclip=clipmore(window(),15);
			res = _markwithcolor(x,y,c);
			clip(oldclip);
			return res;
	else return _plot();
	endif;
endfunction

function cplot (z)
## Plots a grid of complex numbers.
## Obsolete. Use plot2d instead.
##
## See: plot2d
	plot(re(z),im(z)); s=scaling(0); h=holding(1);
	w=z'; plot(re(w),im(w)); holding(h); scaling(s);
	return plot();
endfunction

function scalez (z,scale)
## Scale the matrix z to [-scale,scale].
##
## This is a function used by plot3d with parameter scale=1.
##
## See: plot3d
	if scale==0; return {z,0}; endif;
	h=totalmax(abs(z))+epsilon();
	return {z/h*scale,scale/h};
endfunction

function scalez0 (z:real, scale:number, center:integer=1)
## Scale the matrix z to [-scale,scale].
##
## This is a function used by plot3d.
##
## See: plot3d
	if scale==0; return {z,0}; endif;
	z1=totalmin(z); z2=totalmax(z);
	if center then
		z0=(z1+z2)/2; dz=(z2-z1)/2+epsilon();
	else
		z0=0; dz=(z2-z1)+epsilon();
	endif;
	return {(z-z0)/dz*scale,scale/dz,z0}
endfunction

function f3d (f,m=40)
	return plot3d(f,n=m);
endfunction

function f3dplot (f,xmin=-1,xmax=1,ymin=-1,ymax=1,n=40, ..
	scale=1,fscale=1,frame=1,wire=0,xlabel="x",ylabel="y",zlabel="z", ..
	sliced=0,maps=0,wirecolor=1)
## f3dplot("f") plots a function f(x,y,...) in a square.
##
## Obsolete function. Use plot3d instead.
##
## f3dplot uses map for the evaluation of the function "f".
## "f" may be an expression in x and y.
##
## See: plot3d
	lx=linspace(xmin,xmax,n);
	ly=linspace(ymin,ymax,n);
	{x,y}=field(lx,ly);
	z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0 then {z,h,z0}=scalez0(z,fscale,!sliced); endif;
	if wire;
		ret=framedwire(x,y,z,scale,frame,h,xlabel,ylabel,zlabel,wirecolor,0,z0);
	else
		ret=framedsolid(x,y,z,scale,frame,h,,xlabel,ylabel,zlabel,sliced,0,z0);
	endif;	
	return ret;
endfunction

function f3dplothue (f,xmin=-1,xmax=1,ymin=-1,ymax=1,n=40, ..
	scale=1,fscale=1,frame=1,xlabel="x",ylabel="y",zlabel="z",niveau=none, ..
	maps=0)
## Plots a function f(x,y,...) in a square.
##
## Also f3dplot("f",xmin,xmax,ymin,ymax,n;...).
## f3dplot uses map for the evaluation of the function "f".
## "f" may be an expression in x and y.
##
## Obsolete. Use plot3d.
## See: plot3d
	lx=linspace(xmin,xmax,n);
	ly=linspace(ymin,ymax,n);
	{x,y}=field(lx,ly);
	z=%ploteval2(f,x,y,maps;args());
	h=1; z0=0;
	if fscale!=0; {z,h,z0}=scalez0(z,fscale); endif;
	ret=framedsolidhue(x,y,z,z,scale,1,frame,h,xlabel,ylabel,zlabel,niveau,z0=z0);
endfunction

function overwrite density (x,f=0.9,amb=0.1)
## Makes density plot of the values in the matrix x
## scaled to fit into [0,f].
## See: plot2d
	if f==0;
		_density(x);
		return x;
	else;
		ma=max(max(x)'); mi=min(min(x)'); h=ma-mi;
		if h~=0; h=1; endif;
		xx=amb+(x-mi)/h*(f-amb)*0.99;
		_density(xx);
	return xx;
	endif;
endfunction

function %plot2dcomplex (z)
	oldclip=clipmore(window());
	_plot(re(z),im(z));
	_plot(re(z'),im(z'));
	clip(oldclip);
	return xplot();
endfunction

function plot3dbars (x : real vector, y : real vector, z : real, ..
	z0 : real scalar)
## Plot bars at x[i],y[j] with height z[i,j].
##
## The bars have limits x[i],x[i+1] and y[j],y[j+1] and use
## the values z[i,j]. z must be at least (n-1)x(m-1), if
## x is 1xn and y is 1xm. The bars are sorted correctly before
## they are plotted by plotcubes.
##
## A comfortable way to use is is via plot3d and bar=true.
##
## See: plotcubes, plot3d
	n=cols(x); m=cols(y);
	M=zeros((n-1)*(m-1),6);
	k=1;
	for i=1 to n-1
		for j=1 to m-1
			if x[i+1]<x[i] or y[j+1]<y[j] then
				error("x and y must be sorted for plot3dbar.");
			endif;
			if z0<z[j,i] then
				M[k]=[x[i],x[i+1],y[j],y[j+1],z0,z[j,i]];
			else
				M[k]=[x[i],x[i+1],y[j],y[j+1],z[j,i],z0];
			endif;
			k=k+1;
		end;
	end;
	plotcubes(M);
endfunction

function overwrite plotcubes (M, color=none)
## Plot cubes (parallel to the axes)
##
## See: plotcubes
	v=view();
	x=sin(v[3])*cos(v[4])*v[1]; y=-cos(v[3])*cos(v[4])*v[1];
	d=zeros(1,rows(M));
	for k=1 to rows(M)
		if x>M[k,1] and x<M[k,2] and y>M[k,3] and y<M[k,4] then
			d[k]=0;
		else
			h=(x-M[k,1])^2+(y-M[k,3])^2;
			h=min(h,(x-M[k,1])^2+(y-M[k,4])^2);
			h=min(h,(x-M[k,2])^2+(y-M[k,3])^2);
			h=min(h,(x-M[k,2])^2+(y-M[k,4])^2);
			d[k]=h;
		endif;
	end;
	{ds,i}=sort(d);
	i=flipx(i);
	if color!=none then _plotcubes(M[i],color[i]);
	else _plotcubes(M[i]);
	endif;
endfunction

noantialias=0;

function overwrite insimg (lines:index=25, name:string="", ..
	antialias=1, crop=none)
## Insert a file image (PNG) with default size and name. The image
## appears in the notebook with the given height in lines. If the name
## is not empty, the image will be saved under this name when the
## notebook is saved.
##
## antialiased : looks good for most plots, but not for all.
##
## crop : inserts only a top portion of the graphics. The crop can be
## a vector [top,bottom,left,right] of numbers from 0 to 1, or it can
## be [top,bottom], or only bottom. The defaults are [0,1,0,1].
##
## The crop works well with the aspect function. In fact, if an aspect
## is active with larger width than height, only the active portion of
## the image is inserted automatically.
##
## See: insimg, aspect
	global noantialias,%currentaspect;
	if crop==none then crop=[0,1.1/%currentaspect,0,1];
	elseif typeof(crop)==0 then crop=[0,crop,0,1];
	elseif typeof(crop)==2 and cols(crop)==2 then crop=crop|[0,1];
	endif;
	_insimg(lines,name,antialias&&(!noantialias),crop);
endfunction

function overwrite insrgb (x:real, lines:integer=25, name:string="")
## Insert an rgb image with default size and name.
## The image appears in the notebook with the given height
## in lines. If the name is not empty, the image will
## be saved under this name when the notebook is saved.
## See: insimg, rgb
	return _insrgb(lines,name,x);
endfunction

function plotrgb (x:real, window:real vector=none)
## Plot an rgb image into the plot window.
## The window can be a vector with screen coordinates.
## See: rgb
	if window<>none then _window(window); endif;
	return _barRGB(getred(x),getgreen(x),getblue(x));
endfunction

function overwrite loadimg
## loadimg("filename"), loadimg(n,"filename") 
##
## Insert an image from file with default maximal size.
##
## The image appears in the notebook with the given height
## in lines.
##
## n : maximal number of lines to be used (default is 40)
##
## See: insimg, loadimg
	if argn==2 then return _loadimg(arg1,arg2);
	else if argn==1 then return _loadimg(40,arg1);
	else error("Need a file name for loadimg");
	endif;
endfunction

function overwrite loadanaglyph
## loadanaglyph("left","right") or loadanaglyph(n,"left","right")
##
## Loads two images, combines them to one anaglyph, and inserts the
## images into the notebook.
##
## n : maximal number of lines to be used (default is 40)
##
## See: loadanaglyph, loadimg
	if argn==3 then return _loadanaglyph(arg1,arg2,arg3);
	else if argn==2 then return _loadanaglyph(40,arg1,arg2);
	else error("Need to file names for loadanaglyph.");
	endif;
endfunction

function figure (n:natural, m:index=none)
## figure(nr,nc) divides the graph window into nr rows x nc columns of subplots
## figure(n) sets the current drawing window to the nth subplot counting
## by rows.
## figure(0) sets the plot back to the normal window
	global subwindows,logredraw,usesubwindows;
	if m<>none then
		if m>0 then nr=n; nc=m;
		else nr=n[1]; nc=n[2];
		endif;
		w=window();
		wd=textwidth();
		ht=textheight();
		hr=floor(1024/nr);
		wc=floor(1024/nc);
		for r=1 to nr
			for c=1 to nc
				w=[8*wd+(c-1)*wc,1.5*ht+(r-1)*hr,c*wc-2*wd,r*hr-3*ht];
				subwindows[(r-1)*nc+c]=w;
			end;
		end;
		hold off;
		logredraw=1;
		clg;
		return _window(subwindows[1]);
	else
		hold on;
		logredraw=1;
		if n==0 or n>rows(subwindows) then
			hold off;
			usesubwindows=0;
			return shrinkwindow();
		else 
			usesubwindows=1;
			return _window(subwindows[n]); 
		endif;
	endif
endfunction

function title (text:string)
## Plots a title to the grafik window.
## See: plot2d, plot3d
	ctext(text,[512 0]);
	return text;
endfunction

function fwebplot (fff:string, a:real scalar, b: real scalar, ..
	xstart: real scalar, n: index, color=9)
## Plots an iteration of fff on [a,b] starting from xstart, n steps
## This function can be used to study, demontrate or understand
## fixed point iteteration of the type x(n+1) = f(x(n)).
## See: iterate, sequence
	fplot("x",a,b);
	hold on; color(color); fplot(fff,a,b;args()); color(1); hold off;
	y=niterate(fff,xstart,n+1;args())';
	v=[xstart,0,xstart]|redim(dup(y[1:n],4),[1,4*n])|[y[n+1]];
	v=redim(v,[cols(v)/2,2])';
	hold on; color(10); plot(v[1],v[2]); color(1); hold off;
	return y;
endfunction

function vectorfield (expr:string, x1: real scalar, x2: real scalar, ..
	y1: real scalar, y2: real scalar, nx:index=20, ny:index=20)
## Draw the vector field of a differential equation in x and y.
## expr must be the expression "f(x,y)", which computes the
## derivative of y(x) at x.
	setplot(x1,x2,y1,y2);
	x=linspace(x1,x2,nx-1);
	y=linspace(y1,y2,ny-1);
	{X,Y}=field(x,y);
	V=expr(X,Y);
	n=prod(size(X));
	h=(x2-x1)/nx/4;
	v=redim(V,[n,1]);
	r=sqrt(v*v+1);
	x=redim(X,[n,1]); x=(x-h/r)|(x+h/r);
	y=redim(Y,[n,1]); y=(y-h*v/r)|(y+h*v/r);
	st=linestyle("->"); xplot(x,y); linestyle(st);
	return plot();
endfunction

function vectorfield2 (expr1:string, expr2:string, ..
	x1: real scalar,x2: real scalar,y1: real scalar,y2: real scalar, ..
	nx:index=20, ny:index=20)
## Draw the vector field of a differential equation in x and y.
## expr1 and expr2 must be expressions "f(x,y)", which compute the
## x- and y-value of the derivative of y(x) at x.
	setplot(x1,x2,y1,y2);
	x=linspace(x1,x2,nx-1);
	y=linspace(y1,y2,ny-1);
	{X,Y}=field(x,y); V1=zeros(size(X)); V2=zeros(size(X));
	loop 1 to prod(size(X));
		V1{#}=expr1(X{#},Y{#});
		V2{#}=expr2(X{#},Y{#});
	end;
	n=prod(size(X));
	h=(x2-x1)/nx/4;
	v1=redim(V1,[n,1]);
	v2=redim(V2,[n,1]);
	x=redim(X,[n,1]); x=(x-h*v1)|(x+h*v1);
	y=redim(Y,[n,1]); y=(y-h*v2)|(y+h*v2);
	st=linestyle("->"); xplot(x,y); linestyle(st);
	return plot();
endfunction

function overwrite view
## view(distance, tele, angle1, angle2) sets the perspective for solid and view. 
## distance is the eye distance, tele a zooming factor.
## angle1 is the angle from the negativ y-axis to the positive x-axis.
## angle2 is the angle to the positive z-axis (the height of the eye).
## view() returns the current value of view in a vector.
## view is used in plot3d as the default view.
## See: plot3d, zoom
	if argn()==4; return _view([arg1,arg2,arg3,arg4]);
	else if argn()==1; return _view(arg1);
	else return _view();
	endif;
endfunction

function zoom (f:positive=2.5)
## Set the zoom distance to d.
## See: view
   v=_view();
   _view([v[1],f,v[3],v[4]]);
   return v;
endfunction

function viewangle (a:real scalar=0.4)
## Set the view angle to a.
## See: view
   v=_view();
   _view([v[1],v[2],a,v[4]]);
   return v;
endfunction

function viewheight (a:real scalar=0.4)
## Set the view height angle to a.
## See: view
   v=_view();
   _view([v[1],v[2],v[3],a]);
   return v;
endfunction

function viewdistance (d:positive=5)
## Set the view distance to d.
## See: view
   v=_view();
   _view([d,v[2],v[3],v[4]]);
   return v;
endfunction

function f3daxis (f:string, n:index=25)
## Draw a function defined on [-1,1]^2 with x-, y- and z-axis.
## f3daxis uses map for the evaluation of the function "f".
## 2n is the spacing.
## Fixed view, cannot be rotated.
## See: plot3d
	x=-1:1/n:1; y=x;
	{X,Y}=field(x,y);
	Z=%ploteval2(f,X,Y;args());
	v=view(5,3,0.4,0.2);
	z1=totalmin(Z); z2=totalmax(Z); h=(z2+z1)/2; Z=Z-h;
	i=n+1:2*n+1; j=1:n+1;
	if !holding() then clg; endif;
	hl=holding(1);
	solid(X[i,j],Y[i,j],Z[i,j]);
	linewidth(3); 
	wire([0,0],[0,0],[-1.5,1.5]);
	wire([-1.5,0],[0,0],[-h,-h]);
	wire([0,0],[0,1.5],[-h,-h]);
	linewidth(1);
	solid(X[i,i],Y[i,i],Z[i,i]);
	solid(X[j,j],Y[j,j],Z[j,j]);
	linewidth(3);
	wire([0,1.5],[0,0],[-h,-h]);
	wire([0,0],[-1.5,0],[-h,-h]);
	linewidth(1);
	solid(X[j,i],Y[j,i],Z[j,i]);
	holding(hl);
	view(v);
endfunction

function f3daxispolar (f:string, n:index=20)
## Draw a function defined on [-1,1]^2 with x-, y- and z-axis.
## 2n is the spacing.
## f3daxispolar uses map for the evaluation of the function "f".
## Fixed view, cannot be rotated.
## See: plot3d
	r=flipx(0:1/n:1)'; p=linspace(0,2*pi,8*n);
	X=r*cos(p); Y=r*sin(p);
	Z=%ploteval2(f,X,Y;args());
	v=view(5,3,0.4,0.2);
	z1=totalmin(Z); z2=totalmax(Z); h=(z2+z1)/2; Z=Z-h;
	i=1:n+1;
	j=2*n:4*n+1;
	if !holding() then clg; endif;
	solid(X[i,j],Y[i,j],Z[i,j]); 
	hl=holding(1);
	linewidth(3);
	wire([0,0],[0,0],[-1.5,1.5]);
	wire([-1.5,0],[0,0],[-h,-h]);
	wire([0,0],[0,1.5],[-h,-h]);
	linewidth(1);
	j=1:2*n+1;
	solid(X[i,j],Y[i,j],Z[i,j]);
	j=4*n:6*n+1;
	solid(X[i,j],Y[i,j],Z[i,j]);
	linewidth(3);
	wire([0,1.5],[0,0],[-h,-h]);
	wire([0,0],[-1.5,0],[-h,-h]);
	linewidth(1);
	j=6*n:8*n+1;
	solid(X[i,j],Y[i,j],Z[i,j]);
	holding(hl);
	view(v);
endfunction

function stereo (f:string, a:number=-0.05, s:index=500, ..
	d:number=1, parallel:natural=0)
## Stereo plot of the function f from two viewpoints
## Calls the function f, which must be the name of
## a 3D plot command, and does it twice in two different
## windows. Many people are able to view a 3D picture.
##
## There are two ways: parallel view and squint view.
## Most people find the latter easier. It is the default
## for this function.
	if parallel then a=-a; endif;
	clg;
	m=512;
	win=window();
	window(m-s,m-s/2,m,m+s/2);
	setanaglyph(-d*a);
	f(args());
	window(m,m-s/2,m+s,m+s/2);
	setanaglyph(0);
	hold on;
	f(args());
	hold off;
	window(win);
	setanaglyph(0);
	hold off;
endfunction

function parameterplot (f:string, a:number, d:number, ..
	display=1, c=800, r=100, ..
	amax=1e30, amin=-1e30)
## Call the plot function f with parameter a adjustable by user.
##
## Animate a paramter with the up/down keys, and show a plot depending
## on that parameter. The space key resets the default parameter, and
## the return key ends the function.
##
## f : function of a (and additional semicolon parameters)
## a : starting value parameter a
## d : increment with one keystroke
## display : flag for the display of the parameter value
## c,r : coordinates for the display
## amax,amin : limits for the display
##
## See: twoparameterplot
	astart=a;
	repeat
		f(a,args());
		if display then
			text("a = "|a,c,r);
		endif;
		k=key("up/down, space or return");
		if k==1 then a=a+d;
		elseif k==2 then a=a-d;
		elseif k==32 then a=astart;
		elseif k==13 then return a;
		else "up/down, space or return",
		endif;
		if a<amin then a=amin; endif;
		if a>amax then a=amax; endif;
		a=round(a,12);
	end;
endfunction

function twoparameterplot (f:string, a:number, b:number, d:number, ..
	display=1, c=800, r=100, ..
	amax=1e30, amin=-1e30)
## Call the plot function f with parameter a adjustable by user.
##
## Animate a parameter a with the up/down keys, and a parameter b with
## left/right keys, and show a plot depending on that parameter. The
## space key resets the default parameter, and the return key ends the
## function.
##
## f : function of a,b (and additional semicolon parameters)
## a : starting value of parameter a
## b : second parameter b
## d : increment with one keystroke
## display : flag for the display of the parameter values
## c,r : coordinates for the display
## amax,amin : limits for the display
##
## See: twoparameterplot
	astart=a;
	bstart=b;
	repeat
		f(a,b,args());
		if display then
			text("a = "|a,c,r);
			text("b = "|b,c,r+40);
		endif;
		k=key("up/down, left/right, space or return");
		if k==1 then a=a+d;
		elseif k==2 then a=a-d;
		elseif k==4 then b=b+d;
		elseif k==3 then b=b-d;
		elseif k==32 then a=astart; b=bstart;
		elseif k==13 then return a;
		else "up/down, left/right, space or return",
		endif;
		a=round(a,12);
		if a<amin then a=amin; endif;
		if a>amax then a=amax; endif;
	end;
endfunction

function args makeimpulse (x:real vector,y:real vector,d=0.1)
## Compute {x1,y1} for impulse bar plots.
##
## Computes the correct input for an impulse bar plot
## at points x with values y.
## 
## x : sorted x parameters
## y : values
## d : radius of the plot intervals
	n=cols(x);
	x1=redim((x'-d)|(x'+d),[1,2*n]);
	y1=zeros(1,cols(x1)-1); y1[1:2:2*n-1]=y;
	return {x1,y1}
endfunction

// * Animation

function dragpoints (f : string, x : real, y : real, 
	status="Drag one of the points!")
## Lets the user drag points on a user defined plot.
##
## This function calls a function f(x,y), which draws some plot
## depending on the points at coordinates x[i] and y[i]. Then it lets
## the user drag one of the points with the mouse.
##
## f : f(x,y[,select]) a plot function. select is the index of the
## selected point, if not select==0.
##
## Additional arguments are passed to f.
##
## Returns {x,y}
##
## Example:
## >function f(x,y) := plot2d(x,y,>points,a=0,b=1,c=0,d=1);
## >{x,y}=dragpoints("f",random(1,10),random(1,10));
	xreset=x; yreset=y;
	select=0;
	repeat
		f(x,y,select,args());
		{flag,m,t}=mousedrag(status);
		if flag==0;
			if m==32 then x=xreset; y=yreset; flag=3;
			else break;
			endif;
		endif
		if flag==1 then // mouse down
			{c,r}=toscreen(x,y);
			{cm,rm}=toscreen(m[1],m[2]);
			e=extrema(max(abs(c-cm),abs(r-rm)));
			if e[1]<markersize() then 
				select=e[2];
				x[select]=m[1]; y[select]=m[2];
			endif;
		elseif flag==2 then // mouse moved
			if select>0 then 
				x[select]=m[1]; y[select]=m[2];
			endif;
		elseif flag==3 then // mouse up
			select=0;
		endif;
	end
	return {x,y};
endfunction

function dragvalues (f, names, values, ranges, stops=100,
	x=0.98, y=0.02, w=0.4, tcolor=red, digits=none,
	heading=none, hcolor=red, status="")
## Let the user drag parameters for a plot.	
##
## This function lets the user drag values for plots. A box with
## values appears, by default in the upper right, with an optional
## heading. The values can be dragged to the right or left, and change
## within given ranges and steps.
##
## f : A plot function depending on a vector of parameters.
## names : The names of the parameters for the display.
## values : Initial values of the parameters (string vector of length
##   n or string) 
## ranges : A nx2 vector of ranges for the values
## stops : A vector with the number of subintervals for each range
## x,y,w : Position anchored in the top right (fraction of plot window)
## tcolor : A vector with colors for the values or one scalar color
## digits : A vector of digits for rounding the values or a scalar
## heading : An optional heading string
## hcolor : The color of the heading
## status : An optional string for the status line
##
## Example:
## >function f([a,b]) := plot2d("a*x^2+b*x";a,b,r=1);
## >dragvalues("f",["a","b"],[0,0],[-1,1;-1,1],digits=2, ...
## >  heading="a*x^2+b*x");
	vreset=values;
	
	win=window(); ww=win[3]-win[1]; wh=win[4]-win[2];
	
	ts=textsize();
	w=w*ww;
	x=win[1]+ww*x-w; y=win[2]+ww*y;
	h=(length(names)*1.2+1)*ts[2];
	if heading!=none then h=h+ts[2]*1.2; endif;
	
	select=0; outside=0;
	repeat
		f(values;args());
		
		bst=barstyle("O#"); bcol=barcolor(0);
		bar([x,y,w,h]);
		barstyle(bst); barcolor(bcol);
		
		xt=x+w/2; yt=y+ts[2]/2;
		if heading!=none then
			tc=textcolor(hcolor);
			ctext(heading,[xt,yt]);
			textcolor(tc);
			yt=yt+ts[2]*1.2;
		endif;
		loop 1 to length(names);
			tc=textcolor(tcolor{#});
			if digits==none then text=names{#}+" = "+values{#};
			else text=names{#}+" = "+print(values{#},digits{#},0);
			endif;
			ctext("< "+text+" >",[xt,yt]);
			textcolor(tc);
			yt=yt+ts[2]*1.2;
		end;
		if select and outside then
			wait(0); m=mousepos(); flag=2;
			if mousestate()==3 then flag=3; endif;
		else
			{flag,m,t}=mousedrag(status);
		endif;
		if flag==0 then
			if m==32 then values=vreset; flag=3;
			else break;
			endif;
		endif;
		if flag==1 then // mouse down
			{cm,rm}=toscreen(m[1],m[2]);
			if cm>x and cm<x+w then
				select=floor((rm-(y+ts[2]/2))/(ts[2]*1.2))+1;
				if heading!=none then select=select-1; endif;
				if select>length(names) then select=0; endif;
				if select<0 then select=0; endif;
			endif;
		elseif select and flag==2 then // mouse moved
			{cm1,rm1}=toscreen(m[1],m[2]);
			if outside then sg=sign(cm1-(x+w/2));
			else sg=sign(cm1-cm);
			endif
			vnew=values[select]+ ..
				sg*(ranges[select,2]-ranges[select,1])/stops;
			if digits!=none then 
				round(vnew,digits{select});
			endif,
			values[select]=min(max(vnew,ranges[select,1]),ranges[select,2]);
			cm=cm1; rm=rm1; if cm>=x+w or cm<=x then outside=1; else
			outside=0; endif;
		elseif flag==3 then // mouse up
			select=0;
		endif;	
	end;
	return values;	
endfunction

// The following functions in this section are obsolete, since plot3d
// can now animate 3D graphics under user control. E.g. plot2d and
// plot3d have the parameter >user.

function animate (d:real scalar=0.1)
## Animate pre-defined pages with delay d.
##
## The pages should be generated with calls to addpage, as in the
## following example
##
## deletepages(); 
## title("Creating Animation"); 
## addpage(); 
## showpage(1);
## loop ...
##   ... // graph something
##   addpage();
## end;
##
## See: rotate
	testkey();
	repeat
		loop 2 to pages();
			showpage(#); 
			k=testkey();
			if k==32; key();
			elseif k==13; clg; showpage(0); return pages(); 
			elseif k;
				"Press space to stop, return to end",
				if key()==13; break; endif;
			endif;
			dd=wait(d);
			if (dd<d); showpage(0); return pages(); endif;
		end;
	end;
	showpage(0);
	return pages();
endfunction;	

function rotate (fff:string, d:real scalar=0.01, n:integer=120)
## Shows a rotating animation of the plot fff, which must be a 3D plot
## command.
##
## Obsolete, since the plot can now be rotated with plot3d under user
## control. 
##
## See: plot3d
	deletepages();
	clg;
	title("Creating Animation");
	addpage();
	showpage(1);
	t=linspace(0,2*pi,n);
	v=view();
	loop 1 to cols(t);
		v[3]=t[#]; view(v);
		fff(args());
		addpage();
	end;
	animate(d);
	return pages();
endfunction

function view3d (fff:string, d:real scalar=0.1)
## Calls the plot fff, which must be a 3D plot command, waits for key
## strokes to rotate it.
##
## Use the cursor keys for rotation, del and backspace, or space, to
## reset, return to abort. Returns the current view.
##
## Obsolete, use plot3d.
##
## See: plot3d
	vold=view();
	repeat
		fff(args());
		k=key();
		if k==13; break; endif;
		v=view();
		if (k==1); v[4]=v[4]-d;
		elseif (k==2); v[4]=v[4]+d;
		elseif (k==3); v[3]=v[3]-d;
		elseif (k==4); v[3]=v[3]+d;
		elseif (k==32); v=vold;
		elseif (k==7); v[3]=0;
		elseif (k==6); v[4]=0;
		elseif (k==43); v[1]=max(v[1]-d,1);
		elseif (k==45); v[1]=v[1]+d;
		else
			"Press return to end, cursorkeys to turn,",
			"+, - to zoom, or space to reset.",
			if key()==13; break; endif;
		endif;
		view(v);
	end;
	v=view();
	view(vold);
	return v;
endfunction

function fanimate (fff:string, t:vector, d:real scalar=0.01)
## Animate the function fff
##
## fff should display some graphics depending on t. Additional
## parameters are passed to f.
##
## See: animate
	deletepages();
	clg;
	title("Creating Animation");
	addpage();
	showpage(1);
	loop 1 to cols(t);
		fff(t[#],args());
		addpage();
	end;
	animate(d);
	return pages();
endfunction

// * Framed Plots

// Most of these functions need never be used, since 3D graphics
// should be done with plot3d.

function getframe (x,y,z)
## Gets a box around all points in (x,y,z).
##
## Returns [xmin,xmax,ymin,ymax,zmin,zmax]
	return [totalmin(x),totalmax(x), ..
		totalmin(y),totalmax(y), ..
		totalmin(z),totalmax(z)]
endfunction

function %framez0 (f)
	wire([f[1],f[2],f[2],f[1],f[1]], ..
		[f[3],f[3],f[4],f[4],f[3]],dup(f[5],5)');
	return 0	
endfunction

function %framez1 (f)
	wire([f[1],f[2],f[2],f[1],f[1]], ..
		[f[3],f[3],f[4],f[4],f[3]],dup(f[6],5)');
	return 0
endfunction

function %framexpyp (f)
	wire([f[2],f[2]],[f[4],f[4]],[f[5],f[6]]);
	return 0
endfunction

function %framexpym (f)
	wire([f[2],f[2]],[f[3],f[3]],[f[5],f[6]]);
	return 0
endfunction

function %framexmyp (f)
	wire([f[1],f[1]],[f[4],f[4]],[f[5],f[6]]);
	return 0
endfunction

function %framexmym (f)
	wire([f[1],f[1]],[f[3],f[3]],[f[5],f[6]]);
	return 0
endfunction

function %frame1 (f)
## Draws the back part of the frame (considering view)
	v=view();
	wc=wirecolor(frame3dcolor()); c=color(frame3dcolor());
	x=sin(v[3])*cos(v[4])*v[1];
	y=-cos(v[3])*cos(v[4])*v[1]; 
	z=sin(v[4])*v[1];
	if z>f[5]; %framez0(f); endif;
	if z<f[6]; %framez1(f); endif;
	if x<=f[2]; %framexpyp(f); %framexpym(f); endif;
	if x>=f[1]; %framexmyp(f); %framexmym(f); endif;
	if y<=f[4]; %framexmyp(f); %framexpyp(f); endif;
	if y>=f[3]; %framexmym(f); %framexpym(f); endif;
	wirecolor(wc); color(c);
	return 0
endfunction

function %frame2 (f)
## Draws the front part of the frame (considering view).
	v=view();
	wc=wirecolor(frame3dcolor()); c=color(frame3dcolor());
	x=sin(v[3])*cos(v[4])*v[1]; y=-cos(v[3])*cos(v[4])*v[1]; z=sin(v[4])*v[1];
	if z<=f[5]; %framez0(f); endif;
	if z>=f[6]; %framez1(f); endif;
	if x>=f[2]; %framexpyp(f); %framexpym(f); endif;
	if x<=f[1]; %framexmyp(f); %framexmym(f); endif;
	if y>=f[4]; %framexmyp(f); %framexpyp(f); endif;
	if y<=f[3]; %framexmym(f); %framexpym(f); endif;
	wirecolor(wc); color(c);
	return 0
endfunction

function %scaleframe (x,y,z,%f,m,niv=0)
## Scales the frame and the x,y,z matrix
	s=max([%f[2]-%f[1],%f[4]-%f[3],%f[6]-%f[5]]);
	xm=(%f[2]+%f[1])/2;
	ym=(%f[4]+%f[3])/2;
	zm=(%f[6]+%f[5])/2;
	ff=m/s*2;
	%f=[(%f[1]-xm)*ff{1},(%f[2]-xm)*ff{1}, ..
		(%f[3]-ym)*ff{2},(%f[4]-ym)*ff{2}, ..
		(%f[5]-zm)*ff{3},(%f[6]-zm)*ff{3}];
	return {(x-xm)*ff{1},(y-ym)*ff{2},(z-zm)*ff{3},(niv-zm)*ff{3}}
endfunction

function %scaleframeniveau (x,y,z,values,niveau,%f,m,hf)
## Scales the frame, the x,y,z and the niveau values
	s=max([%f[2]-%f[1],%f[4]-%f[3],%f[6]-%f[5]]);
	xm=(%f[2]+%f[1])/2;
	ym=(%f[4]+%f[3])/2;
	zm=(%f[6]+%f[5])/2;
	ff=m/s*2;
	%f=[(%f[1]-xm)*ff{1},(%f[2]-xm)*ff{1}, ..
		(%f[3]-ym)*ff{2},(%f[4]-ym)*ff{2}, ..
		(%f[5]-zm)*ff{3},(%f[6]-zm)*ff{3}];
	return {(x-xm)*ff{1},(y-ym)*ff{2},(z-zm)*ff{3}, ..
		(values-zm)*ff{3},(niveau*hf-zm)*ff{3}}
endfunction

function %frametick (h1,h2,x1,y1,z1,x2,y2,z2,xz,yz,zz,s)
## Draws a border line of the frame with frame ticks.
	a=0.125; b=1;
	repeat;
	s1=niceform(h1,2); s2=niceform(h2,2);
	h=max(max(strlen(s1),strlen(s2)),strlen(s));
	if h>4 then a=(h+1)*0.025; endif;
	th=textheight();
	r=max(abs([x2-xz,y2-yz,z2-zz]));
	xa=b*x1+a*sign(x2-xz)*r; ya=b*y1+a*sign(y2-yz)*r; za=b*z1+a*sign(z2-zz)*r;
	{cc1,r1}=project(xa,ya,za);
	r=max(abs([x1-xz,y1-yz,z1-zz]));
	xb=a*sign(x1-xz)*r+b*x2; yb=a*sign(y1-yz)*r+b*y2; zb=a*sign(z1-zz)*r+b*z2;
	{cc2,r2}=project(xb,yb,zb);
	if max(abs(cc1-cc2),abs(r1-r2))<th then 
		a=1.5*a; continue; 
	endif;
	_ctext(s1,[cc1,r1-th/2]);
	_ctext(s2,[cc2,r2-th/2]);
	{cc,r}=project((xa+xb)/2,(ya+yb)/2,(za+zb)/2);
	return _ctext(s,[cc,r-th/2]);
	end;
endfunction

function %frameticks (f,of,xlabel="x",ylabel="y",zlabel="z")
## Draws all border lines of the frame with frame ticks.
	v=view();
	x=sin(v[3])*cos(v[4])*v[1]; 
	y=-cos(v[3])*cos(v[4])*v[1]; 
	z=sin(v[4])*v[1];
	xz=(f[1]+f[2])/2;
	yz=(f[3]+f[4])/2;
	zz=(f[5]+f[6])/2;
	if x>=f[2] and y<=f[4] then
		%frametick(of[5],of[6],f[2],f[4],f[5],f[2],f[4],f[6],xz,yz,zz,zlabel);
	elseif y>=f[4] and x>=f[1] then
		%frametick(of[5],of[6],f[1],f[4],f[5],f[1],f[4],f[6],xz,yz,zz,zlabel);
	elseif x<=f[1] and y>=f[3] then
		%frametick(of[5],of[6],f[1],f[3],f[5],f[1],f[3],f[6],xz,yz,zz,zlabel);
	elseif y<=f[3] and x<=f[2] then
		%frametick(of[5],of[6],f[2],f[3],f[5],f[2],f[3],f[6],xz,yz,zz,zlabel);
	endif;
	if x>=f[2] and z<=f[6] then
		%frametick(of[3],of[4],f[2],f[3],f[6],f[2],f[4],f[6],xz,yz,zz,ylabel);
	elseif x>=f[1] and z>=f[6] then
		%frametick(of[3],of[4],f[1],f[3],f[6],f[1],f[4],f[6],xz,yz,zz,ylabel);
	elseif x<=f[1] and z>=f[5] then
		%frametick(of[3],of[4],f[1],f[3],f[5],f[1],f[4],f[5],xz,yz,zz,ylabel);
	elseif x<=f[2] and z<=f[5] then
		%frametick(of[3],of[4],f[2],f[3],f[5],f[2],f[4],f[5],xz,yz,zz,ylabel);
	endif;
	if y>=f[4] and z<=f[6] then
		%frametick(of[1],of[2],f[1],f[4],f[6],f[2],f[4],f[6],xz,yz,zz,xlabel);
	elseif y>=f[3] and z>=f[6] then
		%frametick(of[1],of[2],f[1],f[3],f[6],f[2],f[3],f[6],xz,yz,zz,xlabel);
	elseif y<=f[3] and z>=f[5] then
		%frametick(of[1],of[2],f[1],f[3],f[5],f[2],f[3],f[5],xz,yz,zz,xlabel);
	elseif y>=f[4] and z<=f[5] then
		%frametick(of[1],of[2],f[1],f[4],f[5],f[2],f[4],f[5],xz,yz,zz,xlabel);
	endif;
endfunction

function %slice (x,y,z,s)
	n=max(rows(x),rows(y));
	i=redim(dup(1:n,2)',[1,2*n]);
	if rows(x)>1 then x1=x[i,:]; else x1=x; endif;
	if rows(y)>1 then y1=y[i,:]; else y1=y; endif;
	if rows(z)>1 then 
		z1=z[i,:]; z1[1:2:2*n-1,:]=0;
	else 
		z1=z|0; 
	endif;
	return {x1,y1,z1}
endfunction

function framedplot (f:string, frame:real vector, 
	sframe:real vector=none, 
	xl:string="x", yl:string="y", zl:string="z",
	ticks:integer=1,
	view=none, zoom=none, distance=none, angle=none, height=none,
	center=none)
## General function for framed plots.
##
## This function does the same as the functions called by plot3d. It
## draws a frame around a plot. The frame is drawn in two parts, one
## in the back and one in the front. It can display ticks, which may
## have nothing to do with the frame itself (sframe parameter).
##
## A function using this function can be used in plot3d with parameter
## >own to allow anaglyphs or turning.
##
## f : A 3D plot function. This can also be plot3d with parameter
## <frame. Semicolon parameters are passed to f.
##
## frame : The frame. take care that the plot fits into the frame. The
## frame is a vector of the form [xmin,xmax,ymin,ymax,zmin,zmax]
##
## sframe : The tick values for the frame. If sframe=none then frame
## is used.
##
## See: plot3d
	if !holding() then clg; endif;
	h=holding(1);
	vold=view();
	if view<>none then view(view); endif;
	if zoom<>none then zoom(zoom); endif;
	if distance<>none then viewdistance(distance); endif;
	if angle<>none then viewangle(angle); endif;
	if height<>none then viewheight(height); endif;	
	oldcenter=center();
	if center!=none then center(center); endif;
	%frame1(frame);
	f(args());
	%frame2(frame);
	if sframe==none then sframe=frame; endif;
	if ticks then %frameticks(frame,sframe,xl,yl,zl); endif;
	view(vold);
	center(oldcenter);
	holding(h);	
endfunction

function framedsolid (x:real, y:real, z:real,
	scale:real vector=1, fr:integer=1, hf:number=1,
	disconnect=none,
	xlabel:string="x", ylabel:string="y", zlabel:string="z",
	sliced:integer=0, bar:integer=0, z0:real=0,
	frame:real vector=none, sframe:real vector=none)
## 3D plot with frame.
##
## x, y, and z must be compatible matrices. The determine the
## coordinates of the 3D images of a parameter matrix.
##
## If scale is specified, then the plot is scaled to fit into a cube
## of side length 2*scale centered at 0.
##
## If fr=0 then no frame will be drawn around the plot.
##
## disconnect is a vector of indices containing the rows of the
## matrix, which should not be connected.
##
## sliced is a parameter, which produces sliced plots into the x, or
## the y direction. Should only be used for regular x,y grids.
##
## bar is a parameter, which makes the plot into steps. Should only be
## used for regular x,y grids.
##
## This function should only be used via plot3d.
##
## See: plot3d
	if sliced==1 then 
		{x,y,z}=%slice(x,y,z,sliced);
		disconnect=2:2:max(rows(x),rows(y))-2;
	elseif sliced==2 then
		{x,y,z}=%slice(x',y',z',sliced);
		disconnect=2:2:max(rows(x),rows(y))-2;
	endif;
	ownframe=0;
	if frame==none then 
		frame=getframe(x,y,z); 
	else 
		ownframe=1;
		if sframe==none then sframe=frame; endif;
		scale=0; 
	endif;
	if !holding() then clg; endif;
	h=holding(1);
	if sframe==none then
		sframe=frame; 
		sframe[5]=sframe[5]/hf+z0; sframe[6]=sframe[6]/hf+z0;
	endif;
	niv=0;
	if all(scale>0) then 
		{x,y,z,niv}=%scaleframe(x,y,z,frame,scale); 
	endif;
	if fr then %frame1(frame); endif;
	if bar then
		plot3dbars(x,y',z,min(max(niv,totalmin(z)),totalmax(z)));
	elseif typeof(disconnect)==8 then 
		solid(x,y,z); 
	else 
		solid(x,y,z,disconnect);
	endif;
	if fr then 
		color(2); %frame2(frame);
		if fr<2 then %frameticks(frame,sframe,xlabel,ylabel,zlabel); endif;
		color(1); 
	endif;
	holding(h);
	return frame
endfunction

function framedsolidhue (x:real, y:real, z:real, hue, ..
	scale:real vector=1, f:number=1, fr:integer=1, hf:number=1, ..
	xlabel:string="x", ylabel:string="y", zlabel:string="z", ..
	niveau=none, values=none, ..
	light=[0.5,0.5,1], amb=0.1, max=0.9, huecolor=0, ..
	contourcolor=8, contourwidth=1, z0=0,
	frame=none, sframe=none)
## 3D plot with hue, and optional niveau lines.
##
## x, y, and z must be compatible matrices. They determine the
## coordinates of the 3D images of a parameter matrix. hue is an
## additional compatible matrix, which determines the color shade of
## the matrix at each point. E.g., plot3d computes the hue using a
## light source.
##
## niveau is a row vector of niveaus levels, which are to be drawn.
## values is a matrix of the same size as x, y, and z which contains
## the values, which determine the niveau. By default, the value will
## be z.
##
## If scale is specified, then the plot is scaled to fit into a cube
## of side length 2*scale centered at 0.
##
## If fr=0 then no frame will be drawn around the plot.
##
## disconnect is a vector of indices containing the rows of the
## matrix, which should not be connected.
##
## This function should only be used via plot3d.
##
## See: plot3d
	if frame==none then 
		frame=getframe(x,y,z); 
	else 
		if sframe==none then sframe=frame; endif;
		scale=0; 
	endif;	
	if values==none then values=z; endif;
	if !holding(); clg; endif;
	h=holding(1);
	if sframe==none then
		sframe=frame; 
		sframe[5]=sframe[5]/hf+z0; sframe[6]=sframe[6]/hf+z0;
	endif;
	if all(scale>0) then 
		if typeof(niveau)==8 then {x1,y1,z1}=%scaleframe(x,y,z,frame,scale);
		else {x1,y1,z1,values,niveau}= ..
			%scaleframeniveau(x,y,z,values,niveau,frame,scale,hf);
		endif;
	else {x1,y1,z1}={x,y,z}; 
	endif;
	if hue==none then 
		hue=computehue(x1,y1,z1,light,amb,max,huecolor); 
	endif;
	if fr then %frame1(frame); endif;
	if f then
		if niveau=="auto" then solidhue(x1,y1,z1,hue,f,"auto", ..
			contourcolor=contourcolor,contourwidth=contourwidth); 
		else solidhue(x1,y1,z1,hue,f,niveau=niveau,values=values, ..
			contourcolor=contourcolor,contourwidth=contourwidth); 
		endif;
	else
		if niveau==none then _solidhue(x1,y1,z1,hue);
		elseif niveau=="auto" then solidhue(x1,y1,z1,hue,"auto", ..
			contourcolor=contourcolor,contourwidth=contourwidth); 
		else solidhue(x1,y1,z1,hue,niveau=niveau,values=values, ..
			contourcolor=contourcolor,contourwidth=contourwidth); 
		endif;
	endif;
	if fr; 
		color(2); %frame2(frame);
		if fr<2; %frameticks(frame,sframe,xlabel,ylabel,zlabel); endif;
		color(1);	
	endif;
	holding(h);
	return frame
endfunction

function framedwire (x:real, y:real, z:real,
	scale:real vector=1, fr:integer=1, hf:number=1,
	xlabel:string="x", ylabel:string="y", zlabel:string="z",
	wirecolor:number=1, rowsonly:integer=0, z0:real=0,
	frame:real vector=none, sframe:real vector=none)
## 3D wire plot with frame.
##
## x, y, and z must be compatible matrices. They determine the
## coordinates of the 3D images of a parameter matrix.
##
## If scale is specified, then the plot is scaled to fit into a cube
## of side length 2*scale centered at 0.
##
## If fr=0 then no frame will be drawn around the plot.
##
## With rowsonly=1 the wire will be drawn only in one direction. 
##
## The wirecolor can be specified with the variable wirecolor. Use any
## of the 16 predefined colors, or an rgb value.
##
## This function should only be used via plot3d.
##
## See: plot3d,rgb
	if frame==none then 
		frame=getframe(x,y,z); 
	else 
		if sframe==none then sframe=frame; endif;
		scale=0; 
	endif;	
	if !holding(); clg; endif;
	h=holding(1);
	if sframe==none then
		sframe=frame; 
		sframe[5]=sframe[5]/hf+z0; sframe[6]=sframe[6]/hf+z0;
	endif;
	if all(scale>0) then 
		{x1,y1,z1}=%scaleframe(x,y,z,frame,scale);
	else 
		{x1,y1,z1}={x,y,z}; 
	endif;
	if fr then %frame1(frame); endif;
	wc=wirecolor(wirecolor); 
	if rowsonly then wirerows(x1,y1,z1); else wire(x1,y1,z1); endif;
	wirecolor(wc);
	if fr then
		color(2); %frame2(frame);
		if fr<2 then %frameticks(frame,sframe,xlabel,ylabel,zlabel); endif;
		color(1);
	endif;
	holding(h);
	return frame
endfunction

function framedmark3 (x:real, y:real, z:real,
	scale:real vector=1, fr:integer=1, hf:number=1,
	xlabel:string="x", ylabel:string="y", zlabel:string="z",
	color=1, z0:number=0, 
	frame:real vector=none, sframe:real vector=none)
## 3D point plot with frame.
##
## x, y, and z must be row vectors. They determine the coordinates of
## the 3D images of the points.
##
## This function should only be used via plot3d.
##
## See: plot3d,rgb
	if frame==none then 
		frame=getframe(x,y,z); 
	else 
		if sframe==none then sframe=frame; endif;
		scale=0; 
	endif;	
	if !holding(); clg; endif;
	h=holding(1);
	if sframe==none then
		sframe=frame; 
		sframe[5]=sframe[5]/hf+z0; sframe[6]=sframe[6]/hf+z0;
	endif;
	if all(scale>0) then 
		{x1,y1,z1}=%scaleframe(x,y,z,frame,scale);
	else
		{x1,y1,z1}={x,y,z}; 
	endif
	if fr then %frame1(frame); endif;
	{c0,r0,dist}=project(x1,y1,z1);
	{x0,y0}=fromscreen(c0,r0);
	if length(color)>1 and rows(x)==1 and rows(y)==1 then
		{dist,id}=sort(-dist);
		markwithcolor(x0[id],y0[id],color[id]);
	else
		oc=_color(color);
		mark(x0,y0);
		color(oc);
	endif;
	if fr then
		color(2); %frame2(frame);
		if fr<2 then %frameticks(frame,sframe,xlabel,ylabel,zlabel); endif;
		color(1);
	endif;
	holding(h);
	return frame;
endfunction

