// Statistics with Euler

comment
Statistical functions.
endcomment

// Euler has a lot of routines to generate random numbers, like the
// builtin functions random and normal. Moreover, Euler has builtin
// routines for distributions.

// This file provides more distributions, random numbers, and tests.

// * Distributions

function erf (x:real)
## Gau? normal distribution with standard deviation 1/2
##
## This is the result of integrate(exp(-t^2),t,0,x)*2/sqrt(pi). The
## "erf" function is defined in the literature, and can easily be
## computed from the "normaldis" function of Euler.
##
## See: normaldis
	return 2*_normaldis(sqrt(2)*x)-1
endfunction

function map binsum (i:natural, n:natural, p:number)
## Probability of getting i or less hits in n runs.
##
## Returns the cumulative sum of the binomial distribution with
## probability p. I.e., if we have n independent random throws, with a
## probability of p for a hit, the functions computes the probability
## to get i or less hits.
##
## For large n and i, use "normalsum".
##
## See: normalsum, invbinsum
	if i>=n; return 1; endif;
	d=sqrt(n*p*(1-p));
	i1=n*p-10*d;
	if i1<0; i1=0; endif;
	if i<i1; return 0; endif;
	if i>n*p+10*d; return 1; endif;
	s=n*log(1-p);
	if i>=1;
		j=1:i+1;
		s=cumsum(s|(log((n-j+1)/j*p/(1-p))));
	endif;
	return sum(exp(s[max(1,floor(i1)):i+1]));
endfunction

function overwrite normaldis (x : real, mean : real = 0, dev : real = 1)
## Probability of being smaller than x with the normal distribution.
	return _normaldis((x-mean)/dev);
endfunction

function overwrite invnormaldis (p : real, mean : real = 0, dev : real = 1)
## Probability of being smaller than x with the normal distribution.
	return _invnormaldis(p)*dev+mean;
endfunction

function normalsum (i:natural, n:natural, p:number)
## Probability of getting i or less hits in n runs.
##
## Works like binsum, but is much faster for large n and medium p.
##
## See: binsum
	return normaldis((i+0.5-n*p)/sqrt(n*p*(1-p)));
endfunction

function map invbinsum (x:number, n:natural, p:number)
## Computes i, such that binsum(i,n,p) is just larger than x.
##
## This is the inverse function to binsum. We now the probability for
## i hits in n random events, and want to know i.
##
## See: binsum
	if x<=0; return 0; endif;
	if x>=1; return n; endif;
	s=n*log(1-p);
	d=sqrt(n*p*(1-p));
	i1=floor(n*p-10*d); i2=floor(n*p+10*d);
	if i1<1; i1=1; endif;
	if i2>n; i2=n; endif;
	j=1:i2;
	s=cumsum(s|(log((n-j+1)/j*p/(1-p))));
	return find(cumsum(exp(s[i1:length(s)])),x)+i1-1;
endfunction

function map hypergeomsum (i:natural, n:natural, itot:natural, ntot:natural)
## Hypergemotric sum.
##
## Return the probability of hitting i or less black balls if n are
## chosen out of ntot, and there are a total of itot black balls (and
## ntot-itot white balls, of course).
	if i<itot-(ntot-n); return 0; endif;
	if i>=itot; return 1; endif;
	if itot<=ntot-n;
		s=logbin(ntot-n,itot)-logbin(ntot,itot);
		if i>=1;
			j=1:i;
			s=cumsum(s|(log(n-j+1)-log(j)+log((itot-j+1))-log((ntot-n-itot+j))));
		endif;
	else
		s=log(bin(n,itot-(ntot-n)))-log(bin(ntot,itot));
		if i>itot-(ntot-n);
			j=(itot-(ntot-n)+1):i;
			s=cumsum(s|(log(n-j+1)-log(j)+log(itot-j+1)-log(ntot-n-itot+j)));
		endif;
	endif
	return sum(exp(s));
endfunction

function map gammarestr (x)
## Special Gamma function, works only for 2x natural
## look into gamma.e for the general gamma function
	if floor(x)~=x; return fac(x-1); endif;
	return prod(0.5:1:x-1)*sqrt(pi());
endfunction

function qnormal (x, m=0, d=1)
## Returns the density m-d-normal distribution
##
## This is the density function the Gauss normal distribution with
## mean m and standard deviation 1.
##
## See: normaldis, erf
	return 1/(d*sqrt(2*pi))*exp(-(x-m)^2/2/d^2);
endfunction

function qtdis (t:real, n:nonnegative integer)
## Density of the student t distribution
	return gammarestr((n+1)/2)/(sqrt(pi()*n)* ..
			gammarestr(n/2))*(1+t*t/n)^(-(n+1)/2);
endfunction

function qchidis (x, n)
## density of the chi distribution
	return 1/(2^(n/2)*gammarestr(n/2))*x^(n/2-1)*exp(-x/2);
endfunction

function qfdis (x,n,m)
	return n^(n/2)*m^(m/2)*gammarestr((n+m)/2)/ ..
			(gammarestr(n/2)*gammarestr(m/2))* ..
			x^(n/2-1)*(m+n*x)^(-(n+m)/2);
endfunction

// The following distributions are based on Julia code by John D. Cook.

function randmatrix (n:index, m:index, f:string)
## Apply the random generator to generate a matrix.
	M=zeros(n,m);
	loop 1 to prod(size(M)); 
		M{#} = f(args());
	end;
	return M
endfunction

function randuniform (n : index, m : index, a : number, b : number)
## return a uniform random sample from the interval (a, b)
    return a + random(n,m)*(b - a)
endfunction

function randnormal (n : index, m : index, 
	mean : number = 0, stdev : nonnegative number = 1)
## return a random sample from a normal (Gaussian) distribution
	return normal(n,m)*stdev+mean;
endfunction

function %randexponential (mean : positive number)
## return a random sample from an exponential distribution
    return -mean*log(random())
endfunction

function randexponential (n : index, m : index, 
	mean : positive number = 1)
## return a random matrix from an exponential distribution
    return randmatrix(n,m,"%randexponential";mean);
endfunction

function %randgamma (shape : nonnegative number, scale : nonnegative number)
## return a random sample from a gamma distribution
##
## Implementation based on "A Simple Method for Generating Gamma Variables"
## by George Marsaglia and Wai Wan Tsang.
## ACM Transactions on Mathematical Software
## Vol 26, No 3, September 2000, pages 363-372.
    if shape >= 1.0 then
        d = shape - 1.0/3.0;
        c = 1.0/sqrt(9.0*d);
        repeat
            x = normal();
            v = 1.0 + c*x;
            repeat
	            while v<=0
                x = normal();
                v = 1+c*x;
            end
            v = v*v*v;
            u = random();
            xsq = x*x;
            if u<1-0.0331*xsq*xsq or log(u)<0.5*xsq+d*(1-v+log(v)) then
                return scale*d*v
            endif
        end
    else
        g = %randgamma(shape+1.0, 1.0);
        w = random();
        return scale*g*w^(1.0/shape)
    endif
endfunction

function randgamma (n : index, m : index,
	shape : nonnegative number, scale : nonnegative number)
## return a random matrix from a gamma distribution
##
## Implementation based on "A Simple Method for Generating Gamma Variables"
## by George Marsaglia and Wai Wan Tsang.
## ACM Transactions on Mathematical Software
## Vol 26, No 3, September 2000, pages 363-372.
    return randmatrix(n,m,"%randgamma";shape,scale);
endfunction

function randchi (n : index, m : index, dof : index)
## return a random matrix from a chi square distribution
## with the specified degrees of freedom
    return randgamma(n,m,0.5*dof,2)
endfunction

function randinversegamma (n : index, m : index, 
	shape : positive number, scale : positive number)
## return a random matrix from an inverse gamma random variable
    return 1/randgamma(n,m,shape,1/scale)
endfunction

function randweibull (n : index, m : index,
	shape : positive number, scale : positive number)
## return a random matrix from a Weibull distribution
    return scale * (-log(random(n,m)))^(1.0/shape)
endfunction

function randcauchy (n : index, m : index,
	median : number, scale : positive number)
## return a random matrix from a Cauchy distribution
	p=random(n,m);
    return median + scale*tan(pi*(p - 0.5));
endfunction

function randt (n : index, m : index,
	dof : positive integer)
## return a random sample from a Student t distribution
##
## See Seminumerical Algorithms by Knuth
	y1=normal(n,m);
	y2=randchi(n,m,dof);
	return y1 / sqrt(y2/dof);
endfunction

function %randlaplace (mean : number, scale : positive number)
## return a random sample from a Laplace distribution
## The Laplace distribution is also known as the double exponential distribution.
    u = random();
    if u < 0.5 then
        return mean + scale*log(2.0*u);
    else
        return mean - scale*log(2*(1-u));
    endif
endfunction

function randlaplace (n : index, m : index,
	mean : number, scale : positive number)
## return a random sample from a Laplace distribution
## The Laplace distribution is also known as the double exponential distribution.
    return randmatrix(n,m,"%randlaplace";mean,scale);
endfunction

function randlognormal (n : index, m : index,
	mu : number, sigma : positive number)
## return a random sample from a log-normal distribution
    return exp(randnormal(n,m,mu,sigma));
endfunction

function randbeta (n : index, m : index,
	a: positive number, b : positive number)
## return a random sample from a beta distribution
##
## There are more efficient methods for generating beta samples.
## However such methods are little more efficient and much more complicated.
    u = randgamma(n, m, a, 1.0);
    v = randgamma(n, m, b, 1.0);
    return u / (u + v);
endfunction

// * Descriptive Functions

function meandev (x:numerical, v=none)
## Return the mean value and the standard deviation of [x1,...,xn]
##
## An optional additional parameter v contains the multiplicities of
## x. m=mean(x) will assign the mean value only! If x is a matrix the
## function works on each row.
##
## x : data (1xm or nxm)
## v : multiplicities (1xn or nxm)
##
## See: mean, dev
	if v<>none then
		n=sum(v);
		m=sum(x*v)/n;
		if n>1 then d=sqrt(sum(v*(x-m)^2)/(n-1)); else d=0; endif;
	else
		n=cols(x);
		m=sum(x)/n;
		if n>1 then d=sqrt(sum((x-m)^2)/(n-1)); else d=0; endif;
	endif;
	return {m,d};
endfunction

function mean (x:numerical, v:real vector=none)
## Returns the mean value of x.
##
## An optional additional parameter contains multiplicities.
##
## See: meandev, median
	{m,d}=meandev(x,v); return m;
endfunction

function dev (x:numerical, v:real vector=none)
## Returns the standard deviation of x
##
## An additional parameter may contain multiplicities.
##
## See: meandev
	{m,d}=meandev(x,v); return d;
endfunction

function median (x, v=none, p=0.5)
## Returns a value such that p of the x[i] are less equal.
##
## v are optional multiplicities for the values. If x is a matrix, the
## function works on all rows of x.
##
## x : data (1xm or nxm)
## v : multiplicites (1xm or nxm)
## p : desired percentage
##
## See: mean
	m=rows(x);
	if m>1 then
		y=zeros(m,1);
		loop 1 to m;
			y[#]=median(x[#],v,p);
		end;
		return y;
	else
		if v<>none then
			{xs,i}=sort(x); vsh=v[i];
			n=cols(xs);
			ns=sum(vsh);
			i=1+p*(ns-1); i0=floor(i);
			vs=cumsum(vsh);
			loop 1 to n
				if vs[#]>i0 then 
					return xs[#]; 
				elseif vs[#]+1>i0 then
					k=#+1; 
					repeat; 
						if vsh[k]>0 or k>n then break; endif; 
						k=k+1;
					end;
					return (1-(i-i0))*xs[#]+(i-i0)*xs[k]+0; 
				endif;
			end;
			return xs[n];
		else
			xs=sort(x); 
			n=cols(x);
			i=1+p*(n-1); i0=floor(i);
			if i0==n then return xs[n]; endif;
			return (i-i0)*xs[i+1]+(1-(i-i0))*xs[i];
		endif;
	endif;
endfunction

function pfold (v: real vector, w: real vector)
## Returns the distribution of the sum of two distributions
##
## v[i], w[i] contain the probabilities that each random variable is
## equal to i-1. result[i] contains the probability that the sum of
## the two random variables is i-1.
##
## See: fold, fftfold
	z=zeros(1,length(w)-1);
	return fold(z|flipx(v)|z,w);
endfunction

function quantiles (x,q:vector=[0,0.25,0.5,0.75,1])
## Return the given quantiles for each row of x.
##
## See: boxplot
	v=zeros(rows(x),length(q));
	for i=1 to rows(x);
		for j=1 to cols(q)
			w=x[i,:];
			v[i,j]=median(w,p=q[j]);
		end;
	end
	return v;
endfunction

function covarmatrix (x:real)
## Emprical covariance matrix of the rows of x
	x=x-mean(x);
	return (x.x')/(cols(x)-1);
endfunction

function covar (x:real vector, y:real vector)
## Empirical covariance of x and y
	return sum((x-mean(x))*(y-mean(y)))/(cols(x)-1);
endfunction

function correlmatrix (x:real)
## Correlation matrix of the rows of x
	x=x-mean(x);
	x=x/sqrt(sum(x^2));
	return x.x';
endfunction

function correl (x:real vector, y:real vector)
## Correlation of x and y
	x=x-mean(x); x=x/sqrt(sum(x^2));
	y=y-mean(y); y=y/sqrt(sum(y^2));
	return x.y';
endfunction

function ranks (x)
## Ranks of the elements of x in x.
##
## With multiplicities the rank is the mean rank of the equal
## elements.
##
## Works for reals, real vectors, or string vectors x.
	{x,i}=sort(x);
	r=indexofsorted(x,x)-(multofsorted(x,x)-1)/2;
	{i,i2}=sort(i);
	return r[i2];
endfunction

function rankcorrel (x:real vector, y:real vector)
## Correlation of x and y
	x=ranks(x); x=x-mean(x); x=x/sqrt(sum(x^2));
	y=ranks(y); y=y-mean(y); y=y/sqrt(sum(y^2));
	return x.y';
endfunction

function getfrequencies (x:real vector, r: real vector)
## Count the number of x in the intervals of the sorted vector r.
##
## The function returns the number of x[j] in the intervals r[i-1] to
## r[i].
##
## x : real row vector (1xn)
## r : real sorted row vector (1xm)
##
## Returns the frequencies f as a row vector (1x(m-1)) 
##
## See: count, histo
	return multofsorted((sort(find(r,x))),1:cols(r)-1);
endfunction

function getmultiplicities (x, y, sorted=0)
## Counts how often the elements of x appear in y.
##
## This works for string vectors and for real vectors.
##
## sorted : if true, then y is assumed to be sorted.
##
## See: count, getfrequencies
	if typeof(x)==8 then x=[x]; endif;
	if !sorted then y=sort(y); endif;
	return multofsorted(y,x);
endfunction

function getstatistics (x:real vector, y:real vector=none)
## Return a statics of the values in the vector x.
##
## If y is none, the function returns {xu,mu}, where xu are the
## unique elements of x, and mu are the multiplicities of these
## values.
##
## Else the function returns {xu,yu,m}, where xu are the unique
## elements of x, yu the unique elements of y, and M is a table of
## multiplicities of pairs (xu[i],yu[j]) in (x[k],y[k]), k=1...n.
	if y==none then
	    xu=unique(x);
		return {xu,getmultiplicities(xu,x)};
	else
		if cols(x)!=cols(y) then 
			error("x and y must have same length");
		endif;
		xu=unique(x);
		yu=unique(y);
		i1=indexofsorted(xu,x);
		i2=indexofsorted(yu,y);
		M=zeros(length(xu),length(yu));
		loop 1 to cols(x);
			M[i1[#],i2[#]]=M[i1[#],i2[#]]+1;
		end;
		return {xu,yu,M};
	endif;
endfunction

function empdist (x:real vector, vsorted:real vector)
## Compute the empirical distribution
##
## Needs a sorted vector of empirical values.
	return find(vsorted,x)/length(vsorted);
endfunction

function args histo (d:real vector, n:index=10, 
	integer:integer=0, even:integer=0, v:real vector=none,
	bar=1)
## Computes {x,y} for histogram plots.
##
## d is an 1xm vector.
##
## x - End points of the intervals (equispaced n+1 points)
## y - The number of data in the subintervals (frequencies)
##
## integer=1 - for distributions on integer
## even=1 - better for evenly spaced discrete distributions
## To be used for plot2d and the bar style.
##
## v : intervall boundaries (ordered).
##
## bar : It true the functin returns two vectors for >bar in plot2d.
## If false it returns a sawtooth function for plot2d.
##
## The plot functions plot2d has parameters distribution=1, histgram=1
## to achieve the same effect.
##
## See: plot2d
	if v!=none then
		xx=v;
		d=d[nonzeros(d>=min(v) && d<=max(v))];
		y=count(find(v,d),length(v));
		y=y[2:length(y)];
	else	
		mi=min(d); ma=max(d);
		if even then dc=(ma-mi)/(2*(n-1)); mi=mi-dc; ma=ma+dc; endif;
		if mi~=ma then ma=mi+1; endif;
		if integer then n=floor(ma-mi)+1; ma=ma+0.9999; endif;
		x=zeros(1,n); y=zeros(1,n); h=(d-mi)/(ma-mi)*n;
		c=count(h,n+1); c[n]=c[n]+c[n+1]; y=c[1:n];
		xx=linspace(mi,ma,n);
	endif;
	if !bar then
	    i=2*ones(size(xx));
		xx=multdup(xx,i);
	    i=2*ones(size(y));
		y=multdup(y,i); y=0|y|0;
	endif;
	return {xx,y};
endfunction;

// * Tests

function chitest (x:real vector, y:real vector)
## Perform a xhi^2 test, if x is the expected frequency y
##
## This functions test an observed frequency x against an expected
## frequency y. E.g., if 40 men are found sampling 100 persons, then
## [40,60] has to be tested against [50,50]. The result of the test is
## 0.049, which means that the sample does not obey the expected
## frequency with an error less than 5%.
##
## To get frequencies of data from the data, use "getfrequencies",
## "count", or "histo".
##
## x,y : two real row vectors (1xn)
##
## Returns the error level for the hypothesis that the observed
## frequency x comes from another distribution as the expected
## frequency y.
##
## See: getfrequencies, count, histo
	return 1-chidis(sum((x-y)^2/y),cols(x)-1);
endfunction

function testnormal (r:real vector, n:integer, v:real vector, ..
	m:number, d:number)
## Test an observed frequency for normal distribution.
##
## Test the number of data v[i] in the ranges r[i],r[i+1] against the
## normal distribution with mean m and deviation d, using the chi^2
## method.
##
## r : ranges (sorted 1xm vector)
## n : total number of data
## v : number of data in the ranges (1x(m-1) vector)
## m : expected mean value
## d : expected deviation
##
## Return the error we get, if we reject the normal distribution.
	t=normaldis((r-m)/d);
	k=cols(v); p=(t[2:k+1]-t[1:k])*n;
	return chitest(v,p);
endfunction

function ttest (m:number, d:real scalar, n:natural, mu:number)
## T student test
##
## Test, if the measured mean m with measured deviation d of n data
## comes from a distribution with mean value mu. 
##
## m : mean value of data
## d : standard deviation of data
## n : number of data
## mu : mean value to test for
##
## Returns the error alpha, if we reject that the data come from a
## distribution with mean mu.
	return 1-tdis(abs(m-mu)/d*sqrt(n),n-1);
endfunction

function tcompare (m1:number, d1:number, n1:natural, ..
	m2:number, d2:number, n2:natural)
## Test, if two measured data agree in mean.
##
## The data must be normally distributed. Returns the error you have,
## if you reject this.
##
## m1,m2 : means of the data
## d1,d2 : standard deviation of the data
## n1,n2 : number of data
##
## Returns the error alpha, if we reject that the data come from a
## distribution with the same expected mean.
	h1=d1^2/n1; h2=d2^2/n2;
	return 1-tdis(abs(m1-m2)/sqrt(h1+h2), ..
		(h1+h2)^2/(h1^2/(n1+1)+h2^2/(n2+1))-2);
endfunction

function tcomparedata (x:real vector, y:real vector)
## Compare x and y for same mean
##
## Calls "tcompare" to compare the two observations for the same mean.
##
## Returns the error alpha, if we reject that the data come from a
## distribution with the same expected mean.
##
## See: tcompare
	return tcompare(mean(x),dev(x),cols(x),mean(y),dev(y),cols(y));
endfunction

function tabletest (A:real)
## Chi^2-Test the results a[i,j] for independence of the rows from the columns.
##
## The table test test for indepence of the rows of the tables
## from the column. E.g., if some items are observed [40,50] times
## for men, and [50,30] times for woman, we can ask, if the
## observations depend on the gender. In this case we can reject
## independece with 1.8% error level.
##
## This test should only be used for large table entries.
##
## Return the error you have, if you reject independence.
	c=sum(A); r=sum(A')';
	E=c*r/sum(r);
	return 1-chidis(totalsum((A-E)^2/E),(cols(A)-1)*(rows(A)-1));
endfunction

function expectedtable (A:real)
	c=sum(A); r=sum(A')'; t=totalsum(A);
	return (c*r)/t;
endfunction

function contingency (A:real, correct=1)
## Contigency Coefficent of a matrix A.
##
## If the coefficient is close to 0, we tend to say that the rows and
## the colums are independent.
##
## correct : Correct the coefficient, so that it is between 0 and 1
	n=cols(A); m=rows(A);
	B=expectedtable(A);
	chi=totalsum((A-B)^2/B);
	k=sqrt(chi/(chi+totalsum(A)));
	if !correct return k; endif;
	return k/sqrt(min(n-1,m-1)/min(n,m));
endfunction

function varanalysis
## varanalysis(x1,x2,x3,...) test for same mean.
##
## Test the data sets for the same mean, assuming normal distributed
## data sets.
##
## Returns the error we have, if we reject same mean.
	md=[mean(arg1),cols(arg1)];
	s=sum((arg1-md[1])^2);
	loop 2 to argn();
		x=args(#); mx=mean(x);
		md=md_[mx,cols(x)];
		s=s+sum((x-mx)^2);
	end;
	md=md'; n=sum(md[2]); mt=sum(md[1]*md[2])/n;
	s1=sum(md[2]*(md[1]-mt)^2);
	return 1-fdis((s1/(argn()-1))/(s/(n-argn())),argn()-1,n-argn());
endfunction

function mediantest (a:real vector, b:real vector)
## Median test for equal mean.
##
## Test the two distributions a and b on equal mean value. For this,
## both distributions are checked on exceeding the median of the
## cumulative distribution.
##
## Returns the error we have, if we reject that a and b can have the
## same mean.
	c=sort(a|b); n=cols(c);
	if mod(n,2)==0; m=(c[n/2]+c[n/2+1])/2;
	else m=c[n/2+1];
	endif;
	return 1-hypergeomsum(sum(a<m)-1,cols(a),floor(n/2),n);
endfunction

function ranktest (a:real vector, b:real vector, eps=epsilon())
## The Mann-Whitney test tests a and b on same distribution
##
## Return the error we have, if we reject the same distribution.
	n1=cols(a); n2=cols(b); n=n1+n2;
	{c,i}=sort(a|b); 
	R=ranks(c);
	R1=sum(R*(i<=n1)); R2=sum(R*(i>n1));
	return 1-normaldis(abs((min(R1-n1*(n1+1)/2,R2-n2*(n2+1)/2)-n1*n2/2)/ ..
		sqrt(n1*n2*(n1+n2+1)/12)));
endfunction

function signtest (a:real vector, b:real vector)
## Test, if the expected mean of a is not better than b
##
## Assume a(i) and b(i) are results of a treatment. Then we can ask,
## if a is better than b.
##
## a,b : row vectors of same size
##
## Return the error we have, if we decide that a is better
## than b.
	n=cols(a); i=sum(a>b);
	return 1-binsum(i-1,n,0.5);
endfunction

function wilcoxon (a:real vector, b:real vector, eps=sqrt(epsilon()))
## Test, if the expected mean of a is not better than b
##
## This is a sharper test for the same problem as in "signtest".
##
## Returns the error you have, if you decide that a is better
## than b.
##
## See: signtest
	d=a-b; n=cols(d);
	{c,i}=sort(abs(d)); R=1:n;
	{c1,i1}=sort(c+eps*(n:-1:1));
	R=(R+R[i1])/2;
	R1=sum(R*(d[i]<0)); R2=sum(R*(d[i]>=0));
	W=R2;
	return 1-normaldis((W-n*(n+1)/4)/ ..
		sqrt(n*(n+1)*(2*n+1)/24)));
endfunction

// * Statistical Plots

function boxplot (x:real, lab=none, style="0#",
	textcolor=red)
## Summary of the quantiles in x in graphical form.
##
## Use quantiles to compute the quantiles with default q equal to
## [0,0.25,0.5,0.75,1].
##
## style : If present, it is used as fill style, the default is "O"
##
## See: quantiles, barstyle
	m1=min(x[:,1]'); m2=max(x[:,5]');
	m=(m1+m2)/2; d=(m2-m1)/2*1.05;
	if !holding() then clg; endif;
	h=holding(1);
	setplot(0.5,rows(x)+0.5,m-d,m+d);
	frame();
	if lab==none then
		xgrid(1:rows(x),grid=4);
	else
		w=window();
		ht=textheight();
		loop 1 to length(lab);
			ct=textcolor(textcolor{#});
			{col,row}=toscreen(#,0);
			_ctext(""+lab[#],[col,w[4]+0.4*ht]);
			textcolor(ct);
		end;	
	endif;
	ygrid(ticks(m1,m2));
	loop 1 to rows(x);
		m1=x[#,1]; m2=x[#,5];
		st=style("--"); plot([#,#],[m1,m2]); style(st);
		st=style("-"); 
		plot([#-0.2,#+0.2],[m1,m1]); 
		plot([#-0.2,#+0.2],[m2,m2]); 
		style(st);
		st=barstyle(style);
		plotbar(#-0.4,x[#,2],0.8,x[#,4]-x[#,2]);
		barstyle(st);
		m=x[#,3];
		st=style("-");
		plot([#-0.4,#+0.4],[m,m]);
		style(st);
	end
	holding(h);
endfunction

function columnsplot (x:vector, lab=none,
	style="O#", color=green, textcolor=red)
## Plot the elements of x as columns.
##
## s : a string vector with one label for each element of x.
	m1=min(min(x),0); m2=max(max(x),0);
	m=(m1+m2)/2; d=(m2-m1)/2*1.05;
	if !holding() then clg; endif;
	h=holding(1);
	setplot(0.5,length(x)+0.5,m-d,m+d);
	frame();
	if lab!=none then
		w=window();
		ht=textheight();
		loop 1 to length(lab);
			ct=textcolor(textcolor{#});
			{col,row}=toscreen(#,0);
			_ctext(""+lab[#],[col,w[4]+0.4*ht]);
			textcolor(ct);
		end;
	else
		xgrid(1:length(x),grid=4);
	endif;
	ygrid(ticks(m1,m2));
	loop 1 to cols(x);
		cl=barcolor(color{#});
		st=barstyle(style);
		if x[#]>=0 then
			plotbar(#-0.4,0,0.8,x[#]);
		else
			plotbar(#-0.4,x[#],0.8,-x[#]);
		endif;
		barstyle(st);
		barcolor(cl);
	end;
	holding(h);
endfunction

function dataplot (x:real, y:real, style="[]w", color=1)
## Plot the data (x,y) with point and line plots.
##
## x : real row vector
## y : real row vector or matrix (one row for each data).
## style : a style or a vector of styles
## color : a color or a vector of colors
##
## You can use a vector of styles and a vector of colors. These
## vectors must contain as many elements as there are rows of y.
##
## See: statplot
	if !holding() then clg; endif;
	h=holding(1);
	setplot(totalmin(x),totalmax(x),totalmin(y),totalmax(y));
	xplot();
	loop 1 to rows(y);
		cl=_color(color{#});
		plot(x,y[#,:]);
		_color(cl);
	end;
	loop 1 to rows(y);
		cl=_color(color{#});
		ms=markerstyle(style{#});
		mark(x,y[#,:]);
		markerstyle(ms);
		_color(cl);
	end;
	holding(h);
endfunction

function piechart (x:real vector, style="0#", 
	color=green, lab:string vector=none, r=1.5, textcolor=red)
## plot the data x in a pie chart.
##
## x : the vector of data
## color : a color or a vector of colors (same length as x)
## style : a style or a vector of styles
## lab : a vector of labels (same length as x)
## r : The piechart has radius 1. To leave space use r=1.5.
	if !holding() then clg; endif;
	w=window(); window(0,0,1024,1024);
	h=holding(1);
	setplot(-r,r,-r,r);
	tot=sum(x);
	phi=0|cumsum(x/sum(x))*2*pi;
	loop 1 to cols(x);
		t=linspace(phi[#],phi[#+1],100);
		xs=0|sin(t); ys=0|cos(t);
		cl=barcolor(color{#}); st=barstyle(style{#});
		polygon(xs,ys,1);
		barcolor(cl); barstyle(st);
		if lab!=none then
			tc=textcolor(textcolor{#});
			ts=(phi[#]+phi[#+1])/2;
			xts=sin(ts)*(1+r)/2; yts=cos(ts)*(1+r)/2;
			{col,row}=toscreen(xts,yts);
			_ctext(""+lab[#],[col,row-textheight()/2]);
			textcolor(tc);
		endif;
	end;
	holding(h);	
	window(w);
endfunction

function starplot (v, style="/", color=green, lab:string=none, 
	rays:integer=0, pstyle="[]w", textcolor=red, r=1.5)
## A star like plot with a filled star or with rays and dots only
	if !holding() then clg; endif;
	w=window(); window(0,0,1024,1024);
	h=holding(1);
	vr=max(abs(v))*r;
	setplot(-vr,vr,-vr,vr);
	n=cols(v); t=linspace(0,2pi,n);
	v=v|v[1]; c=v*cos(t); s=v*sin(t);
	loop 1 to n
		if rays then
			cp=color(color{#});
			plot([0,c[#]],[0,s[#]]);
			ms=markerstyle(pstyle);
			mark(c[#],s[#]);
			markerstyle(ms);
			color(cp);
		else
			cl=barcolor(color{#}); st=barstyle(style{#});
			polygon([0,c[#],c[#+1]],[0,s[#],s[#+1]],1);
			barcolor(cl); barstyle(st);
		endif;
		if lab!=none then
			rlab=v[#]+vr*(r-1)/2;
			{col,row}=toscreen(cos(t[#])*rlab,sin(t[#])*rlab);
			tc=textcolor(textcolor{#});
			ctext(""+lab[#],col,row-textheight()/2);
			textcolor(tc);
		endif;
	end;
	holding(h);
	window(w);
endfunction

function logimpulseplot (x, y=none, style="O#", color=green, d=0.1)
## Logarithmic impulse plot of y.
	if y==none then y=x; x=1:cols(y); endif;
	if !holding() then clg; endif;
	h=holding(1);
	frame();
	yl=log(y)/log(10);
	setplot(min(x)-d,max(x)+d,min(min(yl),0),max(max(yl),0));
	bs=barstyle(style);
	loop 1 to cols(x);
		bc=barcolor(color{#});
		plotbar(x[#]-d,0,2d,yl[#]);
		barcolor(bc);
	end;
	barstyle(bs); 
	xgrid(ticks(x));
	p=plot();
	for i=-50 to 50;
		if i<=p[4] and i>=p[3] then
			 ygrid(i,yt="10^"+i);
		endif;
	end;
	holding(h);
endfunction

function columnsplot3d (z:real, srows=none, scols=none,
	angle=30, height=40, zoom=2.5, distance=5,
	crows:vector=none, ccols:vector=none)
## Plot 3D columns from the matrix z.
##
## This function shows a 3D plot of columns with heights z[i,j] in
## a rectangular array. z can be any real nxm matrix.
##
## z : the values to be displayed
## srows : labels for the rows
## scols : labels for the columns
## crows : colors of the rows
## ccols : colors of the columns (alternatively)
	mz=totalmax(z);
	z=z/abs(mz);
	n=rows(z); m=cols(z);
	M=zeros(n*m,6);
	d=1.5/max(n,m);
	x=-n*d:d:(n-1)*d; x=x+d/2;
	y=-m*d:d:(m-1)*d; y=y+d/2;
	k=1;
	c=zeros(1,n*m);
	for i=1 to n;
		for j=1 to m;
			if z[i,j]>=0 then
				M[k]=[y[2*j-1],y[2*j],x[2*i-1],x[2*i],0,z[i,j]];
			else
				M[k]=[y[2*j-1],y[2*j],x[2*i-1],x[2*i],z[i,j],0];
			endif;
			if ccols!=none then c[k]=ccols[j]; endif;
			if crows!=none then c[k]=crows[i]; endif;
			k=k+1;
		end;
	end;
	if crows==none and ccols==none then c=none; endif;
	if !holding() then clg; endif;
	h=holding(1);
	oldcenter=center([0,0,-0.2]);
	oldview=view();
	viewangle(angle);
	viewheight(height);
	zoom(zoom);
	viewdistance(distance);
	plotcubes(M,c);
	th=textheight();
	signum=-sign(cos(angle));
	if scols!=none then
		loop 1 to m
			{col,row}=project((y[2*#-1]+y[2*#])/2,signum*(n*d+d/2),0);
			ctext(""+scols[#],col,row-th/2);
		end
	endif;
	signum=sign(sin(angle));
	if srows!=none then
		loop 1 to n
			{col,row}=project(signum*(m*d+d/2),(x[2*#-1]+x[2*#])/2,0);
			ctext(""+srows[#],col,row-th/2);
		end
	endif;
	view(oldview);
	center(oldcenter);
	holding(h);
endfunction

function mosaicplot (z: real, srows=none, scols=none,
	textcolor=red, color=green, style="O#")
## Moasaic plot of the data in z.
##
## z : matrix with values
## srows, scols : label strings for the rows and columns (string
## vectors)
## color : a color or a vector of colors for the columns of the plot.
## style : a style or a vector of styles.
##
## For an example see the introduction to statistics.
	if !holding() then clg; endif;
	h=holding(1);
	setplot(0,1,0,1);
	n=rows(z); m=cols(z);
	sc=sum(z')';
	cc=zeros(1,n);
	space=0.1/max(n,m);
	xtotal=1-m*space; ytotal=1-n*space;
	dx=1/m;
	for j=1 to m;
		y=0;
		for i=1 to n;
			dy=z[i,j]/sc[j]*ytotal;
			bc=barcolor(color{i});
			st=barstyle(style{i});
			plotbar((j-1)*dx,y,dx-space,dy);
			barstyle(st);
			barcolor(bc);
			if j==1 then cc[i]=y+dy/2; endif;
			y=y+dy+space;
		end;
	end;
	if srows!=none then
		w=window();
		ht=textheight();
		loop 1 to cols(z);
			ct=textcolor(textcolor);
			{col,row}=toscreen((#-0.5)/m,-0.05);
			_ctext(""+srows[#],[col,row-ht/2]);
			textcolor(ct);
		end;
	endif;
	if scols!=none then
		w=window();
		ht=textheight();
		loop 1 to rows(z);
			ct=textcolor(textcolor);
			{col,row}=toscreen(-0.04,cc[#]);
			_rtext(""+scols[#],[col,row-ht/2]);
			textcolor(ct);
		end;
	endif;
	holding(h);
endfunction

function scatterplots (M:real, lab=none,
	ticks=1, grid=4, style="..")
## Plot all rows of M against all rows of M.
##
## The labels are shown in the diagonal of the plot.
##
## lab : labels for the rows.
	n=rows(M);
	figure(n,n);
	for i=1 to n;
		for j=1 to n;
			figure((i-1)*n+j);
			if i==j then
				if lab!=none then
					setplot(-1,1,-1,1);
					{col,row}=toscreen(0,0);
					ctext(lab[i],col,row-textheight()/2); 
				endif;
			else
				plot2d(M[i],M[j],>points, ..
					style=style,grid=grid,ticks=ticks);
			endif;
		end;
	end;
	figure(0);
endfunction

function statplot (x, y=none, plottype="b", 
	pstyle="[]w", lstyle="-", fstyle="O#",
	xl="", yl="")
## Plots x against y.
##
## This is a simple form of using plot2d with point, line or bar
## options.
##
## The available plotplottypes are
##
## 'p' : point plot
## 'l' : line plot
## 'b' : both
## 'h' : histgram plot
## 's' : surface plot
##
## pstyle, lstyle, fstyle : Styles for the points, lines and bars
##
## See: style
	if !holding(); then clg; endif;
	h=holding(1);
	if y==none then y=x; x=1:rows(y); endif;
	if plottype=="p" then
		plot2d(x,y,>points,style=pstyle);
	elseif plottype=="l" then
		plot2d(x,y,style=lstyle);
	elseif plottype=="b" then
		plot2d(x,y,style=lstyle,xl=xl,yl=yl);
		plot2d(x,y,>points,>add,style=pstyle);
	elseif plottype=="h" then
		plot2d(makepulse(x,y),>bar,style=fillstyle,style=fstyle);
	elseif plottype=="s" then
		plot2d(x,y,>bar,style=fstyle);
	else
		error("Unknown plottype (use p,l,b,h,s).");
	endif;
	holding(h);
endfunction

// * Data Tables

function writetable (x:real, 
	fixed:integer=0, wc:index=10, dc:nonnegative integer=2, 
	labc=none, labr=none, wlabr=none, lablength=1, 
	NA=".", NAval=NAN,
	ctok:index=none, tok:string vector=none,
	file=none)
## Write a table x of statical values
##
## Write a table with labels for the columns and rows and formats for
## each row. A typical table looks like this
##
##         A     B     C
##  G   1.02     2     f
##  H   3.05     5     m 
##
## Each number in the table can be translated into a token string.
## This translation can be set with a global variable tok (string
## vector) wich applies to all columns with indices in ctok (index
## vctor). Or it can be set in each column with an assigned variable
## ctok? (string vector), where ? is the number of the column.
##
## See the intreoduction to statistics for an example.
##
## wc : default width for all columns or vector of widths.
## dc : default decimal precission for all columns or vector of
## precissions.
## fixed : use fixed number of decimal digits (boolean or vector of
## boolean).
##
## labc : labels for the columns (string or real vector)
## lablength : increase the width of the columns, if labels are wider.
## labr : labels for the rows (string or real vector)
##
## NA, NAval : Token string and value to represent "Not Available". By
## default "." and NAN is used.
##
## See: readtable
	if file then open(file,"w"); endif;
	if wlabr==none then wlabr=wc; endif;
	if typeof(wc)==0 then wc=wc*ones(1,cols(x)); endif;
	if labc!=none and lablength then
		for j=1 to cols(x)
			wc[j]=max(wc[j],strlen(""+labc[j])+1);
		end;
	endif;
	if labc!=none then
		if labr!=none then write(printstr(" ",wlabr)); endif;
		for j=1 to length(labc)
			w=wc{j};
			write(printstr(labc[j],w));
		end;
		writeln;
	endif;
	for i=1 to rows(x)
		if labr!=none then write(printstr(""+labr[i],wlabr)); endif;
		for j=1 to cols(x)
			w=wc{j}; d=dc{j};
			if x[i,j]==NAval then
				s=printstr(NA,w);
			elseif isvar("tok"+j) then
				t=evaluate("tok"+j);
				if x[i,j]<1 or x[i,j]>length(t) then
					error("Unknown token number "+x[i,j]);
				endif;
				s=printstr(t[x[i,j]],w);
			elseif ctok!=none and any(ctok==j) then
				if x[i,j]<1 or x[i,j]>length(tok) then
					error("Unknown token number "+x[i,j]);
				endif;
				s=printstr(tok[x[i,j]],w);
			else
				t=round(x[i,j],d); 
				if t~=0 then t=0; endif;
				if (fixed{j}) 
					s=printf("%"+w+"."+d+"f",t);
				else
					s=printf("%"+w+"g",t);
				endif
			endif;
			write(s);
		end
		writeln();
	end;
	if file then close(); endif;
endfunction

function printfile (filename:string, lines:index=100)
## Print the first lines of a file
##
## See: open, close
	open(filename,"r");
	loop 1 to lines
		if (eof()) then break; endif;
		getline(),
	end
	close();
endfunction

function readtable (filename:string, clabs=1, rlabs=0,
	NA=".", NAval=NAN,
	ctok:index=none, tokens=[none])
## Read a table from a file.
##
## The table can have a header line (clabs=1) and row labels
## (rlabs=1). The entries of the table can be numbers (with decimal
## dots) or strings. In case of strings, these tokens are translated
## to unique numbers. The tranlation can either be set for each
## column separately in string vectors with names tok1, tok2 etc.,
## or for the complete table in the tokens parameter. 
##
## The the tokens are collected from the columns with indices in the
## ctok vector. If a column has a tok? parameter, tokens are not
## collected automatically from that column.
##
## "Not Available" can be represented by a special string. The
## default is ".". In the numerical table, it is represented by
## default as NAN. If you do not like this, simply let NAN be
## represented by any other string and translate ti into a numerical
## token.
##
## See the introduction for statistics for an example.
##
## clabs : The table has a line with headings
## rlabs : Each line has a heading label.
## NA, MAval : Sets the string and the returned value for NA (not
## available).
## ctok : Indices of the columns, where tokens are to be collected.
## tok1=..., tok2=... : Individual string arays for columns.
##
## Returns {table, heading string, token strings, rowlabel strings}
##
## See: writetable
	open(filename,"r");
	if clabs then heading=strtokens(getline()); 
	else heading=[none]; 
	endif;
	rowlabels=[none];
	M=[];
	repeat
		until eof();
		line=getline();
		if strlen(line)<1 then continue; endif;
		line=strtokens(line);
		v=[];
		start=0;
		if rlabs then 
			rowlabels=rowlabels|line[1]; start=1;
		endif; 
		for j=1 to length(line)-start;
			if line[start+j]==NA then 
				v=v|NAval;
			else
				if isvar("tok"+j) then
					t=evaluate("tok"+j);
					k=indexof(t,line[start+j]);
					v=v|k;
				elseif ctok!=none and any(ctok==j) then
					k=indexof(tokens,line[start+j]);
					if k==0 then
						tokens=tokens|line[start+j];
						v=v|length(tokens);
					else
						v=v|k;
					endif;
				else
					x=evaluate(line[start+j]);
					v=v|x;
				endif;
			endif;
		end;
		M=M_v;
	end;
	close();
	return {M,heading,tokens,rowlabels};
endfunction

function tablecol (M:real, j:nonnegative vector, NAval=NAN)
## The non-NAN values in the columns j of the table M.
##
## To access a table column, you could simply use M[,j], where j is a
## row vector of indices or a single index. But this function skips
## any NAN values in any of the columns j. It returns the columns
## as rows (transposed) and the indices of the rows.
##
## NANval : The value that should be treated as "Not Available"
##
## Returns {colums as rows, indices of non-NAN rows}
	c=M[,j];
	i=nonzeros(!(sum(c==NAval)>0)');
	return {c[i]',i};
endfunction

function selectrows (M:real, j:index, v:real vector, NAval=NAN)
## Select the rows indices i with M[i,j] in v and not-NAN.
	{c,i0}=tablecol(M,j,NAval);
	i=indexof(v,c);
	return i0[nonzeros(i)];
endfunction

function sortedrows (M:real, j:nonnegative integer vector)
## Index of rows for sorted table with respect to columns in j
##
## The table gets sorted in lexicographic order.
##
## Returns : {sorted table, index of sorted values}
	{B,i}=lexsort(M[,j]);
	return i;
endfunction

// * Functions with Extended Accuracy

function %restrictedgamma (x)
## Special Gamma function, works only for 2x natural
## look into gamma.e for the general gamma function
	if floor(x)~=x; return fac(x-1); endif;
	return prod(0.5:1:x-1)*sqrt(pi());
endfunction

function %tdensity (t,n)
	return (1+t*t/n)^(-(n+1)/2)
endfunction

function map xtdis (x,n)
## Returns the t-distribution with n degrees at x
##
## More accurate than tdis, using a Romberg integration.
##
## See: tdis
	t=abs(x);
	if t~=0 then return 0.5;
	else
		y=0.5+%restrictedgamma((n+1)/2)/(sqrt(pi()*n)* ..
			%restrictedgamma(n/2))*romberg("%tdensity",0,t,10;n);
		if x<0; return 1-y; 
		else return y;
		endif;
	endif;
endfunction

function map xinvtdis (y,n)
## Inverse of xtdis
##
## See: xtdis, invtdis
	x=invtdis(y,n);
	if (y>0.99) then
		return bisect("xtdis",x-1,x+1;n,y=y);
	else
		return secant("xtdis",x-0.1,x+0.1;n,y=y);
	endif;
endfunction

function %chidensity (x, n, factor)
	return x^(n/2-1)*exp(-x/2)*factor;
endfunction

function map xchidis (x:real, n:natural)
## Returns the chi^2 distribution with n degrees
##
## More accurate than chidis, using a Romberg integration.
##
## See: chidis
	if not x~=0 then
		factor=1/(2^(n/2)*%restrictedgamma(n/2));
		return romberg("%chidensity",0,abs(x),10;n,factor);
	else 
		return 0;
	endif;
endfunction

function map xinvchidis (y:nonnegative real, n:natural)
## Returns the inverse of xchidis
##
## See: xchidis, invchidis
	if y~=0 then return 0;
	else 
		if y>1 or y~=1 then return 1/epsilon();
		else;
			x=bisect("chidis",0,100;n,y=y,eps=0.001);
			return secant("xchidis",x*0.9,x*1.1;n,y=y);
		endif;
	endif;
endfunction

function %fdensity (x,n,m,factor)
	return factor*x^(n/2-1)*(m+n*x)^(-(n+m)/2);
endfunction

function map xfdis (x:nonnegative real, n:natural, m:natural)
## Returns the F distribution with n and m degrees at x
##
## More accurate than fdis, using a Romberg integration. The
## integration is sometimes unstable.
##
## See: fdis
	if x~=0 then return 0;
	else
		factor=n^(n/2)*m^(m/2)*%restrictedgamma((n+m)/2)/ ..
			(%restrictedgamma(n/2)*%restrictedgamma(m/2));
		return romberg("%fdensity",0,x,10;n,m,factor);
	endif;
endfunction

function map xinvfdis (y,n,m)
## Returns the inverse of xfdis
##
## See: xfdis
	x=bisect("fdis",0,100;n,m,y=y,eps=0.001);
	return secant("xfdis",x*0.9,x*1.1;n,m,y=y);
endfunction

function %fbe (x:real, a:real, b:real)
	return x^(a-1) * (1-x)^(b-1)
endfunction

function %xbeta (a:real, b:real)
	return gamma(a)*gamma(b)/gamma(a+b);
endfunction

function map xbetai (t:nonnegative real, a:real, b:real)
## Incomplete beta function
##
## A more accurate version of betai, using Romberg integration.
##
## See: betai
	return romberg("%fbe",0,t;a,b)/%xbeta(a,b)
endfunction

function map cfnormaldis (x:real, eps=1e-15, interval=0, limit=3)
## Normal distribution with continued fractions
##
## Using Continued Fractions (by H. Vogel following 26.2.15 Abramowitz-Stegun).
##
## I:=Integral{x,oo} Z(u) du mit Z=exp(-u^2/2)/sqrt(2*pi). 
## eps = relative Schachtelungsbreite der Kettenbrche K
##
## Bei interval<>0 Intervall-Ausgabe.
	x = -x;
	if x == 0 then return 1/2; endif;
	if x < 0 then return 1 - StdNorm(-x, eps, interval); endif,
	xx := x^2; Z := exp(-xx / 2) / sqrt(2*pi);
	if x < limit then
		n=0; M=[x,0;1,1].[3,1;-xx,0]; 
		repeat
			n=n + 1;
			M := M . [4*n+1, 1; 2*n*xx, 0].[4*n+3, 1; -(1 + 2*n)*xx, 0];
			M := M / M[2,1];
			K=M[1] / M[2];
			if abs(K[1] - K[2]) <= eps*K[2] then
				if interval then return 1/2 - Z * ~min(K), max(K)~;
				else return 1/2 - Z * K[1];
				endif;	
			endif;
		end;	
	else
		M := [1, 0; x, 1]; n := 0;
		repeat
			n := n + 1;
			M := M.[x, 1; n, 0];
			M := M / M[1, 1];
			K := M[1] / M[2];
			if abs(K[1] - K[2]) <= eps*K[1] then
				if interval then return Z * ~min(K), max(K)~;
				else return Z * K[1];
				endif;
			endif; 
		end;
	endif;
endfunction;

function xnormaldis (x : real, 
	mean : real = 0, dev : real = 1)
## Probability of being smaller than x with the normal distribution.
##
## Alias to normaldis for |x|<5, else uses cfnormaldis.
##
## See: normaldis
    x=(x-mean)/dev;
    if x>5 return 1-cfnormaldis(-x);
    elseif x<-5 return cfnormaldis(x);
	else return _normaldis(x);
endfunction

function xinvnormaldis (p : real, mean : real = 0, dev : real = 1)
## Probability of being smaller than x with the normal distribution.
##
## Alias to invnormaldis for p>1e-7, else uses bisectoin and
## xnormaldis.
##
## See: invnormaldis
    if p>1-1e-7 or p<=1e-7 then
    	x=_invnormaldis(p);
    	x=bisect("xnormaldis",x-0.1,x+0.1,y=p);
    else x=_invnormaldis(p);
    endif;
	return x*dev+mean;
endfunction
